{ %FAIL }
{ Source provided for Free Pascal Bug Report 1306 }
{ Submitted by "Sergey Kosarevsky" on  2000-12-22 }
{ e-mail: freecommander@aport2000.ru }
UNIT OPERAT;
INTERFACE

USES DOS;

OPERATOR [] (A,B:SEARCHREC) R:BOOLEAN;

IMPLEMENTATION

OPERATOR [] (A,B:SEARCHREC) R:BOOLEAN;
BEGIN
END;

BEGIN
END.
