{ don't edit, this file is generated from x86reg.dat }
OT_NONE,
OT_REG_AL,
OT_REG8,
OT_REG_AX,
OT_REG_EAX,
OT_REG_CL,
OT_REG8,
OT_REG_CX,
OT_REG_ECX,
OT_REG8,
OT_REG8,
OT_REG_DX,
OT_REG32,
OT_REG8,
OT_REG8,
OT_REG16,
OT_REG32,
OT_REG16,
OT_REG32,
OT_REG16,
OT_REG32,
OT_REG16,
OT_REG32,
OT_REG16,
OT_REG32,
OT_REG_DESS,
OT_REG_CS,
OT_REG_DESS,
OT_REG_DESS,
OT_REG_FSGS,
OT_REG_FSGS,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_NONE,
OT_REG_DREG,
OT_REG_DREG,
OT_REG_DREG,
OT_REG_DREG,
OT_REG_DREG,
OT_REG_DREG,
OT_REG_CREG,
OT_REG_CREG,
OT_REG_CREG,
OT_REG_CR4,
OT_REG_TREG,
OT_REG_TREG,
OT_REG_TREG,
OT_REG_TREG,
OT_REG_TREG,
OT_FPU0,
OT_FPUREG,
OT_FPUREG,
OT_FPUREG,
OT_FPUREG,
OT_FPUREG,
OT_FPUREG,
OT_FPUREG,
OT_FPU0,
OT_MMXREG,
OT_MMXREG,
OT_MMXREG,
OT_MMXREG,
OT_MMXREG,
OT_MMXREG,
OT_MMXREG,
OT_MMXREG,
OT_XMMREG,
OT_XMMREG,
OT_XMMREG,
OT_XMMREG,
OT_XMMREG,
OT_XMMREG,
OT_XMMREG,
OT_XMMREG,
OT_YMMREG,
OT_YMMREG,
OT_YMMREG,
OT_YMMREG,
OT_YMMREG,
OT_YMMREG,
OT_YMMREG,
OT_YMMREG
