{ Source provided for Free Pascal Bug Report 3700 }
{ Submitted by "Roman" on  2005-02-26 }
{ e-mail: roman@csltd.com.ua }
uses
  variants;
var
  v : olevariant;
  b : boolean;
begin
  b:=VarIsArray(V);
end.
