{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesPDFKit;
interface

type
  PDFAction = objcclass external;
  PDFActionGoTo = objcclass external;
  PDFActionNamed = objcclass external;
  PDFActionRemoteGoTo = objcclass external;
  PDFActionResetForm = objcclass external;
  PDFActionURL = objcclass external;
  PDFAnnotation = objcclass external;
  PDFAnnotationButtonWidget = objcclass external;
  PDFAnnotationChoiceWidget = objcclass external;
  PDFAnnotationCircle = objcclass external;
  PDFAnnotationFreeText = objcclass external;
  PDFAnnotationInk = objcclass external;
  PDFAnnotationLine = objcclass external;
  PDFAnnotationLink = objcclass external;
  PDFAnnotationMarkup = objcclass external;
  PDFAnnotationPopup = objcclass external;
  PDFAnnotationSquare = objcclass external;
  PDFAnnotationStamp = objcclass external;
  PDFAnnotationText = objcclass external;
  PDFAnnotationTextWidget = objcclass external;
  PDFBorder = objcclass external;
  PDFDestination = objcclass external;
  PDFDocument = objcclass external;
  PDFOutline = objcclass external;
  PDFPage = objcclass external;
  PDFSelection = objcclass external;
  PDFThumbnailView = objcclass external;
  PDFView = objcclass external;

type
  PDFActionGoToPrivateVars = objcclass external;
  PDFActionNamedPrivateVars = objcclass external;
  PDFActionPrivateVars = objcclass external;
  PDFActionRemoteGoToPrivateVars = objcclass external;
  PDFActionResetFormPrivateVars = objcclass external;
  PDFActionURLPrivateVars = objcclass external;
  PDFAnnotationButtonWidgetPrivateVars = objcclass external;
  PDFAnnotationChoiceWidgetPrivateVars = objcclass external;
  PDFAnnotationCirclePrivateVars = objcclass external;
  PDFAnnotationFreeTextPrivateVars = objcclass external;
  PDFAnnotationInkPrivateVars = objcclass external;
  PDFAnnotationLinePrivateVars = objcclass external;
  PDFAnnotationLinkPrivateVars = objcclass external;
  PDFAnnotationMarkupPrivateVars = objcclass external;
  PDFAnnotationPopupPrivateVars = objcclass external;
  PDFAnnotationPrivateVars = objcclass external;
  PDFAnnotationSquarePrivateVars = objcclass external;
  PDFAnnotationStampPrivateVars = objcclass external;
  PDFAnnotationTextPrivateVars = objcclass external;
  PDFAnnotationTextWidgetPrivateVars = objcclass external;
  PDFBorderPrivateVars = objcclass external;
  PDFDestinationPrivateVars = objcclass external;
  PDFDocumentPrivateVars = objcclass external;
  PDFOutlinePrivateVars = objcclass external;
  PDFPagePrivateVars = objcclass external;
  PDFSelectionPrivateVars = objcclass external;
  PDFThumbnailViewPrivateVars = objcclass external;
  PDFViewPrivateVars = objcclass external;

implementation
end.
