{ Parsed from IOBluetooth.framework OBEXBluetooth.h }

{$ifdef TYPES}
type
  IOBluetoothOBEXSessionOpenConnectionCallback = procedure (session: OBEXSessionRef; status: OBEXError; refCon: pointer); cdecl;
{$endif}

