{ %FAIL }

unit tb0257a;

{$mode delphi}
{$modeswitch advancedrecords-}

interface

type
  TTest = record
    procedure Test;
  end;

implementation

{ TTest }

procedure TTest.Test;
begin

end;

end.

