{ %version=1.1}
unit ub0437c;

  interface

    uses
       ub0437b;

    type
      prec = ub0437b.prec;

    function f : prec;

  implementation

    function f : prec;
      begin
      end;

end.
