Unit tw1931;

Interface

Const
        Space = #32;
        Tab = #9;
        Cr = #13;
        Lf = #10;
        CrLf = #13#10;
        Slash = '/';
        Dash = '-';
        Comma = ',';

Implementation

End {-- UChar --}.

