{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesAVFoundation;
interface

type
  AVAsset = objcclass external;
  AVAssetExportSession = objcclass external;
  AVAssetImageGenerator = objcclass external;
  AVAssetReader = objcclass external;
  AVAssetReaderAudioMixOutput = objcclass external;
  AVAssetReaderOutput = objcclass external;
  AVAssetReaderOutputMetadataAdaptor = objcclass external;
  AVAssetReaderSampleReferenceOutput = objcclass external;
  AVAssetReaderTrackOutput = objcclass external;
  AVAssetReaderVideoCompositionOutput = objcclass external;
  AVAssetResourceLoader = objcclass external;
  AVAssetResourceLoadingContentInformationRequest = objcclass external;
  AVAssetResourceLoadingDataRequest = objcclass external;
  AVAssetResourceLoadingRequest = objcclass external;
  AVAssetResourceRenewalRequest = objcclass external;
  AVAssetTrack = objcclass external;
  AVAssetTrackGroup = objcclass external;
  AVAssetTrackSegment = objcclass external;
  AVAssetWriter = objcclass external;
  AVAssetWriterInput = objcclass external;
  AVAssetWriterInputGroup = objcclass external;
  AVAssetWriterInputMetadataAdaptor = objcclass external;
  AVAssetWriterInputPassDescription = objcclass external;
  AVAssetWriterInputPixelBufferAdaptor = objcclass external;
  AVAsynchronousVideoCompositionRequest = objcclass external;
  AVAudioBuffer = objcclass external;
  AVAudioChannelLayout = objcclass external;
  AVAudioEngine = objcclass external;
  AVAudioEnvironmentDistanceAttenuationParameters = objcclass external;
  AVAudioEnvironmentNode = objcclass external;
  AVAudioEnvironmentReverbParameters = objcclass external;
  AVAudioFile = objcclass external;
  AVAudioFormat = objcclass external;
  AVAudioIONode = objcclass external;
  AVAudioInputNode = objcclass external;
  AVAudioMix = objcclass external;
  AVAudioMixInputParameters = objcclass external;
  AVAudioMixerNode = objcclass external;
  AVAudioNode = objcclass external;
  AVAudioOutputNode = objcclass external;
  AVAudioPCMBuffer = objcclass external;
  AVAudioPlayer = objcclass external;
  AVAudioPlayerNode = objcclass external;
  AVAudioRecorder = objcclass external;
  AVAudioTime = objcclass external;
  AVAudioUnit = objcclass external;
  AVAudioUnitComponent = objcclass external;
  AVAudioUnitComponentManager = objcclass external;
  AVAudioUnitDelay = objcclass external;
  AVAudioUnitDistortion = objcclass external;
  AVAudioUnitEQ = objcclass external;
  AVAudioUnitEQFilterParameters = objcclass external;
  AVAudioUnitEffect = objcclass external;
  AVAudioUnitGenerator = objcclass external;
  AVAudioUnitMIDIInstrument = objcclass external;
  AVAudioUnitReverb = objcclass external;
  AVAudioUnitSampler = objcclass external;
  AVAudioUnitTimeEffect = objcclass external;
  AVAudioUnitTimePitch = objcclass external;
  AVAudioUnitVarispeed = objcclass external;
  AVCaptureAudioChannel = objcclass external;
  AVCaptureAudioDataOutput = objcclass external;
  AVCaptureAudioFileOutput = objcclass external;
  AVCaptureAudioPreviewOutput = objcclass external;
  AVCaptureConnection = objcclass external;
  AVCaptureDevice = objcclass external;
  AVCaptureDeviceFormat = objcclass external;
  AVCaptureDeviceInput = objcclass external;
  AVCaptureDeviceInputSource = objcclass external;
  AVCaptureFileOutput = objcclass external;
  AVCaptureInput = objcclass external;
  AVCaptureInputPort = objcclass external;
  AVCaptureMetadataOutput = objcclass external;
  AVCaptureMovieFileOutput = objcclass external;
  AVCaptureOutput = objcclass external;
  AVCaptureScreenInput = objcclass external;
  AVCaptureSession = objcclass external;
  AVCaptureStillImageOutput = objcclass external;
  AVCaptureVideoDataOutput = objcclass external;
  AVCaptureVideoPreviewLayer = objcclass external;
  AVComposition = objcclass external;
  AVCompositionTrack = objcclass external;
  AVCompositionTrackSegment = objcclass external;
  AVFragmentedMovie = objcclass external;
  AVFragmentedMovieMinder = objcclass external;
  AVFragmentedMovieTrack = objcclass external;
  AVFrameRateRange = objcclass external;
  AVMIDIPlayer = objcclass external;
  AVMediaSelectionGroup = objcclass external;
  AVMediaSelectionOption = objcclass external;
  AVMetadataFaceObject = objcclass external;
  AVMetadataItem = objcclass external;
  AVMetadataItemFilter = objcclass external;
  AVMetadataMachineReadableCodeObject = objcclass external;
  AVMetadataObject = objcclass external;
  AVMovie = objcclass external;
  AVMovieTrack = objcclass external;
  AVMutableAudioMix = objcclass external;
  AVMutableAudioMixInputParameters = objcclass external;
  AVMutableComposition = objcclass external;
  AVMutableCompositionTrack = objcclass external;
  AVMutableMetadataItem = objcclass external;
  AVMutableTimedMetadataGroup = objcclass external;
  AVMutableVideoComposition = objcclass external;
  AVMutableVideoCompositionInstruction = objcclass external;
  AVMutableVideoCompositionLayerInstruction = objcclass external;
  AVOutputSettingsAssistant = objcclass external;
  AVPlayer = objcclass external;
  AVPlayerItem = objcclass external;
  AVPlayerItemAccessLog = objcclass external;
  AVPlayerItemAccessLogEvent = objcclass external;
  AVPlayerItemErrorLog = objcclass external;
  AVPlayerItemErrorLogEvent = objcclass external;
  AVPlayerItemLegibleOutput = objcclass external;
  AVPlayerItemMetadataOutput = objcclass external;
  AVPlayerItemOutput = objcclass external;
  AVPlayerItemTrack = objcclass external;
  AVPlayerItemVideoOutput = objcclass external;
  AVPlayerLayer = objcclass external;
  AVPlayerMediaSelectionCriteria = objcclass external;
  AVQueuePlayer = objcclass external;
  AVSampleBufferDisplayLayer = objcclass external;
  AVSampleBufferGenerator = objcclass external;
  AVSampleBufferRequest = objcclass external;
  AVSampleCursor = objcclass external;
  AVSynchronizedLayer = objcclass external;
  AVTextStyleRule = objcclass external;
  AVTimedMetadataGroup = objcclass external;
  AVURLAsset = objcclass external;
  AVVideoComposition = objcclass external;
  AVVideoCompositionCoreAnimationTool = objcclass external;
  AVVideoCompositionInstruction = objcclass external;
  AVVideoCompositionLayerInstruction = objcclass external;
  AVVideoCompositionRenderContext = objcclass external;
  AVAssetResourceLoaderDelegateProtocol = objcprotocol external name 'AVAssetResourceLoaderDelegate';
  AVAsynchronousKeyValueLoadingProtocol = objcprotocol external name 'AVAsynchronousKeyValueLoading';
  AVAudio3DMixingProtocol = objcprotocol external name 'AVAudio3DMixing';
  AVAudioMixingProtocol = objcprotocol external name 'AVAudioMixing';
  AVAudioPlayerDelegateProtocol = objcprotocol external name 'AVAudioPlayerDelegate';
  AVAudioRecorderDelegateProtocol = objcprotocol external name 'AVAudioRecorderDelegate';
  AVAudioStereoMixingProtocol = objcprotocol external name 'AVAudioStereoMixing';
  AVCaptureAudioDataOutputSampleBufferDelegateProtocol = objcprotocol external name 'AVCaptureAudioDataOutputSampleBufferDelegate';
  AVCaptureFileOutputDelegateProtocol = objcprotocol external name 'AVCaptureFileOutputDelegate';
  AVCaptureFileOutputRecordingDelegateProtocol = objcprotocol external name 'AVCaptureFileOutputRecordingDelegate';
  AVCaptureMetadataOutputObjectsDelegateProtocol = objcprotocol external name 'AVCaptureMetadataOutputObjectsDelegate';
  AVCaptureVideoDataOutputSampleBufferDelegateProtocol = objcprotocol external name 'AVCaptureVideoDataOutputSampleBufferDelegate';
  AVPlayerItemLegibleOutputPushDelegateProtocol = objcprotocol external name 'AVPlayerItemLegibleOutputPushDelegate';
  AVPlayerItemMetadataOutputPushDelegateProtocol = objcprotocol external name 'AVPlayerItemMetadataOutputPushDelegate';
  AVPlayerItemOutputPullDelegateProtocol = objcprotocol external name 'AVPlayerItemOutputPullDelegate';
  AVPlayerItemOutputPushDelegateProtocol = objcprotocol external name 'AVPlayerItemOutputPushDelegate';
  AVVideoCompositingProtocol = objcprotocol external name 'AVVideoCompositing';
  AVVideoCompositionInstructionProtocol = objcprotocol external name 'AVVideoCompositionInstruction';
  AVVideoCompositionValidationHandlingProtocol = objcprotocol external name 'AVVideoCompositionValidationHandling';

type
  AVAssetExportSessionInternal = objcclass external;
  AVAssetImageGeneratorInternal = objcclass external;
  AVAssetInternal = objcclass external;
  AVAssetReaderAudioMixOutputInternal = objcclass external;
  AVAssetReaderInternal = objcclass external;
  AVAssetReaderOutputInternal = objcclass external;
  AVAssetReaderOutputMetadataAdaptorInternal = objcclass external;
  AVAssetReaderSampleReferenceOutputInternal = objcclass external;
  AVAssetReaderTrackOutputInternal = objcclass external;
  AVAssetReaderVideoCompositionOutputInternal = objcclass external;
  AVAssetResourceLoaderInternal = objcclass external;
  AVAssetResourceLoadingContentInformationRequestInternal = objcclass external;
  AVAssetResourceLoadingDataRequestInternal = objcclass external;
  AVAssetResourceLoadingRequestInternal = objcclass external;
  AVAssetTrackGroupInternal = objcclass external;
  AVAssetTrackInternal = objcclass external;
  AVAssetWriterInputGroupInternal = objcclass external;
  AVAssetWriterInputInternal = objcclass external;
  AVAssetWriterInputMetadataAdaptorInternal = objcclass external;
  AVAssetWriterInputPassDescriptionInternal = objcclass external;
  AVAssetWriterInputPixelBufferAdaptorInternal = objcclass external;
  AVAssetWriterInternal = objcclass external;
  AVAsynchronousVideoCompositionRequestInternal = objcclass external;
  AVAudioMixInputParametersInternal = objcclass external;
  AVAudioMixInternal = objcclass external;
  AVAudioSessionChannelDescription = objcclass external;
  AVCaptureAudioChannelInternal = objcclass external;
  AVCaptureAudioDataOutputInternal = objcclass external;
  AVCaptureAudioFileOutputInternal = objcclass external;
  AVCaptureAudioPreviewOutputInternal = objcclass external;
  AVCaptureConnectionInternal = objcclass external;
  AVCaptureDeviceFormatInternal = objcclass external;
  AVCaptureDeviceInputInternal = objcclass external;
  AVCaptureDeviceInputSourceInternal = objcclass external;
  AVCaptureDeviceInternal = objcclass external;
  AVCaptureFileOutputInternal = objcclass external;
  AVCaptureInputInternal = objcclass external;
  AVCaptureInputPortInternal = objcclass external;
  AVCaptureMetadataOutputInternal = objcclass external;
  AVCaptureMovieFileOutputInternal = objcclass external;
  AVCaptureOutputInternal = objcclass external;
  AVCaptureScreenInputInternal = objcclass external;
  AVCaptureSessionInternal = objcclass external;
  AVCaptureStillImageOutputInternal = objcclass external;
  AVCaptureVideoDataOutputInternal = objcclass external;
  AVCaptureVideoPreviewLayerInternal = objcclass external;
  AVCompositionInternal = objcclass external;
  AVCompositionTrackInternal = objcclass external;
  AVCompositionTrackSegmentInternal = objcclass external;
  AVFragmentedMovieInternal = objcclass external;
  AVFragmentedMovieMinderInternal = objcclass external;
  AVFragmentedMovieTrackInternal = objcclass external;
  AVFrameRateRangeInternal = objcclass external;
  AVMediaDataStorage = objcclass external;
  AVMediaSelectionGroupInternal = objcclass external;
  AVMediaSelectionOptionInternal = objcclass external;
  AVMetadataFaceObjectInternal = objcclass external;
  AVMetadataItemFilterInternal = objcclass external;
  AVMetadataItemInternal = objcclass external;
  AVMetadataMachineReadableCodeObjectInternal = objcclass external;
  AVMetadataObjectInternal = objcclass external;
  AVMovieInternal = objcclass external;
  AVMovieTrackInternal = objcclass external;
  AVMutableAudioMixInputParametersInternal = objcclass external;
  AVMutableAudioMixInternal = objcclass external;
  AVMutableCompositionInternal = objcclass external;
  AVMutableCompositionTrackInternal = objcclass external;
  AVMutableMetadataItemInternal = objcclass external;
  AVMutableTimedMetadataGroupInternal = objcclass external;
  AVMutableVideoCompositionInstructionInternal = objcclass external;
  AVMutableVideoCompositionInternal = objcclass external;
  AVMutableVideoCompositionLayerInstructionInternal = objcclass external;
  AVOutputSettingsAssistantInternal = objcclass external;
  AVPlayerImageProvider = objcclass external;
  AVPlayerInternal = objcclass external;
  AVPlayerItemAccessLogEventInternal = objcclass external;
  AVPlayerItemAccessLogInternal = objcclass external;
  AVPlayerItemErrorLogEventInternal = objcclass external;
  AVPlayerItemErrorLogInternal = objcclass external;
  AVPlayerItemInternal = objcclass external;
  AVPlayerItemLegibleOutputInternal = objcclass external;
  AVPlayerItemMetadataOutputInternal = objcclass external;
  AVPlayerItemOutputInternal = objcclass external;
  AVPlayerItemTrackInternal = objcclass external;
  AVPlayerItemVideoOutputInternal = objcclass external;
  AVPlayerLayerInternal = objcclass external;
  AVPlayerMediaSelectionCriteriaInternal = objcclass external;
  AVQueuePlayerInternal = objcclass external;
  AVSampleBufferDisplayLayerInternal = objcclass external;
  AVSampleBufferGeneratorInternal = objcclass external;
  AVSampleBufferRequestInternal = objcclass external;
  AVSampleCursorInternal = objcclass external;
  AVSynchronizedLayerInternal = objcclass external;
  AVTextStyleRuleInternal = objcclass external;
  AVTimedMetadataGroupInternal = objcclass external;
  AVURLAssetInternal = objcclass external;
  AVVideoCompositionCoreAnimationToolInternal = objcclass external;
  AVVideoCompositionInstructionInternal = objcclass external;
  AVVideoCompositionInternal = objcclass external;
  AVVideoCompositionLayerInstructionInternal = objcclass external;
  AVVideoCompositionRenderContextInternal = objcclass external;
  CALayer = objcclass external;
  NSData = objcclass external;
  NSDictionary = objcclass external;
  NSError = objcclass external;
  NSURL = objcclass external;
  NSURLAuthenticationChallenge = objcclass external;

implementation
end.
