{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesScreenSaver;
interface

type
  ScreenSaverDefaults = objcclass external;
  ScreenSaverView = objcclass external;

implementation
end.
