{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesCoreAudioKit;
interface

type
  AUGenericView = objcclass external;
  AUPannerView = objcclass external;
  AUCustomViewPersistentDataProtocol = objcprotocol external name 'AUCustomViewPersistentData';

type
  NSDictionary = objcclass external;

implementation
end.
