<?xml version="1.0"?>
<CONFIG>
  <ProjectOptions>
    <PathDelim Value="/"/>
    <Version Value="5"/>
    <General>
      <MainUnit Value="0"/>
      <IconPath Value="./"/>
      <TargetFileExt Value=""/>
      <ActiveEditorIndexAtStart Value="4"/>
    </General>
    <LazDoc Paths=""/>
    <Units Count="6">
      <Unit0>
        <CursorPos X="1" Y="48"/>
        <EditorIndex Value="0"/>
        <Filename Value="wanddemo.lpr"/>
        <IsPartOfProject Value="True"/>
        <Loaded Value="True"/>
        <TopLine Value="15"/>
        <UnitName Value="wanddemo"/>
        <UsageCount Value="23"/>
      </Unit0>
      <Unit1>
        <CursorPos X="2" Y="48"/>
        <Filename Value="wanddemo.dpr"/>
        <TopLine Value="15"/>
        <UnitName Value="wanddemo"/>
        <UsageCount Value="20"/>
      </Unit1>
      <Unit2>
        <CursorPos X="3" Y="26"/>
        <EditorIndex Value="2"/>
        <Filename Value="magick_type.inc"/>
        <Loaded Value="True"/>
        <TopLine Value="9"/>
        <UsageCount Value="11"/>
      </Unit2>
      <Unit3>
        <CursorPos X="1" Y="62"/>
        <EditorIndex Value="1"/>
        <Filename Value="magick_wand.pas"/>
        <Loaded Value="True"/>
        <TopLine Value="33"/>
        <UnitName Value="magick_wand"/>
        <UsageCount Value="11"/>
      </Unit3>
      <Unit4>
        <CursorPos X="25" Y="24"/>
        <EditorIndex Value="4"/>
        <Filename Value="magick_image.inc"/>
        <Loaded Value="True"/>
        <TopLine Value="18"/>
        <UsageCount Value="11"/>
      </Unit4>
      <Unit5>
        <CursorPos X="70" Y="80"/>
        <EditorIndex Value="3"/>
        <Filename Value="drawing_wand.inc"/>
        <Loaded Value="True"/>
        <TopLine Value="113"/>
        <UsageCount Value="11"/>
      </Unit5>
    </Units>
    <PublishOptions>
      <Version Value="2"/>
      <IgnoreBinaries Value="False"/>
      <IncludeFileFilter Value="*.(pas|pp|inc|lfm|lpr|lrs|lpi|lpk|sh|xml)"/>
      <ExcludeFileFilter Value="*.(bak|ppu|ppw|o|so);*~;backup"/>
    </PublishOptions>
    <RunParams>
      <local>
        <FormatVersion Value="1"/>
        <LaunchingApplication PathPlusParams="/usr/X11R6/bin/xterm -T 'Lazarus Run Output' -e $(LazarusDir)/tools/runwait.sh $(TargetCmdLine)"/>
      </local>
    </RunParams>
    <JumpHistory Count="3" HistoryIndex="2">
      <Position1>
        <Filename Value="wanddemo.lpr"/>
        <Caret Line="28" Column="20" TopLine="14"/>
      </Position1>
      <Position2>
        <Filename Value="wanddemo.lpr"/>
        <Caret Line="26" Column="16" TopLine="14"/>
      </Position2>
      <Position3>
        <Filename Value="wanddemo.lpr"/>
        <Caret Line="28" Column="17" TopLine="15"/>
      </Position3>
    </JumpHistory>
  </ProjectOptions>
  <CompilerOptions>
    <Version Value="5"/>
    <CodeGeneration>
      <Generate Value="Faster"/>
    </CodeGeneration>
    <Other>
      <CompilerPath Value="$(CompPath)"/>
    </Other>
  </CompilerOptions>
  <Debugging>
    <Exceptions Count="2">
      <Item1>
        <Name Value="ECodetoolError"/>
      </Item1>
      <Item2>
        <Name Value="EFOpenError"/>
      </Item2>
    </Exceptions>
  </Debugging>
</CONFIG>
