{$ifdef fpc}
{$mode delphi}
{$else fpc}
{$define FPC_HAS_TYPE_EXTENDED}
{$endif fpc}
{$define bigfile}

{$i ivarol100.pp }
{$i ivarol101.pp }
{$i ivarol102.pp }
{$i ivarol104.pp }
{$i ivarol105.pp }
{$i ivarol106.pp }
{$i ivarol107.pp }
{$i ivarol108.pp }
{$i ivarol109.pp }
{$i ivarol10.pp }
{$i ivarol110.pp }
{$i ivarol111.pp }
{$i ivarol112.pp }
{$i ivarol113.pp }
{$i ivarol115.pp }
{$i ivarol116.pp }
{$i ivarol118.pp }
{$i ivarol119.pp }
{$i ivarol11.pp }
{$i ivarol120.pp }
{$i ivarol121.pp }
{$i ivarol122.pp }
{$i ivarol123.pp }
{$i ivarol125.pp }
{$i ivarol126.pp }
{$i ivarol127.pp }
{$i ivarol128.pp }
{$i ivarol129.pp }
{$i ivarol130.pp }
{$i ivarol131.pp }
{$i ivarol132.pp }
{$i ivarol134.pp }
{$i ivarol135.pp }
{$i ivarol136.pp }
{$i ivarol137.pp }
{$i ivarol138.pp }
{$i ivarol139.pp }
{$i ivarol140.pp }
{$i ivarol141.pp }
{$i ivarol142.pp }
{$i ivarol143.pp }
{$i ivarol144.pp }
{$i ivarol145.pp }
{$i ivarol146.pp }
{$i ivarol147.pp }
{$i ivarol148.pp }
{$i ivarol149.pp }
{$i ivarol150.pp }
{$i ivarol151.pp }
{$i ivarol152.pp }
{$i ivarol153.pp }
{$i ivarol155.pp }
{$i ivarol156.pp }
{$i ivarol158.pp }
{$i ivarol15.pp }
{$i ivarol160.pp }
{$i ivarol161.pp }
{$i ivarol162.pp }
{$i ivarol164.pp }
{$i ivarol165.pp }
{$i ivarol166.pp }
{$i ivarol167.pp }
{$i ivarol169.pp }
{$i ivarol170.pp }
{$i ivarol171.pp }
{$i ivarol17.pp }
{$i ivarol18.pp }
{$i ivarol19.pp }
{$i ivarol21.pp }
{$i ivarol22.pp }
{$i ivarol23.pp }
{$i ivarol24.pp }
{$i ivarol25.pp }
{$i ivarol26.pp }
{$i ivarol27.pp }
{$i ivarol28.pp }
{$i ivarol29.pp }
{$i ivarol2.pp }
{$i ivarol32.pp }
{$i ivarol34.pp }
{$i ivarol35.pp }
{$i ivarol36.pp }
{$i ivarol37.pp }
{$i ivarol38.pp }
{$i ivarol39.pp }
{$i ivarol40.pp }
{$i ivarol41.pp }
{$i ivarol42.pp }
{$i ivarol43.pp }
{$i ivarol44.pp }
{$i ivarol45.pp }
{$i ivarol46.pp }
{$i ivarol47.pp }
{$i ivarol49.pp }
{$i ivarol50.pp }
{$i ivarol52.pp }
{$i ivarol53.pp }
{$i ivarol54.pp }
{$i ivarol55.pp }
{$i ivarol56.pp }
{$i ivarol57.pp }
{$i ivarol58.pp }
{$i ivarol59.pp }
{$i ivarol60.pp }
{$i ivarol63.pp }
{$i ivarol65.pp }
{$i ivarol66.pp }
{$i ivarol68.pp }
{$i ivarol69.pp }
{$i ivarol70.pp }
{$i ivarol71.pp }
{$i ivarol72.pp }
{$i ivarol73.pp }
{$i ivarol74.pp }
{$i ivarol75.pp }
{$i ivarol76.pp }
{$i ivarol77.pp }
{$i ivarol79.pp }
{$i ivarol80.pp }
{$i ivarol81.pp }
{$i ivarol83.pp }
{$i ivarol85.pp }
{$i ivarol86.pp }
{$i ivarol87.pp }
{$i ivarol88.pp }
{$i ivarol89.pp }
{$i ivarol90.pp }
{$i ivarol92.pp }
{$i ivarol93.pp }
{$i ivarol95.pp }
{$i ivarol97.pp }
{$i ivarol98.pp }
{$i ivarol99.pp }

Begin
  dotest100;
  dotest101;
  dotest102;
  dotest104;
  dotest105;
  dotest106;
  dotest107;
  dotest108;
  dotest109;
  dotest10;
  dotest110;
  dotest111;
  dotest112;
  dotest113;
  dotest115;
  dotest116;
  dotest118;
  dotest119;
  dotest11;
  dotest120;
  dotest121;
  dotest122;
  dotest123;
  dotest125;
  dotest126;
  dotest127;
  dotest128;
  dotest129;
  dotest130;
  dotest131;
  dotest132;
  dotest134;
  dotest135;
  dotest136;
  dotest137;
  dotest138;
  dotest139;
  dotest140;
  dotest141;
  dotest142;
  dotest143;
  dotest144;
  dotest145;
  dotest146;
  dotest147;
  dotest148;
  dotest149;
  dotest150;
  dotest151;
  dotest152;
  dotest153;
  dotest155;
  dotest156;
  dotest158;
  dotest15;
  dotest160;
  dotest161;
  dotest162;
  dotest164;
  dotest165;
  dotest166;
  dotest167;
  dotest169;
  dotest170;
  dotest171;
  dotest17;
  dotest18;
  dotest19;
  dotest21;
  dotest22;
  dotest23;
  dotest24;
  dotest25;
  dotest26;
  dotest27;
  dotest28;
  dotest29;
  dotest2;
  dotest32;
  dotest34;
  dotest35;
  dotest36;
  dotest37;
  dotest38;
  dotest39;
  dotest40;
  dotest41;
  dotest42;
  dotest43;
  dotest44;
  dotest45;
  dotest46;
  dotest47;
  dotest49;
  dotest50;
  dotest52;
  dotest53;
  dotest54;
  dotest55;
  dotest56;
  dotest57;
  dotest58;
  dotest59;
  dotest60;
  dotest63;
  dotest65;
  dotest66;
  dotest68;
  dotest69;
  dotest70;
  dotest71;
  dotest72;
  dotest73;
  dotest74;
  dotest75;
  dotest76;
  dotest77;
  dotest79;
  dotest80;
  dotest81;
  dotest83;
  dotest85;
  dotest86;
  dotest87;
  dotest88;
  dotest89;
  dotest90;
  dotest92;
  dotest93;
  dotest95;
  dotest97;
  dotest98;
  dotest99;
End.
