{ %fail }
{ Source provided for Free Pascal Bug Report 2154 }
{ Submitted by "Sergey Kosarevsky" on  2002-10-01 }
{ e-mail: netsurfer@au.ru }
Var glCombinerOutputNV:Procedure(pname,
                                 pname,
                                 pname,
                                 pname,
                                 pname,
                                 pname,
                                 pname:Longint;
                                 pbool,
                                 pbool,
                                 pbool:Boolean);

Begin
End.
