{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesPubSub;
interface

type
  PSAuthor = objcclass external;
  PSClient = objcclass external;
  PSContent = objcclass external;
  PSEnclosure = objcclass external;
  PSEntry = objcclass external;
  PSFeed = objcclass external;
  PSFeedSettings = objcclass external;
  PSLink = objcclass external;

type
  ABPerson = objcclass external;
  NSArray = objcclass external;
  NSData = objcclass external;
  NSDate = objcclass external;
  NSEnumerator = objcclass external;
  NSError = objcclass external;
  NSSet = objcclass external;
  NSString = objcclass external;
  NSTimeZone = objcclass external;
  NSURL = objcclass external;
  NSXMLElement = objcclass external;

implementation
end.
