{ Parsed from QuartzCore.framework CATransformLayer.h }


{$ifdef TYPES}
type
  CATransformLayerPtr = ^CATransformLayer;
{$endif}

{$ifdef CLASSES}

type
  CATransformLayer = objcclass external (CALayer)
  end;
{$endif}

