{$define FRAMEWORK_LOADED_SCENEKIT}
{$include SceneKit.inc}
{$include SCNBase.inc}
{$include SceneKit_simd.inc}
{$include SceneKitTypes.inc}
{$include SCNAnimation.inc}
{$include SCNBoundingVolume.inc}
{$include SCNSceneRenderer.inc}
{$include SCNShadable.inc}
{$include SCNTechnique.inc}
{$include SCNView.inc}
{$include SCNLayer.inc}
{$include SCNRenderer.inc}
{$include SCNMaterialProperty.inc}
{$include SCNScene.inc}
{$include SCNSceneSource.inc}
{$include SCNAction.inc}
{$include SCNNode.inc}
{$include SCNLight.inc}
{$include SCNCamera.inc}
{$include SCNMaterial.inc}
{$include SCNGeometry.inc}
{$include SCNParametricGeometry.inc}
{$include SCNTransaction.inc}
{$include SCNMorpher.inc}
{$include SCNSkinner.inc}
{$include SCNConstraint.inc}
{$include SCNLevelOfDetail.inc}
{$include SCNParticleSystem.inc}
{$include SCNPhysicsBody.inc}
{$include SCNPhysicsField.inc}
{$include SCNPhysicsShape.inc}
{$include SCNPhysicsWorld.inc}
{$include SCNPhysicsContact.inc}
{$include SCNPhysicsBehavior.inc}
{$include SCNJavascript.inc}
