{ Old file: tbs0341.pp }
{  }

procedure IncLimit(var B: Byte; const Limit: Byte; const Incr: Byte);
begin
end;
procedure IncLimit(var B: Longint; const Limit: Longint; const Incr: Longint);
begin
end;

var
  b : byte;
begin
  inclimit(b,128,3);
end.
