{ don't edit, this file is generated from x86ins.dat }
(
(Ch: []),
(Ch: [Ch_MEAX, Ch_RWAuxiliaryFlag, Ch_WCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUZeroFlag, Ch_WUParityFlag]),
(Ch: [Ch_MEAX, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUOverflowFlag, Ch_WUAuxiliaryFlag, Ch_WUCarryFlag]),
(Ch: [Ch_MEAX, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUOverflowFlag, Ch_WUAuxiliaryFlag, Ch_WUCarryFlag]),
(Ch: [Ch_MEAX, Ch_RWAuxiliaryFlag, Ch_WCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUZeroFlag, Ch_WUParityFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_RWCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WParityFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_W0OverflowFlag, Ch_W0CarryFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_WZeroFlag]),
(Ch: [Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1, Ch_WZeroFlag, Ch_WUCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_Wop2, Ch_Rop1, Ch_WZeroFlag, Ch_WUCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_MOp1]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_ROp1, Ch_All]),
(Ch: [Ch_MEAX]),
(Ch: [Ch_MEAX, Ch_WEDX]),
(Ch: [Ch_W0CarryFlag]),
(Ch: [Ch_W0DirFlag]),
(Ch: [Ch_W0IntFlag]),
(Ch: []),
(Ch: [Ch_RWCarryFlag]),
(Ch: [Ch_ROp1, Ch_ROp2, Ch_NoReadIfEqualRegs, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_RWESI, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_RWESI, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_RWESI, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_All, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_All, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_All, Ch_WZeroFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_MEAX, Ch_WEDX]),
(Ch: [Ch_MEAX]),
(Ch: [Ch_MEAX, Ch_RWAuxiliaryFlag, Ch_RWCarryFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUOverflowFlag]),
(Ch: [Ch_MEAX, Ch_RWAuxiliaryFlag, Ch_RWCarryFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUOverflowFlag]),
(Ch: [Ch_Mop1, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WParityFlag]),
(Ch: [Ch_RWEAX, Ch_WEDX, Ch_WUCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUZeroFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_FPU]),
(Ch: [Ch_RWESP]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_Rop1, Ch_FPU]),
(Ch: [Ch_Wop1, Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU, Ch_RCarryFlag]),
(Ch: [Ch_FPU, Ch_RCarryFlag, Ch_RZeroFlag]),
(Ch: [Ch_FPU, Ch_RZeroFlag]),
(Ch: [Ch_FPU, Ch_RCarryFlag]),
(Ch: [Ch_FPU, Ch_RCarryFlag, Ch_RZeroFlag]),
(Ch: [Ch_FPU, Ch_RZeroFlag]),
(Ch: [Ch_FPU, Ch_RParityFlag]),
(Ch: [Ch_FPU, Ch_RParityFlag]),
(Ch: [Ch_FPU]),
(Ch: [Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_FPU, Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_All]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_FPU, Ch_ROp1]),
(Ch: [Ch_Rop1, Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_ROp1, Ch_FPU]),
(Ch: [Ch_ROp1, Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1, Ch_FPU]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_ROp1, Ch_FPU]),
(Ch: [Ch_ROp1, Ch_FPU]),
(Ch: [Ch_ROp1, Ch_FPU]),
(Ch: [Ch_ROp1, Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: []),
(Ch: [Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_FPU, Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: [Ch_FPU]),
(Ch: []),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_RWEAX, Ch_WEDX, Ch_WUCarryFlag, Ch_WUOverflowFlag, Ch_WUSignFlag, Ch_WUZeroFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_RWEAX, Ch_WEDX, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WUSignFlag, Ch_WUZeroFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Mop1, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WParityFlag]),
(Ch: [Ch_WMemEDI, Ch_RWEDI, Ch_REDX, Ch_RDirFlag]),
(Ch: [Ch_WMemEDI, Ch_RWEDI, Ch_REDX, Ch_RDirFlag]),
(Ch: [Ch_WMemEDI, Ch_RWEDI, Ch_REDX, Ch_RDirFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: []),
(Ch: []),
(Ch: [Ch_All, Ch_ROverflowFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_RECX]),
(Ch: [Ch_RECX]),
(Ch: [Ch_ROp1]),
(Ch: [Ch_WEAX, Ch_RFlags]),
(Ch: [Ch_Wop2, Ch_WZeroFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_RWESP, Ch_WEBP]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: []),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: []),
(Ch: [Ch_ROp1]),
(Ch: []),
(Ch: []),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: []),
(Ch: [Ch_WEAX, Ch_RWESI, Ch_RDirFlag]),
(Ch: [Ch_WEAX, Ch_RWESI, Ch_RDirFlag]),
(Ch: [Ch_WEAX, Ch_RWESI, Ch_RDirFlag]),
(Ch: [Ch_RWECX]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_Wop2, Ch_WZeroFlag]),
(Ch: [Ch_Wop2, Ch_ROP1]),
(Ch: []),
(Ch: []),
(Ch: [Ch_Wop2, Ch_Rop1, Ch_NoReadIfEqualRegs]),
(Ch: [Ch_Rop1, Ch_Wop2]),
(Ch: [Ch_Rop1, Ch_Wop2]),
(Ch: [Ch_RWESI, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_RWESI, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_RWESI, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_RWEAX, Ch_WEDX, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WUSignFlag, Ch_WUZeroFlag, Ch_WUAuxiliaryFlag, Ch_WUParityFlag]),
(Ch: []),
(Ch: [Ch_Mop1, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WParityFlag]),
(Ch: []),
(Ch: [Ch_Mop1]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_W0OverflowFlag, Ch_W0CarryFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_Rop1, Ch_Rop2]),
(Ch: [Ch_All, Ch_RDirFlag]),
(Ch: [Ch_All, Ch_RDirFlag]),
(Ch: [Ch_All, Ch_RDirFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop1, Ch_RWESP]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_RWESP, Ch_WFlags]),
(Ch: [Ch_RWESP, Ch_WFlags]),
(Ch: [Ch_RWESP, Ch_WFLAGS]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_RWESP]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_RWESP, Ch_RFlags]),
(Ch: [Ch_RWESP, Ch_RFlags]),
(Ch: [Ch_RWESP, Ch_RFLAGS]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_RWCarryFlag, Ch_WOverflowFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_RWCarryFlag, Ch_WOverflowFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_WEAX, Ch_WEDX]),
(Ch: [Ch_WEAX, Ch_WEDX]),
(Ch: [Ch_WEAX, Ch_WEDX]),
(Ch: [Ch_RWECX]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_RWECX, Ch_RZeroFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WOverflowFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WOverflowFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_REAX, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WParityFlag, Ch_WCarryFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_WEAX, Ch_RFLAGS]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_NoReadIfEqualRegs, Ch_RWCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WParityFlag]),
(Ch: [Ch_REAX, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_REAX, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_REAX, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: []),
(Ch: []),
(Ch: []),
(Ch: []),
(Ch: []),
(Ch: []),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_MOp3, Ch_Rop2, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_MOp3, Ch_Rop2, Ch_WCarryFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_W1CarryFlag]),
(Ch: [Ch_W1DirFlag]),
(Ch: [Ch_W1IntFlag]),
(Ch: [Ch_REAX, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_REAX, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_REAX, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_Wop1]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_NoReadIfEqualRegs, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_W0OverflowFlag, Ch_W0CarryFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_WZeroFlag]),
(Ch: [Ch_WZeroFlag]),
(Ch: []),
(Ch: []),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_RWop1, Ch_RWop2, Ch_NoReadIfEqualRegs]),
(Ch: [Ch_WEAX, Ch_REBX]),
(Ch: [Ch_WEAX, Ch_REBX]),
(Ch: [Ch_Mop2, Ch_Rop1, Ch_NoReadIfEqualRegs, Ch_W0OverflowFlag, Ch_W0CarryFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WParityFlag, Ch_WUAuxiliaryFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_ROp1, Ch_RWOp2, Ch_RFLAGScc]),
(Ch: [Ch_RFLAGScc]),
(Ch: [Ch_RFLAGScc, Ch_WOp1]),
(Ch: [Ch_RWESI, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_RWESI, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_REAX, Ch_RMemEDI, Ch_RWEDI, Ch_RDirFlag, Ch_WOverflowFlag, Ch_WSignFlag, Ch_WZeroFlag, Ch_WAuxiliaryFlag, Ch_WCarryFlag, Ch_WParityFlag]),
(Ch: [Ch_WEAX, Ch_RWESI, Ch_RDirFlag]),
(Ch: [Ch_REAX, Ch_WMemEDI, Ch_RWEDI, Ch_RDirFlag]),
(Ch: [Ch_WMemEDI, Ch_RWEDI, Ch_REDX, Ch_RDirFlag]),
(Ch: [Ch_All, Ch_RDirFlag]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WFlags]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_ROp1, Ch_WOp2]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Wop2]),
(Ch: [Ch_Rop1, Ch_Wop2]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WFlags]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_ROp1, Ch_WOp2]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_Mop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Wop2]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Mop1, Ch_Rop2]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_WEAX, Ch_WEDX]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WFlags]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WFlags]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Rop3, Ch_Wop4]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Rop3, Ch_Wop4]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_WZeroFlag, Ch_WParityFlag, Ch_WCarryFlag, Ch_W0OverflowFlag, Ch_W0SignFlag, Ch_W0AuxiliaryFlag]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Wop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Wop2, Ch_WFlags, Ch_Rop1]),
(Ch: [Ch_Rop1, Ch_Wop2]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_Rop1, Ch_Rop2, Ch_Wop3]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_All]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: [Ch_Mop3, Ch_Rop2, Ch_Rop1]),
(Ch: []),
(Ch: []),
(Ch: []),
(Ch: [Ch_All]),
(Ch: []),
(Ch: [Ch_WFlags]),
(Ch: [Ch_Wop1, Ch_WFlags]),
(Ch: [Ch_Wop1, Ch_WFlags]),
(Ch: [Ch_WEAX, Ch_WEDX, Ch_RECX]),
(Ch: [Ch_All])
);
