{ %FAIL }

{ Source provided for Free Pascal Bug Report 4541 }
{ Submitted by "Vincent Snijders" on  2005-11-23 }
{ e-mail: vsnijders@quicknet.nl }
program tw4541;

{$mode objfpc}{$H+}

uses
  uw4541;

begin
end.
