{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesSceneKit;
interface

type
  SCNAction = objcclass external;
  SCNAnimationEvent = objcclass external;
  SCNBox = objcclass external;
  SCNCamera = objcclass external;
  SCNCapsule = objcclass external;
  SCNCone = objcclass external;
  SCNConstraint = objcclass external;
  SCNCylinder = objcclass external;
  SCNFloor = objcclass external;
  SCNGeometry = objcclass external;
  SCNGeometryElement = objcclass external;
  SCNGeometrySource = objcclass external;
  SCNHitTestResult = objcclass external;
  SCNIKConstraint = objcclass external;
  SCNLayer = objcclass external;
  SCNLevelOfDetail = objcclass external;
  SCNLight = objcclass external;
  SCNLookAtConstraint = objcclass external;
  SCNMaterial = objcclass external;
  SCNMaterialProperty = objcclass external;
  SCNMorpher = objcclass external;
  SCNNode = objcclass external;
  SCNParticlePropertyController = objcclass external;
  SCNParticleSystem = objcclass external;
  SCNPhysicsBallSocketJoint = objcclass external;
  SCNPhysicsBehavior = objcclass external;
  SCNPhysicsBody = objcclass external;
  SCNPhysicsContact = objcclass external;
  SCNPhysicsField = objcclass external;
  SCNPhysicsHingeJoint = objcclass external;
  SCNPhysicsShape = objcclass external;
  SCNPhysicsSliderJoint = objcclass external;
  SCNPhysicsVehicle = objcclass external;
  SCNPhysicsVehicleWheel = objcclass external;
  SCNPhysicsWorld = objcclass external;
  SCNPlane = objcclass external;
  SCNProgram = objcclass external;
  SCNPyramid = objcclass external;
  SCNRenderer = objcclass external;
  SCNScene = objcclass external;
  SCNSceneSource = objcclass external;
  SCNShape = objcclass external;
  SCNSkinner = objcclass external;
  SCNSphere = objcclass external;
  SCNTechnique = objcclass external;
  SCNText = objcclass external;
  SCNTorus = objcclass external;
  SCNTransaction = objcclass external;
  SCNTransformConstraint = objcclass external;
  SCNTube = objcclass external;
  SCNView = objcclass external;
  SCNActionableProtocol = objcprotocol external name 'SCNActionable';
  SCNAnimatableProtocol = objcprotocol external name 'SCNAnimatable';
  SCNBoundingVolumeProtocol = objcprotocol external name 'SCNBoundingVolume';
  SCNNodeRendererDelegateProtocol = objcprotocol external name 'SCNNodeRendererDelegate';
  SCNPhysicsContactDelegateProtocol = objcprotocol external name 'SCNPhysicsContactDelegate';
  SCNProgramDelegateProtocol = objcprotocol external name 'SCNProgramDelegate';
  SCNSceneExportDelegateProtocol = objcprotocol external name 'SCNSceneExportDelegate';
  SCNSceneRendererDelegateProtocol = objcprotocol external name 'SCNSceneRendererDelegate';
  SCNSceneRendererProtocol = objcprotocol external name 'SCNSceneRenderer';
  SCNShadableProtocol = objcprotocol external name 'SCNShadable';
  SCNTechniqueSupportProtocol = objcprotocol external name 'SCNTechniqueSupport';

type
  SKScene = objcclass external;

implementation
end.
