{ %FAIL }
{ Old file: tbf0086.pp }
{  shows runerror 216                                    OK 0.99.1 (CEC) }


var
 v: word;
 w: shortint;
 z: byte;
 y: integer;

type
 zz: shortint = 255;
Begin
 y:=64000;
 z:=32767;
 w:=64000;
 v:=-1;
end.
