{$define FRAMEWORK_LOADED_ITUNESLIBRARY}
{$include iTunesLibrary.inc}
{$include ITLibDefines.inc}
{$include ITLibAlbum.inc}
{$include ITLibArtist.inc}
{$include ITLibArtwork.inc}
{$include ITLibMediaEntity.inc}
{$include ITLibMediaItem.inc}
{$include ITLibMediaItemVideoInfo.inc}
{$include ITLibPlaylist.inc}
{$include ITLibrary.inc}
