{ %opt=-Sew -vw }

{ Source provided for Free Pascal Bug Report 1948 }
{ Submitted by "Sergey Kosarevsky" on  2002-04-27 }
{ e-mail: netsurfer@au.ru }
Function A:Longint;
Begin
   Exit(0);
End;

Begin
   A;
End.
