{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesAppKit;
interface

type
  NSATSTypesetter = objcclass external;
  NSAccessibilityElement = objcclass external;
  NSActionCell = objcclass external;
  NSAlert = objcclass external;
  NSAnimation = objcclass external;
  NSAnimationContext = objcclass external;
  NSAppearance = objcclass external;
  NSApplication = objcclass external;
  NSArrayController = objcclass external;
  NSBezierPath = objcclass external;
  NSBitmapImageRep = objcclass external;
  NSBox = objcclass external;
  NSBrowser = objcclass external;
  NSBrowserCell = objcclass external;
  NSButton = objcclass external;
  NSButtonCell = objcclass external;
  NSCIImageRep = objcclass external;
  NSCachedImageRep = objcclass external;
  NSCell = objcclass external;
  NSClickGestureRecognizer = objcclass external;
  NSClipView = objcclass external;
  NSCollectionView = objcclass external;
  NSCollectionViewItem = objcclass external;
  NSColor = objcclass external;
  NSColorList = objcclass external;
  NSColorPanel = objcclass external;
  NSColorPicker = objcclass external;
  NSColorSpace = objcclass external;
  NSColorWell = objcclass external;
  NSComboBox = objcclass external;
  NSComboBoxCell = objcclass external;
  NSControl = objcclass external;
  NSController = objcclass external;
  NSCursor = objcclass external;
  NSCustomImageRep = objcclass external;
  NSDatePicker = objcclass external;
  NSDatePickerCell = objcclass external;
  NSDictionaryController = objcclass external;
  NSDockTile = objcclass external;
  NSDocument = objcclass external;
  NSDocumentController = objcclass external;
  NSDraggingImageComponent = objcclass external;
  NSDraggingItem = objcclass external;
  NSDraggingSession = objcclass external;
  NSDrawer = objcclass external;
  NSEPSImageRep = objcclass external;
  NSEvent = objcclass external;
  NSFont = objcclass external;
  NSFontCollection = objcclass external;
  NSFontDescriptor = objcclass external;
  NSFontManager = objcclass external;
  NSFontPanel = objcclass external;
  NSForm = objcclass external;
  NSFormCell = objcclass external;
  NSGestureRecognizer = objcclass external;
  NSGlyphGenerator = objcclass external;
  NSGlyphInfo = objcclass external;
  NSGradient = objcclass external;
  NSGraphicsContext = objcclass external;
  NSHelpManager = objcclass external;
  NSImage = objcclass external;
  NSImageCell = objcclass external;
  NSImageRep = objcclass external;
  NSImageView = objcclass external;
  NSInputManager = objcclass external;
  NSInputServer = objcclass external;
  NSLayoutConstraint = objcclass external;
  NSLayoutManager = objcclass external;
  NSLevelIndicator = objcclass external;
  NSLevelIndicatorCell = objcclass external;
  NSMagnificationGestureRecognizer = objcclass external;
  NSMatrix = objcclass external;
  NSMediaLibraryBrowserController = objcclass external;
  NSMenu = objcclass external;
  NSMenuItem = objcclass external;
  NSMenuItemCell = objcclass external;
  NSMenuView = objcclass external;
  NSMovie = objcclass external;
  NSMovieView = objcclass external;
  NSMutableFontCollection = objcclass external;
  NSMutableParagraphStyle = objcclass external;
  NSNib = objcclass external;
  NSObjectController = objcclass external;
  NSOpenGLContext = objcclass external;
  NSOpenGLLayer = objcclass external;
  NSOpenGLPixelBuffer = objcclass external;
  NSOpenGLPixelFormat = objcclass external;
  NSOpenGLView = objcclass external;
  NSOpenPanel = objcclass external;
  NSOutlineView = objcclass external;
  NSPDFImageRep = objcclass external;
  NSPDFInfo = objcclass external;
  NSPDFPanel = objcclass external;
  NSPICTImageRep = objcclass external;
  NSPageController = objcclass external;
  NSPageLayout = objcclass external;
  NSPanGestureRecognizer = objcclass external;
  NSPanel = objcclass external;
  NSParagraphStyle = objcclass external;
  NSPasteboard = objcclass external;
  NSPasteboardItem = objcclass external;
  NSPathCell = objcclass external;
  NSPathComponentCell = objcclass external;
  NSPathControl = objcclass external;
  NSPathControlItem = objcclass external;
  NSPersistentDocument = objcclass external;
  NSPopUpButton = objcclass external;
  NSPopUpButtonCell = objcclass external;
  NSPopover = objcclass external;
  NSPredicateEditor = objcclass external;
  NSPredicateEditorRowTemplate = objcclass external;
  NSPressGestureRecognizer = objcclass external;
  NSPrintInfo = objcclass external;
  NSPrintOperation = objcclass external;
  NSPrintPanel = objcclass external;
  NSPrinter = objcclass external;
  NSProgressIndicator = objcclass external;
  NSQuickDrawView = objcclass external;
  NSResponder = objcclass external;
  NSRotationGestureRecognizer = objcclass external;
  NSRuleEditor = objcclass external;
  NSRulerMarker = objcclass external;
  NSRulerView = objcclass external;
  NSRunningApplication = objcclass external;
  NSSavePanel = objcclass external;
  NSScreen = objcclass external;
  NSScrollView = objcclass external;
  NSScroller = objcclass external;
  NSSearchField = objcclass external;
  NSSearchFieldCell = objcclass external;
  NSSecureTextField = objcclass external;
  NSSecureTextFieldCell = objcclass external;
  NSSegmentedCell = objcclass external;
  NSSegmentedControl = objcclass external;
  NSShadow = objcclass external;
  NSSharingService = objcclass external;
  NSSharingServicePicker = objcclass external;
  NSSimpleHorizontalTypesetter = objcclass external;
  NSSlider = objcclass external;
  NSSliderCell = objcclass external;
  NSSound = objcclass external;
  NSSpeechRecognizer = objcclass external;
  NSSpeechSynthesizer = objcclass external;
  NSSpellChecker = objcclass external;
  NSSplitView = objcclass external;
  NSSplitViewController = objcclass external;
  NSSplitViewItem = objcclass external;
  NSStackView = objcclass external;
  NSStatusBar = objcclass external;
  NSStatusBarButton = objcclass external;
  NSStatusItem = objcclass external;
  NSStepper = objcclass external;
  NSStepperCell = objcclass external;
  NSStoryboard = objcclass external;
  NSStoryboardSegue = objcclass external;
  NSTabView = objcclass external;
  NSTabViewController = objcclass external;
  NSTabViewItem = objcclass external;
  NSTableCellView = objcclass external;
  NSTableColumn = objcclass external;
  NSTableHeaderCell = objcclass external;
  NSTableHeaderView = objcclass external;
  NSTableRowView = objcclass external;
  NSTableView = objcclass external;
  NSText = objcclass external;
  NSTextAlternatives = objcclass external;
  NSTextAttachment = objcclass external;
  NSTextAttachmentCell = objcclass external;
  NSTextBlock = objcclass external;
  NSTextContainer = objcclass external;
  NSTextField = objcclass external;
  NSTextFieldCell = objcclass external;
  NSTextFinder = objcclass external;
  NSTextInputContext = objcclass external;
  NSTextList = objcclass external;
  NSTextStorage = objcclass external;
  NSTextTab = objcclass external;
  NSTextTable = objcclass external;
  NSTextTableBlock = objcclass external;
  NSTextView = objcclass external;
  NSTitlebarAccessoryViewController = objcclass external;
  NSTokenField = objcclass external;
  NSTokenFieldCell = objcclass external;
  NSToolbar = objcclass external;
  NSToolbarItem = objcclass external;
  NSToolbarItemGroup = objcclass external;
  NSTouch = objcclass external;
  NSTrackingArea = objcclass external;
  NSTreeController = objcclass external;
  NSTreeNode = objcclass external;
  NSTypesetter = objcclass external;
  NSUserDefaultsController = objcclass external;
  NSView = objcclass external;
  NSViewAnimation = objcclass external;
  NSViewController = objcclass external;
  NSVisualEffectView = objcclass external;
  NSWindow = objcclass external;
  NSWindowController = objcclass external;
  NSWorkspace = objcclass external;
  NSAccessibilityButtonProtocol = objcprotocol external name 'NSAccessibilityButton';
  NSAccessibilityCheckBoxProtocol = objcprotocol external name 'NSAccessibilityCheckBox';
  NSAccessibilityContainsTransientUIProtocol = objcprotocol external name 'NSAccessibilityContainsTransientUI';
  NSAccessibilityElementProtocol = objcprotocol external name 'NSAccessibilityElement';
  NSAccessibilityGroupProtocol = objcprotocol external name 'NSAccessibilityGroup';
  NSAccessibilityImageProtocol = objcprotocol external name 'NSAccessibilityImage';
  NSAccessibilityLayoutAreaProtocol = objcprotocol external name 'NSAccessibilityLayoutArea';
  NSAccessibilityLayoutItemProtocol = objcprotocol external name 'NSAccessibilityLayoutItem';
  NSAccessibilityListProtocol = objcprotocol external name 'NSAccessibilityList';
  NSAccessibilityNavigableStaticTextProtocol = objcprotocol external name 'NSAccessibilityNavigableStaticText';
  NSAccessibilityOutlineProtocol = objcprotocol external name 'NSAccessibilityOutline';
  NSAccessibilityProgressIndicatorProtocol = objcprotocol external name 'NSAccessibilityProgressIndicator';
  NSAccessibilityProtocol = objcprotocol external name 'NSAccessibility';
  NSAccessibilityRadioButtonProtocol = objcprotocol external name 'NSAccessibilityRadioButton';
  NSAccessibilityRowProtocol = objcprotocol external name 'NSAccessibilityRow';
  NSAccessibilitySliderProtocol = objcprotocol external name 'NSAccessibilitySlider';
  NSAccessibilityStaticTextProtocol = objcprotocol external name 'NSAccessibilityStaticText';
  NSAccessibilityStepperProtocol = objcprotocol external name 'NSAccessibilityStepper';
  NSAccessibilitySwitchProtocol = objcprotocol external name 'NSAccessibilitySwitch';
  NSAccessibilityTableProtocol = objcprotocol external name 'NSAccessibilityTable';
  NSAlertDelegateProtocol = objcprotocol external name 'NSAlertDelegate';
  NSAnimatablePropertyContainerProtocol = objcprotocol external name 'NSAnimatablePropertyContainer';
  NSAnimationDelegateProtocol = objcprotocol external name 'NSAnimationDelegate';
  NSAppearanceCustomizationProtocol = objcprotocol external name 'NSAppearanceCustomization';
  NSApplicationDelegateProtocol = objcprotocol external name 'NSApplicationDelegate';
  NSBrowserDelegateProtocol = objcprotocol external name 'NSBrowserDelegate';
  NSChangeSpellingProtocol = objcprotocol external name 'NSChangeSpelling';
  NSCollectionViewDelegateProtocol = objcprotocol external name 'NSCollectionViewDelegate';
  NSColorPickingCustomProtocol = objcprotocol external name 'NSColorPickingCustom';
  NSColorPickingDefaultProtocol = objcprotocol external name 'NSColorPickingDefault';
  NSComboBoxCellDataSourceProtocol = objcprotocol external name 'NSComboBoxCellDataSource';
  NSComboBoxDataSourceProtocol = objcprotocol external name 'NSComboBoxDataSource';
  NSComboBoxDelegateProtocol = objcprotocol external name 'NSComboBoxDelegate';
  NSControlTextEditingDelegateProtocol = objcprotocol external name 'NSControlTextEditingDelegate';
  NSDatePickerCellDelegateProtocol = objcprotocol external name 'NSDatePickerCellDelegate';
  NSDockTilePlugInProtocol = objcprotocol external name 'NSDockTilePlugIn';
  NSDraggingDestinationProtocol = objcprotocol external name 'NSDraggingDestination';
  NSDraggingInfoProtocol = objcprotocol external name 'NSDraggingInfo';
  NSDraggingSourceProtocol = objcprotocol external name 'NSDraggingSource';
  NSDrawerDelegateProtocol = objcprotocol external name 'NSDrawerDelegate';
  NSGestureRecognizerDelegateProtocol = objcprotocol external name 'NSGestureRecognizerDelegate';
  NSGlyphStorageProtocol = objcprotocol external name 'NSGlyphStorage';
  NSIgnoreMisspelledWordsProtocol = objcprotocol external name 'NSIgnoreMisspelledWords';
  NSImageDelegateProtocol = objcprotocol external name 'NSImageDelegate';
  NSInputServerMouseTrackerProtocol = objcprotocol external name 'NSInputServerMouseTracker';
  NSInputServiceProviderProtocol = objcprotocol external name 'NSInputServiceProvider';
  NSLayoutManagerDelegateProtocol = objcprotocol external name 'NSLayoutManagerDelegate';
  NSMatrixDelegateProtocol = objcprotocol external name 'NSMatrixDelegate';
  NSMenuDelegateProtocol = objcprotocol external name 'NSMenuDelegate';
  NSOpenSavePanelDelegateProtocol = objcprotocol external name 'NSOpenSavePanelDelegate';
  NSOutlineViewDataSourceProtocol = objcprotocol external name 'NSOutlineViewDataSource';
  NSOutlineViewDelegateProtocol = objcprotocol external name 'NSOutlineViewDelegate';
  NSPageControllerDelegateProtocol = objcprotocol external name 'NSPageControllerDelegate';
  NSPasteboardItemDataProviderProtocol = objcprotocol external name 'NSPasteboardItemDataProvider';
  NSPasteboardReadingProtocol = objcprotocol external name 'NSPasteboardReading';
  NSPasteboardWritingProtocol = objcprotocol external name 'NSPasteboardWriting';
  NSPathCellDelegateProtocol = objcprotocol external name 'NSPathCellDelegate';
  NSPathControlDelegateProtocol = objcprotocol external name 'NSPathControlDelegate';
  NSPopoverDelegateProtocol = objcprotocol external name 'NSPopoverDelegate';
  NSPrintPanelAccessorizingProtocol = objcprotocol external name 'NSPrintPanelAccessorizing';
  NSRuleEditorDelegateProtocol = objcprotocol external name 'NSRuleEditorDelegate';
  NSSeguePerformingProtocol = objcprotocol external name 'NSSeguePerforming';
  NSServicesMenuRequestorProtocol = objcprotocol external name 'NSServicesMenuRequestor';
  NSSharingServiceDelegateProtocol = objcprotocol external name 'NSSharingServiceDelegate';
  NSSharingServicePickerDelegateProtocol = objcprotocol external name 'NSSharingServicePickerDelegate';
  NSSoundDelegateProtocol = objcprotocol external name 'NSSoundDelegate';
  NSSpeechRecognizerDelegateProtocol = objcprotocol external name 'NSSpeechRecognizerDelegate';
  NSSpeechSynthesizerDelegateProtocol = objcprotocol external name 'NSSpeechSynthesizerDelegate';
  NSSplitViewDelegateProtocol = objcprotocol external name 'NSSplitViewDelegate';
  NSStackViewDelegateProtocol = objcprotocol external name 'NSStackViewDelegate';
  NSTabViewDelegateProtocol = objcprotocol external name 'NSTabViewDelegate';
  NSTableViewDataSourceProtocol = objcprotocol external name 'NSTableViewDataSource';
  NSTableViewDelegateProtocol = objcprotocol external name 'NSTableViewDelegate';
  NSTextAttachmentCellProtocol = objcprotocol external name 'NSTextAttachmentCell';
  NSTextDelegateProtocol = objcprotocol external name 'NSTextDelegate';
  NSTextFieldDelegateProtocol = objcprotocol external name 'NSTextFieldDelegate';
  NSTextFinderBarContainerProtocol = objcprotocol external name 'NSTextFinderBarContainer';
  NSTextFinderClientProtocol = objcprotocol external name 'NSTextFinderClient';
  NSTextInputClientProtocol = objcprotocol external name 'NSTextInputClient';
  NSTextInputProtocol = objcprotocol external name 'NSTextInput';
  NSTextLayoutOrientationProviderProtocol = objcprotocol external name 'NSTextLayoutOrientationProvider';
  NSTextStorageDelegateProtocol = objcprotocol external name 'NSTextStorageDelegate';
  NSTextViewDelegateProtocol = objcprotocol external name 'NSTextViewDelegate';
  NSTokenFieldCellDelegateProtocol = objcprotocol external name 'NSTokenFieldCellDelegate';
  NSTokenFieldDelegateProtocol = objcprotocol external name 'NSTokenFieldDelegate';
  NSToolbarDelegateProtocol = objcprotocol external name 'NSToolbarDelegate';
  NSUserInterfaceItemIdentificationProtocol = objcprotocol external name 'NSUserInterfaceItemIdentification';
  NSUserInterfaceItemSearchingProtocol = objcprotocol external name 'NSUserInterfaceItemSearching';
  NSUserInterfaceValidationsProtocol = objcprotocol external name 'NSUserInterfaceValidations';
  NSValidatedUserInterfaceItemProtocol = objcprotocol external name 'NSValidatedUserInterfaceItem';
  NSViewControllerPresentationAnimatorProtocol = objcprotocol external name 'NSViewControllerPresentationAnimator';
  NSWindowDelegateProtocol = objcprotocol external name 'NSWindowDelegate';
  NSWindowRestorationProtocol = objcprotocol external name 'NSWindowRestoration';

type
  CALayer = objcclass external;
  CAMediaTimingFunction = objcclass external;
  CIContext = objcclass external;
  CIFilter = objcclass external;
  CIImage = objcclass external;
  NSAffineTransform = objcclass external;
  NSArray = objcclass external;
  NSAttributedString = objcclass external;
  NSBundle = objcclass external;
  NSCalendar = objcclass external;
  NSCloseCommand = objcclass external;
  NSControlAuxiliary = objcclass external;
  NSData = objcclass external;
  NSDate = objcclass external;
  NSDictionary = objcclass external;
  NSDisplayLink = objcclass external;
  NSEntityDescription = objcclass external;
  NSError = objcclass external;
  NSException = objcclass external;
  NSFetchRequest = objcclass external;
  NSFileWrapper = objcclass external;
  NSFormatter = objcclass external;
  NSIBObjectData = objcclass external;
  NSIndexPath = objcclass external;
  NSIndexSet = objcclass external;
  NSLocale = objcclass external;
  NSLock = objcclass external;
  NSManagedObjectContext = objcclass external;
  NSManagedObjectModel = objcclass external;
  NSMouseTracker = objcclass external;
  NSMutableArray = objcclass external;
  NSMutableCharacterSet = objcclass external;
  NSMutableData = objcclass external;
  NSMutableDictionary = objcclass external;
  NSMutableIndexSet = objcclass external;
  NSMutableSet = objcclass external;
  NSNotification = objcclass external;
  NSNotificationCenter = objcclass external;
  NSOperationQueue = objcclass external;
  NSOrthography = objcclass external;
  NSPointerArray = objcclass external;
  NSPredicate = objcclass external;
  NSPredicateTemplate = objcclass external;
  NSRunStorage = objcclass external;
  NSSavePanelAuxiliary = objcclass external;
  NSScriptCommand = objcclass external;
  NSScriptObjectSpecifier = objcclass external;
  NSSet = objcclass external;
  NSSortDescriptor = objcclass external;
  NSSortedArray = objcclass external;
  NSStorage = objcclass external;
  NSString = objcclass external;
  NSTabWell = objcclass external;
  NSTimeZone = objcclass external;
  NSTimer = objcclass external;
  NSToolbarItemViewer = objcclass external;
  NSURL = objcclass external;
  NSUndoManager = objcclass external;
  NSUserActivity = objcclass external;
  NSUserDefaults = objcclass external;
  NSWindowAuxiliary = objcclass external;
  QTMovie = objcclass external;
  _NSImageAuxiliary = objcclass external;
  _NSOVRowEntry = objcclass external;
  _NSViewAuxiliary = objcclass external;
  NSExtensionRequestHandlingProtocol = objcprotocol external name 'NSExtensionRequestHandling';
  NSMenuItemProtocol = objcprotocol external name 'NSMenuItem';

implementation
end.
