{$define FRAMEWORK_LOADED_FOUNDATION}
{$include Foundation.inc}
{$include NSObjCRuntime.inc}
{$include NSZone.inc}
{$include NSObject.inc}
{$include NSEnumerator.inc}
{$include NSValue.inc}
{$include NSRange.inc}
{$include NSArray.inc}
{$include NSAutoreleasePool.inc}
{$include NSBundle.inc}
{$include NSByteOrder.inc}
{$include NSDate.inc}
{$include NSCalendar.inc}
{$include NSString.inc}
{$include NSCharacterSet.inc}
{$include NSCoder.inc}
{$include NSData.inc}
{$include NSFormatter.inc}
{$include NSDateFormatter.inc}
{$include NSDateIntervalFormatter.inc}
{$include NSMassFormatter.inc}
{$include NSLengthFormatter.inc}
{$include NSEnergyFormatter.inc}
{$include NSDecimal.inc}
{$include NSScanner.inc}
{$include NSDictionary.inc}
{$include NSDecimalNumber.inc}
{$include NSError.inc}
{$include NSException.inc}
{$include NSFileHandle.inc}
{$include NSPathUtilities.inc}
{$include NSFileManager.inc}
{$include NSPointerFunctions.inc}
{$include NSHashTable.inc}
{$include NSHTTPCookie.inc}
{$include NSHTTPCookieStorage.inc}
{$include NSIndexPath.inc}
{$include NSIndexSet.inc}
{$include NSInvocation.inc}
{$include NSJSONSerialization.inc}
{$include NSOrderedSet.inc}
{$include NSSet.inc}
{$include NSKeyValueCoding.inc}
{$include NSKeyValueObserving.inc}
{$include NSPropertyList.inc}
{$include NSGeometry.inc}
{$include NSKeyedArchiver.inc}
{$include NSLocale.inc}
{$include NSLock.inc}
{$include NSMapTable.inc}
{$include NSMethodSignature.inc}
{$include NSNotification.inc}
{$include NSNotificationQueue.inc}
{$include NSNull.inc}
{$include NSNumberFormatter.inc}
{$include NSOperation.inc}
{$include NSOrthography.inc}
{$include NSPointerArray.inc}
{$include NSPort.inc}
{$include NSProcessInfo.inc}
{$include NSProxy.inc}
{$include NSTextCheckingResult.inc}
{$include NSRegularExpression.inc}
{$include NSRunLoop.inc}
{$include NSSortDescriptor.inc}
{$include NSStream.inc}
{$include NSThread.inc}
{$include NSTimeZone.inc}
{$include NSTimer.inc}
{$include NSURLHandle.inc}
{$include NSURL.inc}
{$include NSURLAuthenticationChallenge.inc}
{$include NSURLCache.inc}
{$include NSURLConnection.inc}
{$include NSURLCredential.inc}
{$include NSURLCredentialStorage.inc}
{$include NSURLError.inc}
{$include NSURLProtectionSpace.inc}
{$include NSURLProtocol.inc}
{$include NSURLRequest.inc}
{$include NSURLResponse.inc}
{$include NSUserDefaults.inc}
{$include NSValueTransformer.inc}
{$include NSXMLParser.inc}
{$include FoundationErrors.inc}
{$include NSAttributedString.inc}
{$include NSByteCountFormatter.inc}
{$include NSCache.inc}
{$include NSPredicate.inc}
{$include NSComparisonPredicate.inc}
{$include NSCompoundPredicate.inc}
{$include NSDateComponentsFormatter.inc}
{$include NSExpression.inc}
{$include NSExtensionContext.inc}
{$include NSItemProvider.inc}
{$include NSExtensionItem.inc}
{$include NSExtensionRequestHandling.inc}
{$include NSFileCoordinator.inc}
{$include NSFilePresenter.inc}
{$include NSFileVersion.inc}
{$include NSFileWrapper.inc}
{$include NSLinguisticTagger.inc}
{$include NSMetadataAttributes.inc}
{$include NSMetadata.inc}
{$include NSNetServices.inc}
{$include NSProgress.inc}
{$include NSUbiquitousKeyValueStore.inc}
{$include NSUndoManager.inc}
{$include NSURLSession.inc}
{$include NSUserActivity.inc}
{$include NSUUID.inc}
{$include NSArchiver.inc}
{$include NSBackgroundActivityScheduler.inc}
{$include NSCalendarDate.inc}
{$include NSConnection.inc}
{$include NSDistantObject.inc}
{$include NSDistributedNotificationCenter.inc}
{$include NSPortCoder.inc}
{$include NSPortMessage.inc}
{$include NSPortNameServer.inc}
{$include NSProtocolChecker.inc}
{$include NSTask.inc}
{$include NSXMLNodeOptions.inc}
{$include NSXMLNode.inc}
{$include NSXMLDTD.inc}
{$include NSXMLDTDNode.inc}
{$include NSXMLDocument.inc}
{$include NSXMLElement.inc}
{$include NSURLDownload.inc}
{$include NSAffineTransform.inc}
{$include NSAppleEventDescriptor.inc}
{$include NSAppleEventManager.inc}
{$include NSAppleScript.inc}
{$include NSClassDescription.inc}
{$include NSDistributedLock.inc}
{$include NSGarbageCollector.inc}
{$include NSHFSFileTypes.inc}
{$include NSHost.inc}
{$include NSObjectScripting.inc}
{$include NSScriptClassDescription.inc}
{$include NSScriptCoercionHandler.inc}
{$include NSScriptCommand.inc}
{$include NSScriptCommandDescription.inc}
{$include NSScriptExecutionContext.inc}
{$include NSScriptKeyValueCoding.inc}
{$include NSScriptObjectSpecifiers.inc}
{$include NSScriptStandardSuiteCommands.inc}
{$include NSScriptSuiteRegistry.inc}
{$include NSScriptWhoseTests.inc}
{$include NSSpellServer.inc}
{$include NSUserNotification.inc}
{$include NSUserScriptTask.inc}
{$include NSXPCConnection.inc}
