{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesGameKit;
interface

type
  GKAchievement = objcclass external;
  GKAchievementChallenge = objcclass external;
  GKAchievementDescription = objcclass external;
  GKAchievementViewController = objcclass external;
  GKChallenge = objcclass external;
  GKChallengeEventHandler = objcclass external;
  GKChallengesViewController = objcclass external;
  GKDialogController = objcclass external;
  GKFriendRequestComposeViewController = objcclass external;
  GKGameCenterViewController = objcclass external;
  GKInvite = objcclass external;
  GKLeaderboard = objcclass external;
  GKLeaderboardSet = objcclass external;
  GKLeaderboardViewController = objcclass external;
  GKLocalPlayer = objcclass external;
  GKMatch = objcclass external;
  GKMatchRequest = objcclass external;
  GKMatchmaker = objcclass external;
  GKMatchmakerViewController = objcclass external;
  GKNotificationBanner = objcclass external;
  GKPlayer = objcclass external;
  GKSavedGame = objcclass external;
  GKScore = objcclass external;
  GKScoreChallenge = objcclass external;
  GKSession = objcclass external;
  GKTurnBasedEventHandler = objcclass external;
  GKTurnBasedExchange = objcclass external;
  GKTurnBasedExchangeReply = objcclass external;
  GKTurnBasedMatch = objcclass external;
  GKTurnBasedMatchmakerViewController = objcclass external;
  GKTurnBasedParticipant = objcclass external;
  GKVoiceChat = objcclass external;
  GKVoiceChatService = objcclass external;
  GKAchievementViewControllerDelegateProtocol = objcprotocol external name 'GKAchievementViewControllerDelegate';
  GKChallengeEventHandlerDelegateProtocol = objcprotocol external name 'GKChallengeEventHandlerDelegate';
  GKChallengeListenerProtocol = objcprotocol external name 'GKChallengeListener';
  GKChallengesViewControllerDelegateProtocol = objcprotocol external name 'GKChallengesViewControllerDelegate';
  GKFriendRequestComposeViewControllerDelegateProtocol = objcprotocol external name 'GKFriendRequestComposeViewControllerDelegate';
  GKGameCenterControllerDelegateProtocol = objcprotocol external name 'GKGameCenterControllerDelegate';
  GKInviteEventListenerProtocol = objcprotocol external name 'GKInviteEventListener';
  GKLeaderboardViewControllerDelegateProtocol = objcprotocol external name 'GKLeaderboardViewControllerDelegate';
  GKLocalPlayerListenerProtocol = objcprotocol external name 'GKLocalPlayerListener';
  GKMatchDelegateProtocol = objcprotocol external name 'GKMatchDelegate';
  GKMatchmakerViewControllerDelegateProtocol = objcprotocol external name 'GKMatchmakerViewControllerDelegate';
  GKSavedGameListenerProtocol = objcprotocol external name 'GKSavedGameListener';
  GKSessionDelegateProtocol = objcprotocol external name 'GKSessionDelegate';
  GKTurnBasedEventHandlerDelegateProtocol = objcprotocol external name 'GKTurnBasedEventHandlerDelegate';
  GKTurnBasedEventListenerProtocol = objcprotocol external name 'GKTurnBasedEventListener';
  GKTurnBasedMatchmakerViewControllerDelegateProtocol = objcprotocol external name 'GKTurnBasedMatchmakerViewControllerDelegate';
  GKViewControllerProtocol = objcprotocol external name 'GKViewController';
  GKVoiceChatClientProtocol = objcprotocol external name 'GKVoiceChatClient';

type
  GKGame = objcclass external;
  GKPlayerInternal = objcclass external;
  GKThreadsafeDictionary = objcclass external;
  NSImage = objcclass external;
  NSViewController = objcclass external;
  UIImage = objcclass external;
  UIViewController = objcclass external;

implementation
end.
