unit tb0497b;

interface

function compress (dest : Pbyte;
                   var destLen : cardinal;
                   const source : array of byte;
                   sourceLen : cardinal) : integer;

implementation

function compress (dest : Pbyte;
                   var destLen : cardinal;
                   const source : array of byte;
                   sourceLen : cardinal) : integer;
begin
end;


end.