{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesInstallerPlugins;
interface

type
  InstallerPane = objcclass external;
  InstallerSection = objcclass external;
  InstallerState = objcclass external;

implementation
end.
