{$INCLUDE base/wincewindowd.inc}
{$INCLUDE base/wincekeyboardd.inc}
{$INCLUDE base/wincemoused.inc}
{$INCLUDE gdi/wincebitmapinfod.inc}
{$INCLUDE gdi/wincegdiconsoled.inc}
{$INCLUDE gapi/wincegapiconsoled.inc}

{$INCLUDE base/wincewindowi.inc}
{$INCLUDE base/wincekeyboardi.inc}
{$INCLUDE base/wincemousei.inc}
{$INCLUDE gdi/wincebitmapinfoi.inc}
{$INCLUDE gdi/wincegdiconsolei.inc}
{$INCLUDE gapi/wincegapiconsolei.inc}
