{$define FRAMEWORK_LOADED_MEDIALIBRARY}
{$include MediaLibrary.inc}
{$include MLMediaTypes.inc}
{$include MLMediaLibrary.inc}
{$include MLMediaSource.inc}
{$include MLMediaGroup.inc}
{$include MLMediaObject.inc}
