{$define FRAMEWORK_LOADED_OSAKIT}
{$include OSAKit.inc}
{$include OSALanguage.inc}
{$include OSALanguageInstance.inc}
{$include OSAScript.inc}
{$include OSAScriptView.inc}
{$include OSAScriptController.inc}
