
const
   SIOCADDRT = $890B;
   SIOCDELRT = $890C;
   SIOCRTMSG = $890D;
   SIOCGIFNAME = $8910;
   SIOCSIFLINK = $8911;
   SIOCGIFCONF = $8912;
   SIOCGIFFLAGS = $8913;
   SIOCSIFFLAGS = $8914;
   SIOCGIFADDR = $8915;
   SIOCSIFADDR = $8916;
   SIOCGIFDSTADDR = $8917;
   SIOCSIFDSTADDR = $8918;
   SIOCGIFBRDADDR = $8919;
   SIOCSIFBRDADDR = $891a;
   SIOCGIFNETMASK = $891b;
   SIOCSIFNETMASK = $891c;
   SIOCGIFMETRIC = $891d;
   SIOCSIFMETRIC = $891e;
   SIOCGIFMEM = $891f;
   SIOCSIFMEM = $8920;
   SIOCGIFMTU = $8921;
   SIOCSIFMTU = $8922;
   SIOCSIFHWADDR = $8924;
   SIOCGIFENCAP = $8925;
   SIOCSIFENCAP = $8926;
   SIOCGIFHWADDR = $8927;
   SIOCGIFSLAVE = $8929;
   SIOCSIFSLAVE = $8930;
   SIOCADDMULTI = $8931;
   SIOCDELMULTI = $8932;
   SIOCGIFINDEX = $8933;
   SIOGIFINDEX = SIOCGIFINDEX;
   SIOCSIFPFLAGS = $8934;
   SIOCGIFPFLAGS = $8935;
   SIOCDIFADDR = $8936;
   SIOCSIFHWBROADCAST = $8937;
   SIOCGIFCOUNT = $8938;
   SIOCGIFBR = $8940;
   SIOCSIFBR = $8941;
   SIOCGIFTXQLEN = $8942;
   SIOCSIFTXQLEN = $8943;
   SIOCDARP = $8953;
   SIOCGARP = $8954;
   SIOCSARP = $8955;
   SIOCDRARP = $8960;
   SIOCGRARP = $8961;
   SIOCSRARP = $8962;
   SIOCGIFMAP = $8970;
   SIOCSIFMAP = $8971;
   SIOCADDDLCI = $8980;
   SIOCDELDLCI = $8981;
   SIOCDEVPRIVATE = $89F0;
   SIOCPROTOPRIVATE = $89E0;


{ ---------------------------------------------------------------------
    Borland compatibility types
  ---------------------------------------------------------------------}

// Type
