{ Parsed from CoreMedia.framework CMTextMarkup.h }

{$ifndef CMTEXTMARKUP_H}

{$ifdef EXTERNAL_SYMBOLS}
var
  kCMTextMarkupAttribute_ForegroundColorARGB: CFStringRef { available starting in __MAC_10_9, __IPHONE_6_0 }; cvar; external;
  kCMTextMarkupAttribute_BackgroundColorARGB: CFStringRef { available starting in __MAC_10_9, __IPHONE_6_0 }; cvar; external;
  kCMTextMarkupAttribute_CharacterBackgroundColorARGB: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_BoldStyle: CFStringRef { available starting in __MAC_10_9, __IPHONE_6_0 }; cvar; external;
  kCMTextMarkupAttribute_ItalicStyle: CFStringRef { available starting in __MAC_10_9, __IPHONE_6_0 }; cvar; external;
  kCMTextMarkupAttribute_UnderlineStyle: CFStringRef { available starting in __MAC_10_9, __IPHONE_6_0 }; cvar; external;
  kCMTextMarkupAttribute_FontFamilyName: CFStringRef { available starting in __MAC_10_9, __IPHONE_6_0 }; cvar; external;
  kCMTextMarkupAttribute_GenericFontFamilyName: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_Default: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_Serif: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_SansSerif: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_Monospace: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_ProportionalSerif: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_ProportionalSansSerif: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_MonospaceSerif: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_MonospaceSansSerif: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_Casual: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_Cursive: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_Fantasy: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupGenericFontName_SmallCapital: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_BaseFontSizePercentageRelativeToVideoHeight: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_RelativeFontSize: CFStringRef { available starting in __MAC_10_9, __IPHONE_6_0 }; cvar; external;
  kCMTextMarkupAttribute_VerticalLayout: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextVerticalLayout_LeftToRight: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextVerticalLayout_RightToLeft: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_Alignment: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAlignmentType_Start: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAlignmentType_Middle: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAlignmentType_End: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAlignmentType_Left: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAlignmentType_Right: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_TextPositionPercentageRelativeToWritingDirection: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_OrthogonalLinePositionPercentageRelativeToWritingDirection: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_WritingDirectionSizePercentage: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupAttribute_CharacterEdgeStyle: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupCharacterEdgeStyle_None: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupCharacterEdgeStyle_Raised: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupCharacterEdgeStyle_Depressed: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupCharacterEdgeStyle_Uniform: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
  kCMTextMarkupCharacterEdgeStyle_DropShadow: CFStringRef { available starting in __MAC_10_9, __IPHONE_7_0 }; cvar; external;
{$endif}

{$endif}
