{ don't edit, this file is generated from x86ins.dat }
(
A_NONE,
A_ADC,
A_ADD,
A_AND,
A_BSF,
A_BSR,
A_BSWAP,
A_BT,
A_BTC,
A_BTR,
A_BTS,
A_CALL,
A_CBW,
A_CDQ,
A_CLC,
A_CLD,
A_CLI,
A_CLTS,
A_CMC,
A_CMP,
A_CMPSB,
A_CMPSD,
A_CMPSW,
A_CMPXCHG,
A_CMPXCHG486,
A_CMPXCHG8B,
A_CPUID,
A_CWD,
A_CWDE,
A_DEC,
A_DIV,
A_EMMS,
A_ENTER,
A_F2XM1,
A_FABS,
A_FADD,
A_FADDP,
A_FBLD,
A_FBSTP,
A_FCHS,
A_FCLEX,
A_FCMOVB,
A_FCMOVBE,
A_FCMOVE,
A_FCMOVNB,
A_FCMOVNBE,
A_FCMOVNE,
A_FCMOVNU,
A_FCMOVU,
A_FCOM,
A_FCOMI,
A_FCOMIP,
A_FCOMP,
A_FCOMPP,
A_FCOS,
A_FDECSTP,
A_FDISI,
A_FDIV,
A_FDIVP,
A_FDIVR,
A_FDIVRP,
A_FEMMS,
A_FENI,
A_FFREE,
A_FIADD,
A_FICOM,
A_FICOMP,
A_FIDIV,
A_FIDIVR,
A_FILD,
A_FIMUL,
A_FINCSTP,
A_FINIT,
A_FIST,
A_FISTP,
A_FISTTP,
A_FISUB,
A_FISUBR,
A_FLD,
A_FLD1,
A_FLDCW,
A_FLDENV,
A_FLDL2E,
A_FLDL2T,
A_FLDLG2,
A_FLDLN2,
A_FLDPI,
A_FLDZ,
A_FMUL,
A_FMULP,
A_FNCLEX,
A_FNDISI,
A_FNENI,
A_FNINIT,
A_FNOP,
A_FNSAVE,
A_FNSTCW,
A_FNSTENV,
A_FNSTSW,
A_FPATAN,
A_FPREM,
A_FPREM1,
A_FPTAN,
A_FRNDINT,
A_FRSTOR,
A_FSAVE,
A_FSCALE,
A_FSETPM,
A_FSIN,
A_FSINCOS,
A_FSQRT,
A_FST,
A_FSTCW,
A_FSTENV,
A_FSTP,
A_FSTSW,
A_FSUB,
A_FSUBP,
A_FSUBR,
A_FSUBRP,
A_FTST,
A_FUCOM,
A_FUCOMI,
A_FUCOMIP,
A_FUCOMP,
A_FUCOMPP,
A_FWAIT,
A_FXAM,
A_FXCH,
A_FXTRACT,
A_FYL2X,
A_FYL2XP1,
A_HLT,
A_IBTS,
A_ICEBP,
A_IDIV,
A_IMUL,
A_IN,
A_INC,
A_INSB,
A_INSD,
A_INSW,
A_INT,
A_INT01,
A_INT1,
A_INT03,
A_INT3,
A_INVD,
A_INVLPG,
A_IRET,
A_IRETD,
A_IRETW,
A_IRETQ,
A_JECXZ,
A_JRCXZ,
A_JMP,
A_LAHF,
A_LAR,
A_LCALL,
A_LEA,
A_LEAVE,
A_LFS,
A_LGDT,
A_LGS,
A_LIDT,
A_LJMP,
A_LLDT,
A_LMSW,
A_LOADALL,
A_LOADALL286,
A_LOCK,
A_LODSB,
A_LODSD,
A_LODSW,
A_LOOP,
A_LOOPE,
A_LOOPNE,
A_LOOPNZ,
A_LOOPZ,
A_LSL,
A_LSS,
A_LTR,
A_MONITOR,
A_MOV,
A_MOVD,
A_MOVQ,
A_MOVSB,
A_MOVSD,
A_MOVSQ,
A_MOVSW,
A_MOVSX,
A_MOVZX,
A_MUL,
A_MWAIT,
A_NEG,
A_NOP,
A_NOT,
A_OR,
A_OUT,
A_OUTSB,
A_OUTSD,
A_OUTSW,
A_PACKSSDW,
A_PACKSSWB,
A_PACKUSWB,
A_PADDB,
A_PADDD,
A_PADDSB,
A_PADDSIW,
A_PADDSW,
A_PADDUSB,
A_PADDUSW,
A_PADDW,
A_PAND,
A_PANDN,
A_PAVEB,
A_PAVGUSB,
A_PCMPEQB,
A_PCMPEQD,
A_PCMPEQW,
A_PCMPGTB,
A_PCMPGTD,
A_PCMPGTW,
A_PDISTIB,
A_PF2ID,
A_PFACC,
A_PFADD,
A_PFCMPEQ,
A_PFCMPGE,
A_PFCMPGT,
A_PFMAX,
A_PFMIN,
A_PFMUL,
A_PFRCP,
A_PFRCPIT1,
A_PFRCPIT2,
A_PFRSQIT1,
A_PFRSQRT,
A_PFSUB,
A_PFSUBR,
A_PI2FD,
A_PMACHRIW,
A_PMADDWD,
A_PMAGW,
A_PMULHRIW,
A_PMULHRWA,
A_PMULHRWC,
A_PMULHW,
A_PMULLW,
A_PMVGEZB,
A_PMVLZB,
A_PMVNZB,
A_PMVZB,
A_POP,
A_POPF,
A_POPFW,
A_POPFQ,
A_POR,
A_PREFETCH,
A_PREFETCHW,
A_PSLLD,
A_PSLLDQ,
A_PSLLQ,
A_PSLLW,
A_PSRAD,
A_PSRAW,
A_PSRLD,
A_PSRLQ,
A_PSRLW,
A_PSUBB,
A_PSUBD,
A_PSUBSB,
A_PSUBSIW,
A_PSUBSW,
A_PSUBUSB,
A_PSUBUSW,
A_PSUBW,
A_PUNPCKHBW,
A_PUNPCKHDQ,
A_PUNPCKHWD,
A_PUNPCKLBW,
A_PUNPCKLDQ,
A_PUNPCKLWD,
A_PUSH,
A_PUSHF,
A_PUSHFW,
A_PUSHFQ,
A_PXOR,
A_RCL,
A_RCR,
A_RDSHR,
A_RDMSR,
A_RDPMC,
A_RDTSC,
A_REP,
A_REPE,
A_REPNE,
A_REPNZ,
A_REPZ,
A_RET,
A_RETF,
A_RETN,
A_RETW,
A_RETFW,
A_RETNW,
A_RETFD,
A_RETQ,
A_RETFQ,
A_RETNQ,
A_ROL,
A_ROR,
A_RSDC,
A_RSLDT,
A_RSM,
A_SAHF,
A_SAL,
A_SAR,
A_SBB,
A_SCASB,
A_SCASD,
A_SCASQ,
A_SCASW,
A_SEGCS,
A_SEGDS,
A_SEGES,
A_SEGFS,
A_SEGGS,
A_SEGSS,
A_SGDT,
A_SHL,
A_SHLD,
A_SHR,
A_SHRD,
A_SIDT,
A_SLDT,
A_SMI,
A_SMINT,
A_SMINTOLD,
A_SMSW,
A_STC,
A_STD,
A_STI,
A_STOSB,
A_STOSD,
A_STOSW,
A_STR,
A_SUB,
A_SVDC,
A_SVLDT,
A_SVTS,
A_SYSCALL,
A_SYSENTER,
A_SYSEXIT,
A_SYSRET,
A_TEST,
A_UD1,
A_UD2,
A_UMOV,
A_VERR,
A_VERW,
A_WAIT,
A_WBINVD,
A_WRSHR,
A_WRMSR,
A_XADD,
A_XBTS,
A_XCHG,
A_XLAT,
A_XLATB,
A_XOR,
A_XSTORE,
A_XCRYPTECB,
A_XCRYPTCBC,
A_XCRYPTCFB,
A_XCRYPTOFB,
A_CMOVcc,
A_Jcc,
A_SETcc,
A_MOVS,
A_CMPS,
A_SCAS,
A_LODS,
A_STOS,
A_INS,
A_OUTS,
A_ADDPS,
A_ADDSS,
A_ANDNPS,
A_ANDPS,
A_CMPEQPS,
A_CMPEQSS,
A_CMPLEPS,
A_CMPLESS,
A_CMPLTPS,
A_CMPLTSS,
A_CMPNEQPS,
A_CMPNEQSS,
A_CMPNLEPS,
A_CMPNLESS,
A_CMPNLTPS,
A_CMPNLTSS,
A_CMPORDPS,
A_CMPORDSS,
A_CMPUNORDPS,
A_CMPUNORDSS,
A_CMPPS,
A_CMPSS,
A_COMISS,
A_CVTPI2PS,
A_CVTPS2PI,
A_CVTSI2SS,
A_CVTSS2SI,
A_CVTTPS2PI,
A_CVTTSS2SI,
A_DIVPS,
A_DIVSS,
A_LDMXCSR,
A_MAXPS,
A_MAXSS,
A_MINPS,
A_MINSS,
A_MOVAPS,
A_MOVHPS,
A_MOVLHPS,
A_MOVLPS,
A_MOVHLPS,
A_MOVMSKPS,
A_MOVNTPS,
A_MOVSS,
A_MOVUPS,
A_MULPS,
A_MULSS,
A_ORPS,
A_RCPPS,
A_RCPSS,
A_RSQRTPS,
A_RSQRTSS,
A_SHUFPS,
A_SQRTPS,
A_SQRTSS,
A_STMXCSR,
A_SUBPS,
A_SUBSS,
A_UCOMISS,
A_UNPCKHPS,
A_UNPCKLPS,
A_XORPS,
A_FXRSTOR,
A_FXSAVE,
A_PREFETCHNTA,
A_PREFETCHT0,
A_PREFETCHT1,
A_PREFETCHT2,
A_SFENCE,
A_MASKMOVQ,
A_MOVNTQ,
A_PAVGB,
A_PAVGW,
A_PEXTRW,
A_PINSRW,
A_PMAXSW,
A_PMAXUB,
A_PMINSW,
A_PMINUB,
A_PMOVMSKB,
A_PMULHUW,
A_PSADBW,
A_PSHUFW,
A_PFNACC,
A_PFPNACC,
A_PI2FW,
A_PF2IW,
A_PSWAPD,
A_FFREEP,
A_MASKMOVDQU,
A_CLFLUSH,
A_MOVNTDQ,
A_MOVNTI,
A_MOVNTPD,
A_PAUSE,
A_LFENCE,
A_MFENCE,
A_MOVDQA,
A_MOVDQU,
A_MOVDQ2Q,
A_MOVQ2DQ,
A_PADDQ,
A_PMULUDQ,
A_PSHUFD,
A_PSHUFHW,
A_PSHUFLW,
A_PSRLDQ,
A_PSUBQ,
A_PUNPCKHQDQ,
A_PUNPCKLQDQ,
A_ADDPD,
A_ADDSD,
A_ANDNPD,
A_ANDPD,
A_CMPEQPD,
A_CMPEQSD,
A_CMPLEPD,
A_CMPLESD,
A_CMPLTPD,
A_CMPLTSD,
A_CMPNEQPD,
A_CMPNEQSD,
A_CMPNLEPD,
A_CMPNLESD,
A_CMPNLTPD,
A_CMPNLTSD,
A_CMPORDPD,
A_CMPORDSD,
A_CMPUNORDPD,
A_CMPUNORDSD,
A_CMPPD,
A_COMISD,
A_CVTDQ2PD,
A_CVTDQ2PS,
A_CVTPD2DQ,
A_CVTPD2PI,
A_CVTPD2PS,
A_CVTPI2PD,
A_CVTPS2DQ,
A_CVTPS2PD,
A_CVTSD2SI,
A_CVTSD2SS,
A_CVTSI2SD,
A_CVTSS2SD,
A_CVTTPD2PI,
A_CVTTPD2DQ,
A_CVTTPS2DQ,
A_CVTTSD2SI,
A_DIVPD,
A_DIVSD,
A_MAXPD,
A_MAXSD,
A_MINPD,
A_MINSD,
A_MOVAPD,
A_MOVHPD,
A_MOVLPD,
A_MOVMSKPD,
A_MOVUPD,
A_MULPD,
A_MULSD,
A_ORPD,
A_SHUFPD,
A_SQRTPD,
A_SQRTSD,
A_SUBPD,
A_SUBSD,
A_UCOMISD,
A_UNPCKHPD,
A_UNPCKLPD,
A_XORPD,
A_ADDSUBPD,
A_ADDSUBPS,
A_HADDPD,
A_HADDPS,
A_HSUBPD,
A_HSUBPS,
A_LDDQU,
A_MOVDDUP,
A_MOVSHDUP,
A_MOVSLDUP,
A_VMREAD,
A_VMWRITE,
A_VMCALL,
A_VMLAUNCH,
A_VMRESUME,
A_VMXOFF,
A_VMXON,
A_VMCLEAR,
A_VMPTRLD,
A_VMPTRST,
A_VMRUN,
A_VMMCALL,
A_VMLOAD,
A_VMSAVE,
A_STGI,
A_CLGI,
A_SKINIT,
A_INVLPGA,
A_MONTMUL,
A_XSHA1,
A_XSHA256,
A_DMINT,
A_RDM,
A_MOVABS,
A_MOVSXD,
A_CQO,
A_CDQE,
A_CMPXCHG16B,
A_MOVNTSS,
A_MOVNTSD,
A_INSERTQ,
A_EXTRQ,
A_LZCNT,
A_PABSB,
A_PABSW,
A_PABSD,
A_PALIGNR,
A_PHADDW,
A_PHADDD,
A_PHADDSW,
A_PHSUBW,
A_PHSUBD,
A_PHSUBSW,
A_PMADDUBSW,
A_PMULHRSW,
A_PSHUFB,
A_PSIGNB,
A_PSIGNW,
A_PSIGND,
A_BLENDPS,
A_BLENDPD,
A_BLENDVPS,
A_BLENDVPD,
A_DPPS,
A_DPPD,
A_EXTRACTPS,
A_INSERTPS,
A_MOVNTDQA,
A_MPSADBW,
A_PACKUSDW,
A_PBLENDVB,
A_PBLENDW,
A_PCMPEQQ,
A_PEXTRB,
A_PEXTRD,
A_PEXTRQ,
A_PHMINPOSUW,
A_PINSRB,
A_PINSRD,
A_PINSRQ,
A_PMAXSB,
A_PMAXSD,
A_PMAXUD,
A_PMAXUW,
A_PMINSB,
A_PMINSD,
A_PMINUW,
A_PMINUD,
A_PMOVSXBW,
A_PMOVSXBD,
A_PMOVSXBQ,
A_PMOVSXWD,
A_PMOVSXWQ,
A_PMOVSXDQ,
A_PMOVZXBW,
A_PMOVZXBD,
A_PMOVZXBQ,
A_PMOVZXWD,
A_PMOVZXWQ,
A_PMOVZXDQ,
A_PMULDQ,
A_PMULLD,
A_PTEST,
A_ROUNDPS,
A_ROUNDPD,
A_ROUNDSS,
A_ROUNDSD,
A_CRC32,
A_PCMPESTRI,
A_PCMPESTRM,
A_PCMPISTRI,
A_PCMPISTRM,
A_PCMPGTQ,
A_POPCNT,
A_AESENC,
A_AESENCLAST,
A_AESDEC,
A_AESDECLAST,
A_AESIMC,
A_AESKEYGENASSIST,
A_RDTSCP,
A_STOSQ,
A_LODSQ,
A_CMPSQ,
A_VADDPD,
A_VADDPS,
A_VADDSD,
A_VADDSS,
A_VADDSUBPD,
A_VADDSUBPS,
A_VAESDEC,
A_VAESDECLAST,
A_VAESENC,
A_VAESENCLAST,
A_VAESIMC,
A_VAESKEYGENASSIST,
A_VANDNPD,
A_VANDNPS,
A_VANDPD,
A_VANDPS,
A_VBLENDPD,
A_VBLENDPS,
A_VBLENDVPD,
A_VBLENDVPS,
A_VBROADCASTF128,
A_VBROADCASTSD,
A_VBROADCASTSS,
A_VCMPEQPS,
A_VCMPLTPS,
A_VCMPLEPS,
A_VCMPUNORDPS,
A_VCMPNEQPS,
A_VCMPNLTPS,
A_VCMPNLEPS,
A_VCMPORDPS,
A_VCMPEQ_UQPS,
A_VCMPNGEPS,
A_VCMPNGTPS,
A_VCMPFALSEPS,
A_VCMPNEQ_OQPS,
A_VCMPGEPS,
A_VCMPGTPS,
A_VCMPTRUEPS,
A_VCMPEQ_OSPS,
A_VCMPLT_OQPS,
A_VCMPLE_OQPS,
A_VCMPUNORD_SPS,
A_VCMPNEQ_USPS,
A_VCMPNLT_UQPS,
A_VCMPNLE_UQPS,
A_VCMPORD_SPS,
A_VCMPEQ_USPS,
A_VCMPNGE_UQPS,
A_VCMPNGT_UQPS,
A_VCMPFALSE_OSPS,
A_VCMPNEQ_OSPS,
A_VCMPGE_OQPS,
A_VCMPGT_OQPS,
A_VCMPTRUE_USPS,
A_VCMPEQPD,
A_VCMPLTPD,
A_VCMPLEPD,
A_VCMPUNORDPD,
A_VCMPNEQPD,
A_VCMPNLTPD,
A_VCMPNLEPD,
A_VCMPORDPD,
A_VCMPEQ_UQPD,
A_VCMPNGEPD,
A_VCMPNGTPD,
A_VCMPFALSEPD,
A_VCMPNEQ_OQPD,
A_VCMPGEPD,
A_VCMPGTPD,
A_VCMPTRUEPD,
A_VCMPEQ_OSPD,
A_VCMPLT_OQPD,
A_VCMPLE_OQPD,
A_VCMPUNORD_SPD,
A_VCMPNEQ_USPD,
A_VCMPNLT_UQPD,
A_VCMPNLE_UQPD,
A_VCMPORD_SPD,
A_VCMPEQ_USPD,
A_VCMPNGE_UQPD,
A_VCMPNGT_UQPD,
A_VCMPFALSE_OSPD,
A_VCMPNEQ_OSPD,
A_VCMPGE_OQPD,
A_VCMPGT_OQPD,
A_VCMPTRUE_USPD,
A_VCMPPD,
A_VCMPPS,
A_VCMPSD,
A_VCMPSS,
A_VCOMISD,
A_VCOMISS,
A_VCVTDQ2PD,
A_VCVTDQ2PS,
A_VCVTPD2DQ,
A_VCVTPD2PS,
A_VCVTPS2DQ,
A_VCVTPS2PD,
A_VCVTSD2SI,
A_VCVTSD2SS,
A_VCVTSI2SD,
A_VCVTSI2SS,
A_VCVTSS2SD,
A_VCVTSS2SI,
A_VCVTTPD2DQ,
A_VCVTTPS2DQ,
A_VCVTTSD2SI,
A_VCVTTSS2SI,
A_VDIVPD,
A_VDIVPS,
A_VDIVSD,
A_VDIVSS,
A_VDPPD,
A_VDPPS,
A_VEXTRACTF128,
A_VEXTRACTPS,
A_VHADDPD,
A_VHADDPS,
A_VHSUBPD,
A_VHSUBPS,
A_VINSERTF128,
A_VINSERTPS,
A_VLDDQU,
A_VLDMXCSR,
A_VMASKMOVDQU,
A_VMASKMOVPD,
A_VMASKMOVPS,
A_VMAXPD,
A_VMAXPS,
A_VMAXSD,
A_VMAXSS,
A_VMINPD,
A_VMINPS,
A_VMINSD,
A_VMINSS,
A_VMOVAPD,
A_VMOVAPS,
A_VMOVD,
A_VMOVDDUP,
A_VMOVDQA,
A_VMOVDQU,
A_VMOVHLPS,
A_VMOVHPD,
A_VMOVHPS,
A_VMOVLHPS,
A_VMOVLPD,
A_VMOVLPS,
A_VMOVMSKPD,
A_VMOVMSKPS,
A_VMOVNTDQ,
A_VMOVNTDQA,
A_VMOVNTPD,
A_VMOVNTPS,
A_VMOVQ,
A_VMOVSD,
A_VMOVSHDUP,
A_VMOVSLDUP,
A_VMOVSS,
A_VMOVUPD,
A_VMOVUPS,
A_VMPSADBW,
A_VMULPD,
A_VMULPS,
A_VMULSD,
A_VMULSS,
A_VORPD,
A_VORPS,
A_VPABSB,
A_VPABSD,
A_VPABSW,
A_VPACKSSDW,
A_VPACKSSWB,
A_VPACKUSDW,
A_VPACKUSWB,
A_VPADDB,
A_VPADDD,
A_VPADDQ,
A_VPADDSB,
A_VPADDSW,
A_VPADDUSB,
A_VPADDUSW,
A_VPADDW,
A_VPALIGNR,
A_VPAND,
A_VPANDN,
A_VPAVGB,
A_VPAVGW,
A_VPBLENDVB,
A_VPBLENDW,
A_VPCLMULQDQ,
A_VPCMPEQB,
A_VPCMPEQD,
A_VPCMPEQQ,
A_VPCMPEQW,
A_VPCMPESTRI,
A_VPCMPESTRM,
A_VPCMPGTB,
A_VPCMPGTD,
A_VPCMPGTQ,
A_VPCMPGTW,
A_VPCMPISTRI,
A_VPCMPISTRM,
A_VPERM2F128,
A_VPERMILPD,
A_VPERMILPS,
A_VPEXTRB,
A_VPEXTRD,
A_VPEXTRQ,
A_VPEXTRW,
A_VPHADDD,
A_VPHADDSW,
A_VPHADDW,
A_VPHMINPOSUW,
A_VPHSUBD,
A_VPHSUBSW,
A_VPHSUBW,
A_VPINSRB,
A_VPINSRD,
A_VPINSRQ,
A_VPINSRW,
A_VPMADDUBSW,
A_VPMADDWD,
A_VPMAXSB,
A_VPMAXSD,
A_VPMAXSW,
A_VPMAXUB,
A_VPMAXUD,
A_VPMAXUW,
A_VPMINSB,
A_VPMINSD,
A_VPMINSW,
A_VPMINUB,
A_VPMINUD,
A_VPMINUW,
A_VPMOVMSKB,
A_VPMOVSXBD,
A_VPMOVSXBQ,
A_VPMOVSXBW,
A_VPMOVSXDQ,
A_VPMOVSXWD,
A_VPMOVSXWQ,
A_VPMOVZXBD,
A_VPMOVZXBQ,
A_VPMOVZXBW,
A_VPMOVZXDQ,
A_VPMOVZXWD,
A_VPMOVZXWQ,
A_VPMULDQ,
A_VPMULHRSW,
A_VPMULHUW,
A_VPMULHW,
A_VPMULLD,
A_VPMULLW,
A_VPMULUDQ,
A_VPOR,
A_VPSADBW,
A_VPSHUFB,
A_VPSHUFD,
A_VPSHUFHW,
A_VPSHUFLW,
A_VPSIGNB,
A_VPSIGND,
A_VPSIGNW,
A_VPSLLD,
A_VPSLLDQ,
A_VPSLLQ,
A_VPSLLW,
A_VPSRAD,
A_VPSRAW,
A_VPSRLD,
A_VPSRLDQ,
A_VPSRLQ,
A_VPSRLW,
A_VPSUBB,
A_VPSUBD,
A_VPSUBQ,
A_VPSUBSB,
A_VPSUBSW,
A_VPSUBUSB,
A_VPSUBUSW,
A_VPSUBW,
A_VPTEST,
A_VPUNPCKHBW,
A_VPUNPCKHDQ,
A_VPUNPCKHQDQ,
A_VPUNPCKHWD,
A_VPUNPCKLBW,
A_VPUNPCKLDQ,
A_VPUNPCKLQDQ,
A_VPUNPCKLWD,
A_VPXOR,
A_VRCPPS,
A_VRCPSS,
A_VROUNDPD,
A_VROUNDPS,
A_VROUNDSD,
A_VROUNDSS,
A_VRSQRTPS,
A_VRSQRTSS,
A_VSHUFPD,
A_VSHUFPS,
A_VSQRTPD,
A_VSQRTPS,
A_VSQRTSD,
A_VSQRTSS,
A_VSTMXCSR,
A_VSUBPD,
A_VSUBPS,
A_VSUBSD,
A_VSUBSS,
A_VTESTPD,
A_VTESTPS,
A_VUCOMISD,
A_VUCOMISS,
A_VUNPCKHPD,
A_VUNPCKHPS,
A_VUNPCKLPD,
A_VUNPCKLPS,
A_VXORPD,
A_VXORPS,
A_VZEROALL,
A_VZEROUPPER,
A_ANDN,
A_BEXTR,
A_TZCNT,
A_RORX,
A_SARX,
A_SHLX,
A_SHRX,
A_VBROADCASTI128,
A_VEXTRACTI128,
A_VINSERTI128,
A_VPBLENDD,
A_VPBROADCASTB,
A_VPBROADCASTD,
A_VPBROADCASTQ,
A_VPBROADCASTW,
A_VPERM2I128,
A_VPERMD,
A_VPERMPD,
A_VPERMPS,
A_VPERMQ,
A_VPMASKMOVD,
A_VPMASKMOVQ,
A_VPSLLVD,
A_VPSLLVQ,
A_VPSRAVD,
A_VPSRLVD,
A_VPSRLVQ,
A_VGATHERDPD,
A_VGATHERDPS,
A_VGATHERQPD,
A_VGATHERQPS,
A_VPGATHERDD,
A_VPGATHERDQ,
A_VPGATHERQD,
A_VPGATHERQQ,
A_VFMADD132PD,
A_VFMADD213PD,
A_VFMADD231PD,
A_VFMADDPD,
A_VFMADD132PS,
A_VFMADD213PS,
A_VFMADD231PS,
A_VFMADD132SD,
A_VFMADD213SD,
A_VFMADD231SD,
A_VFMADD132SS,
A_VFMADD213SS,
A_VFMADD231SS,
A_VFMADDSUB132PD,
A_VFMADDSUB213PD,
A_VFMADDSUB231PD,
A_VFMADDSUB132PS,
A_VFMADDSUB213PS,
A_VFMADDSUB231PS,
A_VFMSUBADD132PD,
A_VFMSUBADD213PD,
A_VFMSUBADD231PD,
A_VFMSUBADD132PS,
A_VFMSUBADD213PS,
A_VFMSUBADD231PS,
A_VFMSUB132PD,
A_VFMSUB213PD,
A_VFMSUB231PD,
A_VFMSUB132PS,
A_VFMSUB213PS,
A_VFMSUB231PS,
A_VFMSUB132SD,
A_VFMSUB213SD,
A_VFMSUB231SD,
A_VFMSUB132SS,
A_VFMSUB213SS,
A_VFMSUB231SS,
A_VFNMADD132PD,
A_VFNMADD213PD,
A_VFNMADD231PD,
A_VFNMADD132PS,
A_VFNMADD213PS,
A_VFNMADD231PS,
A_VFNMADD132SD,
A_VFNMADD213SD,
A_VFNMADD231SD,
A_VFNMADD132SS,
A_VFNMADD213SS,
A_VFNMADD231SS,
A_VFNMSUB132PD,
A_VFNMSUB213PD,
A_VFNMSUB231PD,
A_VFNMSUB132PS,
A_VFNMSUB213PS,
A_VFNMSUB231PS,
A_VFNMSUB132SD,
A_VFNMSUB213SD,
A_VFNMSUB231SD,
A_VFNMSUB132SS,
A_VFNMSUB213SS,
A_VFNMSUB231SS,
A_XACQUIRE,
A_XRELEASE,
A_XBEGIN,
A_XABORT,
A_XEND,
A_XTEST,
A_RDRAND,
A_RDSEED,
A_XGETBV,
A_PREFETCHWT1
);
