{ Parsed from AVFoundation.framework AVAnimation.h }

{$ifdef EXTERNAL_SYMBOLS}
var
  AVCoreAnimationBeginTimeAtZero: CFTimeInterval { available in 10_7, 4_0 }; cvar; external;
  AVLayerVideoGravityResizeAspect: NSString { available in 10_7, 4_0 }; cvar; external;
  AVLayerVideoGravityResizeAspectFill: NSString { available in 10_7, 4_0 }; cvar; external;
  AVLayerVideoGravityResize: NSString { available in 10_7, 4_0 }; cvar; external;
{$endif}

