{ %fail }

{ Source provided for Free Pascal Bug Report 3114 }
{ Submitted by "Maarten Bekers" on  2004-05-23 }
{ e-mail: fpc@elexer.com }
unit tw3114;

{$mode delphi}

interface

function whee();

implementation

function whee();
begin
end;

end.
