{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesCryptoTokenKit;
interface

type
  TKSmartCard = objcclass external;
  TKSmartCardATR = objcclass external;
  TKSmartCardATRInterfaceGroup = objcclass external;
  TKSmartCardSlot = objcclass external;
  TKSmartCardSlotManager = objcclass external;

implementation
end.
