sl.Add('Africa;Eastern Africa;Burundi;BDI;10524117');
sl.Add('Africa;Eastern Africa;Comoros;COM;795601');
sl.Add('Africa;Eastern Africa;Djibouti;DJI;942333');
sl.Add('Africa;Eastern Africa;Eritrea;ERI;5395998');
sl.Add('Africa;Eastern Africa;Ethiopia;ETH;102403196');
sl.Add('Africa;Eastern Africa;Kenya;KEN;48461567');
sl.Add('Africa;Eastern Africa;Madagascar;MDG;24894551');
sl.Add('Africa;Eastern Africa;Malawi;MWI;18091575');
sl.Add('Africa;Eastern Africa;Mauritius;MUS;1263473');
sl.Add('Africa;Eastern Africa;Mozambique;MOZ;28829476');
sl.Add('Africa;Eastern Africa;Rwanda;RWA;11917508');
sl.Add('Africa;Eastern Africa;Seychelles;SYC;94677');
sl.Add('Africa;Eastern Africa;Somalia;SOM;14317996');
sl.Add('Africa;Eastern Africa;Tanzania;TZA;55572201');
sl.Add('Africa;Eastern Africa;Uganda;UGA;41487965');
sl.Add('Africa;Eastern Africa;Zambia;ZMB;16591390');
sl.Add('Africa;Eastern Africa;Zimbabwe;ZWE;16150362');
sl.Add('Africa;Middle Africa;Angola;AGO;28813463');
sl.Add('Africa;Middle Africa;Cameroon;CMR;23439189');
sl.Add('Africa;Middle Africa;Central African Republic;CAF;4594621');
sl.Add('Africa;Middle Africa;Chad;TCD;14452543');
sl.Add('Africa;Middle Africa;Congo, Dem. Rep.;COD;78736153');
sl.Add('Africa;Middle Africa;Congo, Rep.;COG;5125821');
sl.Add('Africa;Middle Africa;Equatorial Guinea;GNQ;1221490');
sl.Add('Africa;Middle Africa;Gabon;GAB;1979786');
sl.Add('Africa;Middle Africa;Sao Tome and Principe;STP;199910');
sl.Add('Africa;Middle Africa;South Sudan;SSD;12230730');
sl.Add('Africa;Northern Africa;Algeria;DZA;40606052');
sl.Add('Africa;Northern Africa;Egypt, Arab Rep.;EGY;95688681');
sl.Add('Africa;Northern Africa;Libya;LBY;6293253');
sl.Add('Africa;Northern Africa;Morocco;MAR;35276786');
sl.Add('Africa;Northern Africa;Sudan;SDN;39578828');
sl.Add('Africa;Northern Africa;Tunisia;TUN;11403248');
sl.Add('Africa;Southern Africa;Botswana;BWA;2250260');
sl.Add('Africa;Southern Africa;Lesotho;LSO;2203821');
sl.Add('Africa;Southern Africa;Namibia;NAM;2479713');
sl.Add('Africa;Southern Africa;South Africa;ZAF;55908865');
sl.Add('Africa;Southern Africa;Swaziland;SWZ;1343098');
sl.Add('Africa;Western Africa;Benin;BEN;10872298');
sl.Add('Africa;Western Africa;Burkina Faso;BFA;18646433');
sl.Add('Africa;Western Africa;Cabo Verde;CPV;539560');
sl.Add('Africa;Western Africa;Cote d''Ivoire;CIV;23695919');
sl.Add('Africa;Western Africa;Gambia, The;GMB;2038501');
sl.Add('Africa;Western Africa;Ghana;GHA;28206728');
sl.Add('Africa;Western Africa;Guinea;GIN;12395924');
sl.Add('Africa;Western Africa;Guinea-Bissau;GNB;1815698');
sl.Add('Africa;Western Africa;Liberia;LBR;4613823');
sl.Add('Africa;Western Africa;Mali;MLI;17994837');
sl.Add('Africa;Western Africa;Mauritania;MRT;4301018');
sl.Add('Africa;Western Africa;Niger;NER;20672987');
sl.Add('Africa;Western Africa;Nigeria;NGA;185989640');
sl.Add('Africa;Western Africa;Senegal;SEN;15411614');
sl.Add('Africa;Western Africa;Sierra Leone;SLE;7396190');
sl.Add('Africa;Western Africa;Togo;TGO;7606374');
sl.Add('Americas;Caribbean;Antigua and Barbuda;ATG;100963');
sl.Add('Americas;Caribbean;Aruba;ABW;104822');
sl.Add('Americas;Caribbean;Bahamas, The;BHS;391232');
sl.Add('Americas;Caribbean;Barbados;BRB;284996');
sl.Add('Americas;Caribbean;British Virgin Islands;VGB;30661');
sl.Add('Americas;Caribbean;Cayman Islands;CYM;60765');
sl.Add('Americas;Caribbean;Cuba;CUB;11475982');
sl.Add('Americas;Caribbean;Curacao;CUW;159999');
sl.Add('Americas;Caribbean;Dominica;DMA;73543');
sl.Add('Americas;Caribbean;Dominican Republic;DOM;10648791');
sl.Add('Americas;Caribbean;Grenada;GRD;107317');
sl.Add('Americas;Caribbean;Haiti;HTI;10847334');
sl.Add('Americas;Caribbean;Jamaica;JAM;2881355');
sl.Add('Americas;Caribbean;Puerto Rico;PRI;3411307');
sl.Add('Americas;Caribbean;Sint Maarten (Dutch part);SXM;40005');
sl.Add('Americas;Caribbean;St. Kitts and Nevis;KNA;54821');
sl.Add('Americas;Caribbean;St. Lucia;LCA;178015');
sl.Add('Americas;Caribbean;St. Martin (French part);MAF;31949');
sl.Add('Americas;Caribbean;St. Vincent and the Grenadines;VCT;109643');
sl.Add('Americas;Caribbean;Trinidad and Tobago;TTO;1364962');
sl.Add('Americas;Caribbean;Turks and Caicos Islands;TCA;34900');
sl.Add('Americas;Caribbean;Virgin Islands (U.S.);VIR;102951');
sl.Add('Americas;Central America;Belize;BLZ;366954');
sl.Add('Americas;Central America;Costa Rica;CRI;4857274');
sl.Add('Americas;Central America;El Salvador;SLV;6344722');
sl.Add('Americas;Central America;Guatemala;GTM;16582469');
sl.Add('Americas;Central America;Honduras;HND;9112867');
sl.Add('Americas;Central America;Nicaragua;NIC;6149928');
sl.Add('Americas;Central America;Panama;PAN;4034119');
sl.Add('Americas;Northern America;Bermuda;BMU;65331');
sl.Add('Americas;Northern America;Canada;CAN;36286425');
sl.Add('Americas;Northern America;Greenland;GRL;56186');
sl.Add('Americas;Northern America;Mexico;MEX;127540423');
sl.Add('Americas;Northern America;United States;USA;323127513');
sl.Add('Americas;South America;Argentina;ARG;43847430');
sl.Add('Americas;South America;Bolivia;BOL;10887882');
sl.Add('Americas;South America;Brazil;BRA;207652865');
sl.Add('Americas;South America;Chile;CHL;17909754');
sl.Add('Americas;South America;Colombia;COL;48653419');
sl.Add('Americas;South America;Ecuador;ECU;16385068');
sl.Add('Americas;South America;Guyana;GUY;773303');
sl.Add('Americas;South America;Paraguay;PRY;6725308');
sl.Add('Americas;South America;Peru;PER;31773839');
sl.Add('Americas;South America;Suriname;SUR;558368');
sl.Add('Americas;South America;Uruguay;URY;3444006');
sl.Add('Americas;South America;Venezuela, RB;VEN;31568179');
sl.Add('Asia;Central Asia;Kazakhstan;KAZ;17797032');
sl.Add('Asia;Central Asia;Kyrgyz Republic;KGZ;6082700');
sl.Add('Asia;Central Asia;Tajikistan;TJK;8734951');
sl.Add('Asia;Central Asia;Turkmenistan;TKM;5662544');
sl.Add('Asia;Central Asia;Uzbekistan;UZB;31848200');
sl.Add('Asia;Eastern Asia;China;CHN;1378665000');
sl.Add('Asia;Eastern Asia;Hong Kong SAR, China;HKG;7346700');
sl.Add('Asia;Eastern Asia;Japan;JPN;126994511');
sl.Add('Asia;Eastern Asia;Korea, Dem. People''s Rep.;PRK;25368620');
sl.Add('Asia;Eastern Asia;Korea, Rep.;KOR;51245707');
sl.Add('Asia;Eastern Asia;Macao SAR, China;MAC;612167');
sl.Add('Asia;Eastern Asia;Mongolia;MNG;3027398');
sl.Add('Asia;South-Eastern Asia;Brunei Darussalam;BRN;423196');
sl.Add('Asia;South-Eastern Asia;Cambodia;KHM;15762370');
sl.Add('Asia;South-Eastern Asia;Indonesia;IDN;261115456');
sl.Add('Asia;South-Eastern Asia;Lao PDR;LAO;6758353');
sl.Add('Asia;South-Eastern Asia;Malaysia;MYS;31187265');
sl.Add('Asia;South-Eastern Asia;Myanmar;MMR;52885223');
sl.Add('Asia;South-Eastern Asia;Philippines;PHL;103320222');
sl.Add('Asia;South-Eastern Asia;Singapore;SGP;5607283');
sl.Add('Asia;South-Eastern Asia;Thailand;THA;68863514');
sl.Add('Asia;South-Eastern Asia;Timor-Leste;TLS;1268671');
sl.Add('Asia;South-Eastern Asia;Vietnam;VNM;92701100');
sl.Add('Asia;Southern Asia;Afghanistan;AFG;34656032');
sl.Add('Asia;Southern Asia;Bangladesh;BGD;162951560');
sl.Add('Asia;Southern Asia;Bhutan;BTN;797765');
sl.Add('Asia;Southern Asia;India;IND;1324171354');
sl.Add('Asia;Southern Asia;Iran, Islamic Rep.;IRN;80277428');
sl.Add('Asia;Southern Asia;Maldives;MDV;417492');
sl.Add('Asia;Southern Asia;Nepal;NPL;28982771');
sl.Add('Asia;Southern Asia;Pakistan;PAK;193203476');
sl.Add('Asia;Southern Asia;Sri Lanka;LKA;21203000');
sl.Add('Asia;Western Asia;Armenia;ARM;2924816');
sl.Add('Asia;Western Asia;Azerbaijan;AZE;9762274');
sl.Add('Asia;Western Asia;Bahrain;BHR;1425171');
sl.Add('Asia;Western Asia;Georgia;GEO;3719300');
sl.Add('Asia;Western Asia;Iraq;IRQ;37202572');
sl.Add('Asia;Western Asia;Israel;ISR;8547100');
sl.Add('Asia;Western Asia;Jordan;JOR;9455802');
sl.Add('Asia;Western Asia;Kuwait;KWT;4052584');
sl.Add('Asia;Western Asia;Lebanon;LBN;6006668');
sl.Add('Asia;Western Asia;Oman;OMN;4424762');
sl.Add('Asia;Western Asia;Qatar;QAT;2569804');
sl.Add('Asia;Western Asia;Saudi Arabia;SAU;32275687');
sl.Add('Asia;Western Asia;Syrian Arab Republic;SYR;18430453');
sl.Add('Asia;Western Asia;Turkey;TUR;79512426');
sl.Add('Asia;Western Asia;United Arab Emirates;ARE;9269612');
sl.Add('Asia;Western Asia;West Bank and Gaza;PSE;4551566');
sl.Add('Asia;Western Asia;Yemen, Rep.;YEM;27584213');
sl.Add('Europe;Central Europe;Slovak Republic;SVK;5428704');
sl.Add('Europe;Eastern Europe;Belarus;BLR;9507120');
sl.Add('Europe;Eastern Europe;Bulgaria;BGR;7127822');
sl.Add('Europe;Eastern Europe;Cyprus;CYP;1170125');
sl.Add('Europe;Eastern Europe;Czech Republic;CZE;10561633');
sl.Add('Europe;Eastern Europe;Hungary;HUN;9817958');
sl.Add('Europe;Eastern Europe;Kosovo;UNK;1816200');
sl.Add('Europe;Eastern Europe;Moldova;MDA;3552000');
sl.Add('Europe;Eastern Europe;Poland;POL;37948016');
sl.Add('Europe;Eastern Europe;Romania;ROU;19705301');
sl.Add('Europe;Eastern Europe;Russian Federation;RUS;144342396');
sl.Add('Europe;Eastern Europe;Ukraine;UKR;45004645');
sl.Add('Europe;Northern Europe;Channel Islands;GGY;164541');
sl.Add('Europe;Northern Europe;Denmark;DNK;5731118');
sl.Add('Europe;Northern Europe;Estonia;EST;1316481');
sl.Add('Europe;Northern Europe;Faroe Islands;FRO;49117');
sl.Add('Europe;Northern Europe;Finland;FIN;5495096');
sl.Add('Europe;Northern Europe;Iceland;ISL;334252');
sl.Add('Europe;Northern Europe;Ireland;IRL;4773095');
sl.Add('Europe;Northern Europe;Isle of Man;IMN;83737');
sl.Add('Europe;Northern Europe;Latvia;LVA;1960424');
sl.Add('Europe;Northern Europe;Lithuania;LTU;2872298');
sl.Add('Europe;Northern Europe;Norway;NOR;5232929');
sl.Add('Europe;Northern Europe;Sweden;SWE;9903122');
sl.Add('Europe;Northern Europe;United Kingdom;GBR;65637239');
sl.Add('Europe;Southern Europe;Albania;ALB;2876101');
sl.Add('Europe;Southern Europe;Andorra;AND;77281');
sl.Add('Europe;Southern Europe;Bosnia and Herzegovina;BIH;3516816');
sl.Add('Europe;Southern Europe;Croatia;HRV;4170600');
sl.Add('Europe;Southern Europe;Gibraltar;GIB;34408');
sl.Add('Europe;Southern Europe;Greece;GRC;10746740');
sl.Add('Europe;Southern Europe;Italy;ITA;60600590');
sl.Add('Europe;Southern Europe;Macedonia, FYR;MKD;2081206');
sl.Add('Europe;Southern Europe;Malta;MLT;436947');
sl.Add('Europe;Southern Europe;Montenegro;MNE;622781');
sl.Add('Europe;Southern Europe;Portugal;PRT;10324611');
sl.Add('Europe;Southern Europe;San Marino;SMR;33203');
sl.Add('Europe;Southern Europe;Serbia;SRB;7057412');
sl.Add('Europe;Southern Europe;Slovenia;SVN;2064845');
sl.Add('Europe;Southern Europe;Spain;ESP;46443959');
sl.Add('Europe;Western Europe;Austria;AUT;8747358');
sl.Add('Europe;Western Europe;Belgium;BEL;11348159');
sl.Add('Europe;Western Europe;France;FRA;66896109');
sl.Add('Europe;Western Europe;Germany;DEU;82667685');
sl.Add('Europe;Western Europe;Liechtenstein;LIE;37666');
sl.Add('Europe;Western Europe;Luxembourg;LUX;582972');
sl.Add('Europe;Western Europe;Monaco;MCO;38499');
sl.Add('Europe;Western Europe;Netherlands;NLD;17018408');
sl.Add('Europe;Western Europe;Switzerland;CHE;8372098');
sl.Add('Oceania;Australia and New Zealand;Australia;AUS;24127159');
sl.Add('Oceania;Australia and New Zealand;New Zealand;NZL;4692700');
sl.Add('Oceania;Melanesia;Fiji;FJI;898760');
sl.Add('Oceania;Melanesia;New Caledonia;NCL;278000');
sl.Add('Oceania;Melanesia;Papua New Guinea;PNG;8084991');
sl.Add('Oceania;Melanesia;Solomon Islands;SLB;599419');
sl.Add('Oceania;Melanesia;Vanuatu;VUT;270402');
sl.Add('Oceania;Micronesia;Guam;GUM;162896');
sl.Add('Oceania;Micronesia;Kiribati;KIR;114395');
sl.Add('Oceania;Micronesia;Marshall Islands;MHL;53066');
sl.Add('Oceania;Micronesia;Micronesia, Fed. Sts.;FSM;104937');
sl.Add('Oceania;Micronesia;Nauru;NRU;13049');
sl.Add('Oceania;Micronesia;Northern Mariana Islands;MNP;55023');
sl.Add('Oceania;Micronesia;Palau;PLW;21503');
sl.Add('Oceania;Polynesia;American Samoa;ASM;55599');
sl.Add('Oceania;Polynesia;French Polynesia;PYF;280208');
sl.Add('Oceania;Polynesia;Samoa;WSM;195125');
sl.Add('Oceania;Polynesia;Tonga;TON;107122');
sl.Add('Oceania;Polynesia;Tuvalu;TUV;11097');
