{$define FRAMEWORK_LOADED_GAMECONTROLLER}
{$include GameController.inc}
{$include GCControllerElement.inc}
{$include GCControllerAxisInput.inc}
{$include GCControllerButtonInput.inc}
{$include GCControllerDirectionPad.inc}
{$include GCMotion.inc}
{$include GCGamepad.inc}
{$include GCGamepadSnapshot.inc}
{$include GCExtendedGamepad.inc}
{$include GCExtendedGamepadSnapshot.inc}
{$include GCController.inc}
