{******************************************************************************
 *****************************************************************************}
A_NONE,
A_ABCD,
A_ADD,A_ADDcc,A_ADDX,A_ADDXcc,
A_AND,A_ANDcc,A_ANDN,A_ANDNcc,
{Branching instructions}
A_JMP,
A_JMPL,
A_CALL,
A_BA,A_Bxx,A_FBA,A_FBxx,
A_CBccc,
A_FLUSH,
{Load instructions}
A_LDSB,A_LDSH,A_LDSTUB,
A_LDUB,A_LDUH,A_LD,A_LDD,A_LDF,A_LDFSR,A_LDDF,A_LDC,A_LDCSR,A_LDDC,A_LDX,A_LDUW,A_LDSW,
A_LDSBA,A_LDSHA,A_LDUBA,A_LDUHA,A_LDA,A_LDDA,
A_LDSTUBA,
A_MULScc,
A_MULX,
A_NOP,
A_OR,A_ORcc,A_ORN,A_ORNcc,
A_RDASR,A_RDY,A_RDPSR,A_RDWIM,A_RDTBR,
A_RESTORE,
A_RET,A_RETL,
A_SAVE,
A_SDIV,A_SDIVcc,A_SDIVX,
A_SMUL,A_SMULcc,
A_SETHI,
A_SLL,A_SRL,A_SRA,A_SLLX,A_SRLX,A_SRAX,
A_STB,A_STH,A_ST,A_STD,A_STF,A_STDF,A_STFSR,A_STDFQ,A_STX,
A_STC,A_STDC,A_STCSR,A_STDCQ,
A_STBA,A_STHA,A_STA,A_STDA,
A_SUB,A_SUBcc,A_SUBX,A_SUBXcc,
A_SWAP,A_SWAPA,A_Ticc,A_TADDcc,A_TSUBcc,A_TADDccTV,A_TSUBccTV,
A_TA,A_Txx,
A_UDIV,A_UDIVcc,A_UDIVX,
A_UMUL,A_UMULcc,
A_UNIMP,
A_WRASR,A_WRY,A_WRPSR,A_WRWIM,A_WRTBR,
A_XNOR,A_XNORcc,
A_XOR,A_XORcc,
{Floating-point instructions}
A_FiTOs,A_FiTOd,A_FiTOq,
A_FsTOi,A_FdTOi,A_FqTOi,
A_FsTOd,A_FsTOq,
A_FdTOs,A_FdTOq,
A_FqTOd,A_FqTOs,
A_FMOVs,A_FNEGs,A_FABSs,
A_FSQRTs,A_FSQRTd,A_FSQRTq,
A_FADDs,A_FADDd,A_FADDq,
A_FSUBs,A_FSUBd,A_FSUBq,
A_FMULs,A_FMULd,A_FMULQ,
A_FdMULq,A_FsMULd,
A_FDIVs,A_FDIVd,A_FDIVq,
A_FCMPs,A_FCMPd,A_FCMPq,
A_FCPop1,A_CPop2,
{Synthetic instructions}
A_btst,A_bset,A_bclr,A_btog,
A_clr,A_clrb,A_clrh,
A_cmp,
A_dec,A_deccc,
A_inc,A_inccc,
A_MOV,A_MOVcc,A_MOVRZ,A_MOVRLEZ,A_MOVRLZ,A_MOVRNZ,A_MOVRGZ,A_MOVRGEZ,
A_NEG,
A_not,
A_set,
A_skipz,A_skipnz,
A_tst,
{ Internal instructions }
A_FMOVd,
A_FABSd,
A_FABSq,
A_FNEGd,
A_FNEGq,
{ Memory barrier instructions }
A_STBAR,
A_MEMBAR,

{ SPARC v9 }
A_FLUSHW,
A_CAS,
A_CASA,
A_CASX,
A_CASXA

