{$define FRAMEWORK_LOADED_PDFKIT}
{$include PDFKit.inc}
{$include PDFAction.inc}
{$include PDFActionGoTo.inc}
{$include PDFActionNamed.inc}
{$include PDFActionResetForm.inc}
{$include PDFActionRemoteGoTo.inc}
{$include PDFActionURL.inc}
{$include PDFPage.inc}
{$include PDFAnnotation.inc}
{$include PDFAnnotationButtonWidget.inc}
{$include PDFAnnotationChoiceWidget.inc}
{$include PDFAnnotationCircle.inc}
{$include PDFAnnotationFreeText.inc}
{$include PDFAnnotationInk.inc}
{$include PDFAnnotationLine.inc}
{$include PDFAnnotationLink.inc}
{$include PDFAnnotationMarkup.inc}
{$include PDFAnnotationPopup.inc}
{$include PDFAnnotationSquare.inc}
{$include PDFAnnotationStamp.inc}
{$include PDFAnnotationText.inc}
{$include PDFAnnotationTextWidget.inc}
{$include PDFBorder.inc}
{$include PDFDestination.inc}
{$include PDFDocument.inc}
{$include PDFOutline.inc}
{$include PDFSelection.inc}
{$include PDFThumbnailView.inc}
{$include PDFView.inc}
