{
   Dummy unit to compile everything in one go

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
}
unit buildwinutilsbase;

interface

uses
    flatsb, winver, mmsystem, comconst, commctrl, comobj, commdlg,
    ole2, activex, shellapi, shlobj, oleserver,  shfolder, richedit,
    imagehlp, wininet, uxtheme, dwmapi, multimon, htmlhelp, winutils,
    comserv, winspool, imm, imm_dyn, nb30, stdole2,
    eventsink, typelib, libkinect10, urlmon, winhttp,
    shlwapi, httpapi;

implementation

end.
