program tb0659a;

{$mode fpc}

{$ifdef FPC_TP}
{$fatal FPC_TP defined}
{$endif}
{$ifdef FPC_OBJFPC}
{$fatal FPC_OBJFPC defined}
{$endif}
{$ifdef FPC_DELPHI}
{$fatal FPC_DELPHI defined}
{$endif}
{$ifdef FPC_MACPAS}
{$fatal FPC_MACPAS defined}
{$endif}
{$ifdef FPC_ISO}
{$fatal FPC_ISO defined}
{$endif}
{$ifdef FPC_EXTENDEDPASCAL}
{$fatal FPC_EXTENDEDPASCAL defined}
{$endif}

begin
end.
