{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesGameController;
interface

type
  GCController = objcclass external;
  GCControllerAxisInput = objcclass external;
  GCControllerButtonInput = objcclass external;
  GCControllerDirectionPad = objcclass external;
  GCControllerElement = objcclass external;
  GCExtendedGamepad = objcclass external;
  GCExtendedGamepadSnapshot = objcclass external;
  GCGamepad = objcclass external;
  GCGamepadSnapshot = objcclass external;
  GCMotion = objcclass external;

implementation
end.
