{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesSpriteKit;
interface

type
  SK3DNode = objcclass external;
  SKAction = objcclass external;
  SKConstraint = objcclass external;
  SKCropNode = objcclass external;
  SKEffectNode = objcclass external;
  SKEmitterNode = objcclass external;
  SKFieldNode = objcclass external;
  SKKeyframeSequence = objcclass external;
  SKLabelNode = objcclass external;
  SKLightNode = objcclass external;
  SKMutableTexture = objcclass external;
  SKNode = objcclass external;
  SKPhysicsBody = objcclass external;
  SKPhysicsContact = objcclass external;
  SKPhysicsJoint = objcclass external;
  SKPhysicsJointFixed = objcclass external;
  SKPhysicsJointLimit = objcclass external;
  SKPhysicsJointPin = objcclass external;
  SKPhysicsJointSliding = objcclass external;
  SKPhysicsJointSpring = objcclass external;
  SKPhysicsWorld = objcclass external;
  SKRange = objcclass external;
  SKReachConstraints = objcclass external;
  SKRegion = objcclass external;
  SKScene = objcclass external;
  SKShader = objcclass external;
  SKShapeNode = objcclass external;
  SKSpriteNode = objcclass external;
  SKTexture = objcclass external;
  SKTextureAtlas = objcclass external;
  SKTransition = objcclass external;
  SKUniform = objcclass external;
  SKVideoNode = objcclass external;
  SKView = objcclass external;
  SKPhysicsContactDelegateProtocol = objcprotocol external name 'SKPhysicsContactDelegate';
  SKSceneDelegateProtocol = objcprotocol external name 'SKSceneDelegate';

type
  AVPlayer = objcclass external;
  CIFilter = objcclass external;
  SCNNode = objcclass external;
  SCNScene = objcclass external;

implementation
end.
