{ Parsed from AppKit.framework NSAppleScriptExtensions.h }

{$ifdef CLASSES}

type
  NSAppleScript_NSExtensions = objccategory external name 'NSExtensions' (NSAppleScript)
    function richTextSource: NSAttributedString; message 'richTextSource';
  end;
{$endif}

