{ Parsed from ServiceManagement.framework SMErrors.h }

{$ifndef __SERVICE_MANAGEMENT_ERRORS__}
{$ifdef TYPES}
const
  kSMErrorInternalFailure = 2;
  kSMErrorInvalidSignature = 3;
  kSMErrorAuthorizationFailure = 4;
  kSMErrorToolNotValid = 5;
  kSMErrorJobNotFound = 6;
  kSMErrorServiceUnavailable = 7;
  kSMErrorJobPlistNotFound = 8;
  kSMErrorJobMustBeEnabled = 9;
  kSMErrorInvalidPlist = 10;
{$endif}

{$endif}
