{$mode delphi}
{$modeswitch objectivec1}
{$modeswitch cvar}
{$packrecords c}

unit DefinedClassesCoreData;
interface

type
  NSAsynchronousFetchRequest = objcclass external;
  NSAsynchronousFetchResult = objcclass external;
  NSAtomicStore = objcclass external;
  NSAtomicStoreCacheNode = objcclass external;
  NSAttributeDescription = objcclass external;
  NSBatchUpdateRequest = objcclass external;
  NSBatchUpdateResult = objcclass external;
  NSEntityDescription = objcclass external;
  NSEntityMapping = objcclass external;
  NSEntityMigrationPolicy = objcclass external;
  NSExpressionDescription = objcclass external;
  NSFetchRequest = objcclass external;
  NSFetchRequestExpression = objcclass external;
  NSFetchedPropertyDescription = objcclass external;
  NSIncrementalStore = objcclass external;
  NSIncrementalStoreNode = objcclass external;
  NSManagedObject = objcclass external;
  NSManagedObjectContext = objcclass external;
  NSManagedObjectID = objcclass external;
  NSManagedObjectModel = objcclass external;
  NSMappingModel = objcclass external;
  NSMergeConflict = objcclass external;
  NSMergePolicy = objcclass external;
  NSMigrationManager = objcclass external;
  NSPersistentStore = objcclass external;
  NSPersistentStoreAsynchronousResult = objcclass external;
  NSPersistentStoreCoordinator = objcclass external;
  NSPersistentStoreRequest = objcclass external;
  NSPersistentStoreResult = objcclass external;
  NSPropertyDescription = objcclass external;
  NSPropertyMapping = objcclass external;
  NSRelationshipDescription = objcclass external;
  NSSaveChangesRequest = objcclass external;

type
  NSArray = objcclass external;
  NSData = objcclass external;
  NSDictionary = objcclass external;
  NSError = objcclass external;
  NSExpression = objcclass external;
  NSMigrationContext = objcclass external;
  NSNotification = objcclass external;
  NSPredicate = objcclass external;
  NSProgress = objcclass external;
  NSSet = objcclass external;
  NSURL = objcclass external;
  NSUndoManager = objcclass external;

implementation
end.
