
const
   _XOPEN_IOV_MAX = _POSIX_UIO_MAXIOV;
   NL_ARGMAX = _POSIX_ARG_MAX;
   NL_LANGMAX = _POSIX2_LINE_MAX;
   NL_MSGMAX = INT_MAX;
   NL_NMAX = INT_MAX;
   NL_SETMAX = INT_MAX;
   NL_TEXTMAX = INT_MAX;
   NZERO = 20;
   WORD_BIT = 32;
   LONG_BIT = 32;

