{$include gba_types.inc}
{$include gba_base.inc}
{$include gba_helper.inc}


{$include boyscout.inc}
{$include fade.inc}
{$include gba_affine.inc}
{$include gba_compression.inc}
{$include gba_console.inc}
{$include gba_dma.inc}
{$include gba_input.inc}
{$include gba_interrupt.inc}
{$include gba_multiboot.inc}
{$include gba_sio.inc}
{$include gba_sound.inc}
{$include gba_sprites.inc}
{$include gba_systemcalls.inc}
{$include gba_timers.inc}
{$include gba_video.inc}
{ $include mappy.inc}
{ $include mbv2.inc}
{$include pcx.inc}

