

function dirname(__path:Pchar):Pchar;cdecl;external clib name 'dirname';
function __xpg_basename(__path:Pchar):Pchar;cdecl;external clib name '__xpg_basename';
// maybe add later. function basename(__path:Pchar):Pchar;cdecl;external clib name '__xpg_basename';

{ ---------------------------------------------------------------------
    Borland compatibility types
  ---------------------------------------------------------------------}

// Type
