
const
  UC_PROP_REC_COUNT = 4065;
  UC_PROP_ARRAY : array[0..(UC_PROP_REC_COUNT-1)] of TUC_Prop = (
    (CategoryData : 232; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 200; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 201; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 177; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 136; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 152; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 104; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 112; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 96; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 1; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 2; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 3; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 4; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 5; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 6; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 7; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 8; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 64; CCC : 0; NumericIndex : 9; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $62; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $64; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $66; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $68; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6A; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6C; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6D; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6E; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6F; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $70; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $72; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $74; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $76; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $78; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7A; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 88; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $41; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $42; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $43; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $44; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $45; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $46; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $47; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $48; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $49; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4B; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4D; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4E; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4F; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $50; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $51; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $52; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $53; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $54; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $55; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $56; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $57; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $58; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $59; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5A; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 120; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 208; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 2; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 3; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9C; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 1; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 128; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 10; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 11; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 12; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E0; byte1 : $00; byte2 : $00;); DecompositionID : 0),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $00; byte2 : $00;); DecompositionID : 1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E2; byte1 : $00; byte2 : $00;); DecompositionID : 2),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $00; byte2 : $00;); DecompositionID : 3),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E4; byte1 : $00; byte2 : $00;); DecompositionID : 4),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $00; byte2 : $00;); DecompositionID : 5),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E6; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $00; byte2 : $00;); DecompositionID : 6),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E8; byte1 : $00; byte2 : $00;); DecompositionID : 7),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $00; byte2 : $00;); DecompositionID : 8),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EA; byte1 : $00; byte2 : $00;); DecompositionID : 9),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EB; byte1 : $00; byte2 : $00;); DecompositionID : 10),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EC; byte1 : $00; byte2 : $00;); DecompositionID : 11),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $ED; byte1 : $00; byte2 : $00;); DecompositionID : 12),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EE; byte1 : $00; byte2 : $00;); DecompositionID : 13),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EF; byte1 : $00; byte2 : $00;); DecompositionID : 14),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F0; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F1; byte1 : $00; byte2 : $00;); DecompositionID : 15),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F2; byte1 : $00; byte2 : $00;); DecompositionID : 16),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $00; byte2 : $00;); DecompositionID : 17),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F4; byte1 : $00; byte2 : $00;); DecompositionID : 18),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F5; byte1 : $00; byte2 : $00;); DecompositionID : 19),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F6; byte1 : $00; byte2 : $00;); DecompositionID : 20),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F8; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F9; byte1 : $00; byte2 : $00;); DecompositionID : 21),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FA; byte1 : $00; byte2 : $00;); DecompositionID : 22),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FB; byte1 : $00; byte2 : $00;); DecompositionID : 23),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FC; byte1 : $00; byte2 : $00;); DecompositionID : 24),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FD; byte1 : $00; byte2 : $00;); DecompositionID : 25),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FE; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 26),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C1; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 27),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C2; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 28),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C3; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 29),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 30),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C5; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 31),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C6; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 32),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C8; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 33),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C9; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 34),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 35),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CB; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 36),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CC; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 37),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CD; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 38),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CE; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 39),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CF; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 40),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D0; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D1; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 41),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D2; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 42),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D3; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 43),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D4; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 44),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D5; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 45),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D6; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 46),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D8; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D9; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 47),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DA; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 48),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DB; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 49),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DC; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 50),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DD; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 51),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DE; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $78; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 52),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $01; byte1 : $01; byte2 : $00;); DecompositionID : 53),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 54),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $03; byte1 : $01; byte2 : $00;); DecompositionID : 55),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 56),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $05; byte1 : $01; byte2 : $00;); DecompositionID : 57),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 58),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $07; byte1 : $01; byte2 : $00;); DecompositionID : 59),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 60),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $09; byte1 : $01; byte2 : $00;); DecompositionID : 61),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 62),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0B; byte1 : $01; byte2 : $00;); DecompositionID : 63),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 64),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0D; byte1 : $01; byte2 : $00;); DecompositionID : 65),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 66),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0F; byte1 : $01; byte2 : $00;); DecompositionID : 67),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 68),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $11; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $13; byte1 : $01; byte2 : $00;); DecompositionID : 69),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 70),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $15; byte1 : $01; byte2 : $00;); DecompositionID : 71),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 72),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $17; byte1 : $01; byte2 : $00;); DecompositionID : 73),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 74),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $19; byte1 : $01; byte2 : $00;); DecompositionID : 75),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 76),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1B; byte1 : $01; byte2 : $00;); DecompositionID : 77),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 78),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1D; byte1 : $01; byte2 : $00;); DecompositionID : 79),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 80),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1F; byte1 : $01; byte2 : $00;); DecompositionID : 81),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 82),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $21; byte1 : $01; byte2 : $00;); DecompositionID : 83),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 84),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $01; byte2 : $00;); DecompositionID : 85),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 86),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $01; byte2 : $00;); DecompositionID : 87),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 88),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $29; byte1 : $01; byte2 : $00;); DecompositionID : 89),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 90),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2B; byte1 : $01; byte2 : $00;); DecompositionID : 91),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 92),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $01; byte2 : $00;); DecompositionID : 93),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 94),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2F; byte1 : $01; byte2 : $00;); DecompositionID : 95),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 96),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $00; byte2 : $00;); DecompositionID : 97),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $32; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $01; byte2 : $00;); DecompositionID : 98),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $34; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 99),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $01; byte2 : $00;); DecompositionID : 100),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $36; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 101),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3A; byte1 : $01; byte2 : $00;); DecompositionID : 102),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $39; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 103),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3C; byte1 : $01; byte2 : $00;); DecompositionID : 104),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3B; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 105),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3E; byte1 : $01; byte2 : $00;); DecompositionID : 106),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3D; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 107),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $40; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3F; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $41; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $44; byte1 : $01; byte2 : $00;); DecompositionID : 108),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $43; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 109),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $46; byte1 : $01; byte2 : $00;); DecompositionID : 110),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $45; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 111),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $48; byte1 : $01; byte2 : $00;); DecompositionID : 112),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $47; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 113),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $01; byte2 : $00;); DecompositionID : 114),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 115),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $01; byte2 : $00;); DecompositionID : 116),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4E; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 117),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $01; byte2 : $00;); DecompositionID : 118),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $50; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 119),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $52; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $01; byte2 : $00;); DecompositionID : 120),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $54; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 121),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $01; byte2 : $00;); DecompositionID : 122),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $56; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 123),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $01; byte2 : $00;); DecompositionID : 124),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $58; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 125),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5B; byte1 : $01; byte2 : $00;); DecompositionID : 126),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5A; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 127),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5D; byte1 : $01; byte2 : $00;); DecompositionID : 128),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5C; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 129),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5F; byte1 : $01; byte2 : $00;); DecompositionID : 130),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5E; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 131),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $01; byte2 : $00;); DecompositionID : 132),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $60; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 133),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $01; byte2 : $00;); DecompositionID : 134),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $62; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 135),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $01; byte2 : $00;); DecompositionID : 136),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $64; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 137),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $66; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $01; byte2 : $00;); DecompositionID : 138),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $68; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 139),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $01; byte2 : $00;); DecompositionID : 140),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6A; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 141),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6D; byte1 : $01; byte2 : $00;); DecompositionID : 142),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6C; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 143),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6F; byte1 : $01; byte2 : $00;); DecompositionID : 144),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6E; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 145),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $01; byte2 : $00;); DecompositionID : 146),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $70; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 147),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $01; byte2 : $00;); DecompositionID : 148),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $72; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 149),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $01; byte2 : $00;); DecompositionID : 150),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $74; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 151),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $01; byte2 : $00;); DecompositionID : 152),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $76; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 153),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FF; byte1 : $00; byte2 : $00;); DecompositionID : 154),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7A; byte1 : $01; byte2 : $00;); DecompositionID : 155),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $79; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 156),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7C; byte1 : $01; byte2 : $00;); DecompositionID : 157),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7B; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 158),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7E; byte1 : $01; byte2 : $00;); DecompositionID : 159),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7D; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 160),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $43; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $82; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $84; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $54; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $88; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $87; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $56; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8C; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8B; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5B; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $92; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $91; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $60; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F6; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $68; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $99; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3D; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6F; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $72; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A1; byte1 : $01; byte2 : $00;); DecompositionID : 161),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 162),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A3; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A5; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $80; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A8; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A7; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AD; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $88; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B0; byte1 : $01; byte2 : $00;); DecompositionID : 163),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AF; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 164),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8A; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8B; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B4; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B3; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B6; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B5; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $92; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B9; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BD; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F7; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C6; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $C6; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CC; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $CC; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CE; byte1 : $01; byte2 : $00;); DecompositionID : 165),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CD; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 166),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D0; byte1 : $01; byte2 : $00;); DecompositionID : 167),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CF; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 168),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D2; byte1 : $01; byte2 : $00;); DecompositionID : 169),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D1; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 170),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D4; byte1 : $01; byte2 : $00;); DecompositionID : 171),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D3; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 172),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D6; byte1 : $01; byte2 : $00;); DecompositionID : 173),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D5; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 174),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D8; byte1 : $01; byte2 : $00;); DecompositionID : 175),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D7; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 176),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DA; byte1 : $01; byte2 : $00;); DecompositionID : 177),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D9; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 178),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DC; byte1 : $01; byte2 : $00;); DecompositionID : 179),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DB; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 180),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $01; byte2 : $00;); DecompositionID : 181),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DE; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 182),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $01; byte2 : $00;); DecompositionID : 183),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E0; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 184),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $01; byte2 : $00;); DecompositionID : 185),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E2; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 186),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E4; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $01; byte2 : $00;); DecompositionID : 187),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E6; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 188),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $01; byte2 : $00;); DecompositionID : 189),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E8; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 190),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EB; byte1 : $01; byte2 : $00;); DecompositionID : 191),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EA; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 192),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $ED; byte1 : $01; byte2 : $00;); DecompositionID : 193),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EC; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 194),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EF; byte1 : $01; byte2 : $00;); DecompositionID : 195),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EE; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 196),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 197),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F1; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F1; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F5; byte1 : $01; byte2 : $00;); DecompositionID : 198),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F4; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 199),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $95; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BF; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F9; byte1 : $01; byte2 : $00;); DecompositionID : 200),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F8; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 201),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FB; byte1 : $01; byte2 : $00;); DecompositionID : 202),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FA; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 203),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FD; byte1 : $01; byte2 : $00;); DecompositionID : 204),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FC; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 205),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FF; byte1 : $01; byte2 : $00;); DecompositionID : 206),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FE; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 207),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $01; byte1 : $02; byte2 : $00;); DecompositionID : 208),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 209),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $03; byte1 : $02; byte2 : $00;); DecompositionID : 210),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 211),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $05; byte1 : $02; byte2 : $00;); DecompositionID : 212),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 213),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $07; byte1 : $02; byte2 : $00;); DecompositionID : 214),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 215),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $09; byte1 : $02; byte2 : $00;); DecompositionID : 216),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 217),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0B; byte1 : $02; byte2 : $00;); DecompositionID : 218),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 219),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0D; byte1 : $02; byte2 : $00;); DecompositionID : 220),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 221),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0F; byte1 : $02; byte2 : $00;); DecompositionID : 222),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 223),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $11; byte1 : $02; byte2 : $00;); DecompositionID : 224),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 225),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $13; byte1 : $02; byte2 : $00;); DecompositionID : 226),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 227),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $15; byte1 : $02; byte2 : $00;); DecompositionID : 228),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 229),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $17; byte1 : $02; byte2 : $00;); DecompositionID : 230),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 231),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $19; byte1 : $02; byte2 : $00;); DecompositionID : 232),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 233),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1B; byte1 : $02; byte2 : $00;); DecompositionID : 234),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 235),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1D; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1F; byte1 : $02; byte2 : $00;); DecompositionID : 236),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 237),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9E; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $02; byte2 : $00;); DecompositionID : 238),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 239),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $29; byte1 : $02; byte2 : $00;); DecompositionID : 240),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 241),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2B; byte1 : $02; byte2 : $00;); DecompositionID : 242),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 243),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $02; byte2 : $00;); DecompositionID : 244),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 245),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2F; byte1 : $02; byte2 : $00;); DecompositionID : 246),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 247),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $31; byte1 : $02; byte2 : $00;); DecompositionID : 248),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $30; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 249),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $02; byte2 : $00;); DecompositionID : 250),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $32; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 251),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3C; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3B; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9A; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $66; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7E; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7F; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $41; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $80; byte1 : $01; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $89; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8C; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $46; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $48; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4E; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6F; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6D; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $70; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $81; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $86; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $89; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8F; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $90; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AB; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $93; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $94; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8D; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $97; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $96; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $62; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AD; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9C; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6E; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9D; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9F; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $64; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A9; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B1; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $44; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B1; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $45; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B7; byte1 : $01; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 24; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 230; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 232; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 220; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 216; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 202; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 1; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 230; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 252),
    (CategoryData : 40; CCC : 230; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 253),
    (CategoryData : 40; CCC : 230; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 254),
    (CategoryData : 40; CCC : 230; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 255),
    (CategoryData : 40; CCC : 240; NumericIndex : 0; SimpleUpperCase : (byte0 : $99; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 233; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 234; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $70; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $72; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 24; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 256),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $76; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FD; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FE; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FF; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 136; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 257),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 258),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AC; byte1 : $03; byte2 : $00;); DecompositionID : 259),
    (CategoryData : 136; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 260),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AD; byte1 : $03; byte2 : $00;); DecompositionID : 261),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AE; byte1 : $03; byte2 : $00;); DecompositionID : 262),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AF; byte1 : $03; byte2 : $00;); DecompositionID : 263),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CC; byte1 : $03; byte2 : $00;); DecompositionID : 264),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CD; byte1 : $03; byte2 : $00;); DecompositionID : 265),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CE; byte1 : $03; byte2 : $00;); DecompositionID : 266),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 267),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B1; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B2; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B3; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B4; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B5; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B6; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B7; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B8; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B9; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BA; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BB; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BC; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BD; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BE; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BF; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C0; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C1; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C3; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C4; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C5; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C6; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C7; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C8; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CA; byte1 : $03; byte2 : $00;); DecompositionID : 268),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CB; byte1 : $03; byte2 : $00;); DecompositionID : 269),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $86; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 270),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $88; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 271),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $89; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 272),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 273),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 274),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $91; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $92; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $93; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $94; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $95; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $96; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $97; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $99; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9A; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9B; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9D; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9E; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9F; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A1; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A3; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A5; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A7; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A9; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 275),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AB; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 276),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8C; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 277),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 278),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8F; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 279),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D7; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 280),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 281),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CF; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D8; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DA; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DC; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DE; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E0; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E2; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E4; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E6; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E8; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EB; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EA; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $ED; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EC; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EF; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EE; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F9; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7F; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F8; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F7; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F2; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FB; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FA; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7B; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7C; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $03; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $50; byte1 : $04; byte2 : $00;); DecompositionID : 282),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $04; byte2 : $00;); DecompositionID : 283),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $52; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $04; byte2 : $00;); DecompositionID : 284),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $54; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $56; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $04; byte2 : $00;); DecompositionID : 285),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $58; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5A; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5B; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5C; byte1 : $04; byte2 : $00;); DecompositionID : 286),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5D; byte1 : $04; byte2 : $00;); DecompositionID : 287),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5E; byte1 : $04; byte2 : $00;); DecompositionID : 288),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5F; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $30; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $31; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $32; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $34; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $36; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $38; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $39; byte1 : $04; byte2 : $00;); DecompositionID : 289),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3A; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3B; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3C; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3D; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3E; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3F; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $40; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $44; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $46; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $48; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4A; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4C; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4E; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $11; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $13; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $15; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $17; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $19; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 290),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1B; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1D; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1F; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $21; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $23; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $25; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $27; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $29; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2B; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2D; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2F; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 291),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $01; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 292),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $03; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 293),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $05; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $07; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 294),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $09; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0B; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 295),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0D; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 296),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 297),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0F; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $60; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $62; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $64; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $66; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $68; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6A; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6D; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6C; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6F; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6E; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $70; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $72; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $74; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $04; byte2 : $00;); DecompositionID : 298),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $76; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 299),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $78; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7B; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7A; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7C; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7F; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7E; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $80; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 56; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8B; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8D; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8C; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8F; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $91; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $90; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $93; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $92; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $95; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $94; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $97; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $96; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $99; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9B; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9A; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9D; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9C; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9F; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9E; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A1; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A3; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A5; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A7; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A9; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AB; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AD; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AF; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B1; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B3; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B5; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B7; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B9; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BB; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BD; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BF; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CF; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C2; byte1 : $04; byte2 : $00;); DecompositionID : 300),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C1; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 301),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C4; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C3; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C6; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C5; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C8; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CA; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C9; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CC; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CB; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CE; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CD; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D1; byte1 : $04; byte2 : $00;); DecompositionID : 302),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D0; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 303),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D3; byte1 : $04; byte2 : $00;); DecompositionID : 304),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D2; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 305),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D5; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D4; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D7; byte1 : $04; byte2 : $00;); DecompositionID : 306),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D6; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 307),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D8; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $04; byte2 : $00;); DecompositionID : 308),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DA; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 309),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $04; byte2 : $00;); DecompositionID : 310),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DC; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 311),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $04; byte2 : $00;); DecompositionID : 312),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DE; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 313),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E0; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $04; byte2 : $00;); DecompositionID : 314),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E2; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 315),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $04; byte2 : $00;); DecompositionID : 316),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E4; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 317),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $04; byte2 : $00;); DecompositionID : 318),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E6; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 319),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E8; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EB; byte1 : $04; byte2 : $00;); DecompositionID : 320),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EA; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 321),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $ED; byte1 : $04; byte2 : $00;); DecompositionID : 322),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EC; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 323),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EF; byte1 : $04; byte2 : $00;); DecompositionID : 324),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EE; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 325),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F1; byte1 : $04; byte2 : $00;); DecompositionID : 326),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F0; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 327),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $04; byte2 : $00;); DecompositionID : 328),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F2; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 329),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F5; byte1 : $04; byte2 : $00;); DecompositionID : 330),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F4; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 331),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F7; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F6; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F9; byte1 : $04; byte2 : $00;); DecompositionID : 332),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F8; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 333),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FB; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FA; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FD; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FC; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FF; byte1 : $04; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FE; byte1 : $04; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $01; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $03; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $05; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $07; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $09; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0B; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0D; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0F; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $11; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $13; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $15; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $17; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $19; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1B; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1D; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1F; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $21; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $29; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2B; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2F; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $62; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $64; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $66; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $68; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6A; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6C; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6D; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6E; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6F; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $70; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $72; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $74; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $76; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $78; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7A; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7B; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7C; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7E; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7F; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $80; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $82; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $84; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $86; byte1 : $05; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $31; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $32; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $33; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $34; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $35; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $36; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $37; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $38; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $39; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3A; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3B; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3C; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3D; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3E; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3F; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $40; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $41; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $42; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $43; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $44; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $45; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $46; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $47; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $48; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $49; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4B; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4D; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4E; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4F; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $50; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $51; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $52; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $53; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $54; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $55; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $56; byte1 : $05; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 222; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 228; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 10; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 11; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 12; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 13; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 14; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 15; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 16; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 17; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 18; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 19; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 20; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 21; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 22; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 23; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 24; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 25; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 30; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 31; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 32; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 334),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 335),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 336),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 337),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 338),
    (CategoryData : 40; CCC : 27; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 28; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 29; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 33; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 34; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 35; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 339),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 340),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 341),
    (CategoryData : 40; CCC : 36; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 342),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 343),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 344),
    (CategoryData : 40; CCC : 7; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 9; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 345),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 346),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 347),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 348),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 349),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 350),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 351),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 352),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 353),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 354),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 355),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 356),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 357),
    (CategoryData : 80; CCC : 0; NumericIndex : 13; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 14; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 15; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 16; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 358),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 359),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 360),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 361),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 362),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 363),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 364),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 365),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 366),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 367),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 368),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 369),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 370),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 371),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 372),
    (CategoryData : 80; CCC : 0; NumericIndex : 17; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 18; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 19; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 373),
    (CategoryData : 40; CCC : 84; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 91; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 374),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 375),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 376),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 377),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 378),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 379),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 380),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 381),
    (CategoryData : 80; CCC : 0; NumericIndex : 20; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 21; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 22; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 23; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 24; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 25; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 26; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 382),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 383),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 384),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 385),
    (CategoryData : 40; CCC : 103; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 107; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 118; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 122; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 27; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 28; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 29; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 30; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 31; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 32; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 33; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 34; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 35; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 386),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 387),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 388),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 389),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 390),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 391),
    (CategoryData : 40; CCC : 129; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 130; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 392),
    (CategoryData : 40; CCC : 132; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 393),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 394),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 395),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 396),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 397),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 398),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 399),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 400),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 401),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 402),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 403),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $01; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $02; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $03; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $04; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $05; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $06; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $07; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $08; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $09; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0A; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0B; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0C; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0D; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0E; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0F; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $10; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $11; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $12; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $13; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $14; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $15; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $16; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $17; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $18; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $19; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1A; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1B; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1C; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1D; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1E; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1F; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $20; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $21; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $22; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $24; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $2D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 34; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 4; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 5; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 6; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 7; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 8; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 9; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 36; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 37; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 38; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 39; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 40; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 41; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 42; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 43; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 44; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $70; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $72; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $74; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $76; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $78; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7A; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7B; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7C; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7E; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7F; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $80; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $82; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $84; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $86; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $87; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $88; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $89; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8A; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8B; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8C; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8D; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8E; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8F; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $90; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $91; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $92; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $93; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $94; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $95; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $96; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $97; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $98; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $99; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9A; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9B; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9C; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9D; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9E; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9F; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A0; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A1; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A2; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A3; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A4; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A5; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A6; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A7; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A8; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A9; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AA; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AB; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AC; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AD; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AE; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AF; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B0; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B1; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B2; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B3; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B4; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B5; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B6; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B7; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B8; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B9; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BA; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BB; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BC; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BD; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BE; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BF; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F8; byte1 : $13; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F9; byte1 : $13; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FA; byte1 : $13; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FB; byte1 : $13; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FC; byte1 : $13; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FD; byte1 : $13; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F0; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F1; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F2; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F3; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F4; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F5; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 45; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 46; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 47; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 404),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 405),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 406),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 407),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 408),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 409),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 410),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 411),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 412),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 413),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 414),
    (CategoryData : 48; CCC : 9; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7D; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $63; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 214; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $01; byte1 : $1E; byte2 : $00;); DecompositionID : 415),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 416),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $03; byte1 : $1E; byte2 : $00;); DecompositionID : 417),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 418),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $05; byte1 : $1E; byte2 : $00;); DecompositionID : 419),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 420),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $07; byte1 : $1E; byte2 : $00;); DecompositionID : 421),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 422),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $09; byte1 : $1E; byte2 : $00;); DecompositionID : 423),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 424),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0B; byte1 : $1E; byte2 : $00;); DecompositionID : 425),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 426),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0D; byte1 : $1E; byte2 : $00;); DecompositionID : 427),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 428),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $0F; byte1 : $1E; byte2 : $00;); DecompositionID : 429),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 430),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $11; byte1 : $1E; byte2 : $00;); DecompositionID : 431),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 432),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $13; byte1 : $1E; byte2 : $00;); DecompositionID : 433),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 434),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $15; byte1 : $1E; byte2 : $00;); DecompositionID : 435),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 436),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $17; byte1 : $1E; byte2 : $00;); DecompositionID : 437),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 438),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $19; byte1 : $1E; byte2 : $00;); DecompositionID : 439),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 440),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1B; byte1 : $1E; byte2 : $00;); DecompositionID : 441),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 442),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1D; byte1 : $1E; byte2 : $00;); DecompositionID : 443),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 444),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $1F; byte1 : $1E; byte2 : $00;); DecompositionID : 445),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 446),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $21; byte1 : $1E; byte2 : $00;); DecompositionID : 447),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 448),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $1E; byte2 : $00;); DecompositionID : 449),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 450),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $1E; byte2 : $00;); DecompositionID : 451),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 452),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $1E; byte2 : $00;); DecompositionID : 453),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 454),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $29; byte1 : $1E; byte2 : $00;); DecompositionID : 455),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 456),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2B; byte1 : $1E; byte2 : $00;); DecompositionID : 457),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 458),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $1E; byte2 : $00;); DecompositionID : 459),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 460),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2F; byte1 : $1E; byte2 : $00;); DecompositionID : 461),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 462),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $31; byte1 : $1E; byte2 : $00;); DecompositionID : 463),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $30; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 464),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $1E; byte2 : $00;); DecompositionID : 465),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $32; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 466),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $1E; byte2 : $00;); DecompositionID : 467),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $34; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 468),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $1E; byte2 : $00;); DecompositionID : 469),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $36; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 470),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $39; byte1 : $1E; byte2 : $00;); DecompositionID : 471),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $38; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 472),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3B; byte1 : $1E; byte2 : $00;); DecompositionID : 473),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 474),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3D; byte1 : $1E; byte2 : $00;); DecompositionID : 475),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 476),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3F; byte1 : $1E; byte2 : $00;); DecompositionID : 477),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 478),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $1E; byte2 : $00;); DecompositionID : 479),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $40; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 480),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $1E; byte2 : $00;); DecompositionID : 481),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $42; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 482),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $1E; byte2 : $00;); DecompositionID : 483),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $44; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 484),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $1E; byte2 : $00;); DecompositionID : 485),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $46; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 486),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $1E; byte2 : $00;); DecompositionID : 487),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $48; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 488),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $1E; byte2 : $00;); DecompositionID : 489),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 490),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $1E; byte2 : $00;); DecompositionID : 491),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 492),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $1E; byte2 : $00;); DecompositionID : 493),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 494),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $1E; byte2 : $00;); DecompositionID : 495),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $50; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 496),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $1E; byte2 : $00;); DecompositionID : 497),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $52; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 498),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $1E; byte2 : $00;); DecompositionID : 499),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $54; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 500),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $1E; byte2 : $00;); DecompositionID : 501),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $56; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 502),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $1E; byte2 : $00;); DecompositionID : 503),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $58; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 504),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5B; byte1 : $1E; byte2 : $00;); DecompositionID : 505),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 506),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5D; byte1 : $1E; byte2 : $00;); DecompositionID : 507),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 508),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5F; byte1 : $1E; byte2 : $00;); DecompositionID : 509),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 510),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $1E; byte2 : $00;); DecompositionID : 511),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $60; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 512),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $1E; byte2 : $00;); DecompositionID : 513),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $62; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 514),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $1E; byte2 : $00;); DecompositionID : 515),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $64; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 516),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $1E; byte2 : $00;); DecompositionID : 517),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $66; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 518),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $1E; byte2 : $00;); DecompositionID : 519),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $68; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 520),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $1E; byte2 : $00;); DecompositionID : 521),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 522),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6D; byte1 : $1E; byte2 : $00;); DecompositionID : 523),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 524),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6F; byte1 : $1E; byte2 : $00;); DecompositionID : 525),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 526),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $1E; byte2 : $00;); DecompositionID : 527),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $70; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 528),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $1E; byte2 : $00;); DecompositionID : 529),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $72; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 530),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $1E; byte2 : $00;); DecompositionID : 531),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $74; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 532),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $1E; byte2 : $00;); DecompositionID : 533),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $76; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 534),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $1E; byte2 : $00;); DecompositionID : 535),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $78; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 536),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7B; byte1 : $1E; byte2 : $00;); DecompositionID : 537),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 538),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $1E; byte2 : $00;); DecompositionID : 539),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 540),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7F; byte1 : $1E; byte2 : $00;); DecompositionID : 541),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 542),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $1E; byte2 : $00;); DecompositionID : 543),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $80; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 544),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $1E; byte2 : $00;); DecompositionID : 545),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $82; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 546),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $1E; byte2 : $00;); DecompositionID : 547),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $84; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 548),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $87; byte1 : $1E; byte2 : $00;); DecompositionID : 549),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $86; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 550),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $89; byte1 : $1E; byte2 : $00;); DecompositionID : 551),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $88; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 552),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8B; byte1 : $1E; byte2 : $00;); DecompositionID : 553),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 554),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8D; byte1 : $1E; byte2 : $00;); DecompositionID : 555),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8C; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 556),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8F; byte1 : $1E; byte2 : $00;); DecompositionID : 557),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 558),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $91; byte1 : $1E; byte2 : $00;); DecompositionID : 559),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $90; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 560),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $93; byte1 : $1E; byte2 : $00;); DecompositionID : 561),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $92; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 562),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $95; byte1 : $1E; byte2 : $00;); DecompositionID : 563),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $94; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 564),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 565),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 566),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 567),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 568),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $60; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 569),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A1; byte1 : $1E; byte2 : $00;); DecompositionID : 570),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 571),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A3; byte1 : $1E; byte2 : $00;); DecompositionID : 572),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 573),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A5; byte1 : $1E; byte2 : $00;); DecompositionID : 574),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 575),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A7; byte1 : $1E; byte2 : $00;); DecompositionID : 576),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 577),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A9; byte1 : $1E; byte2 : $00;); DecompositionID : 578),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 579),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AB; byte1 : $1E; byte2 : $00;); DecompositionID : 580),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 581),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AD; byte1 : $1E; byte2 : $00;); DecompositionID : 582),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 583),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AF; byte1 : $1E; byte2 : $00;); DecompositionID : 584),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 585),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B1; byte1 : $1E; byte2 : $00;); DecompositionID : 586),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 587),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B3; byte1 : $1E; byte2 : $00;); DecompositionID : 588),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 589),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B5; byte1 : $1E; byte2 : $00;); DecompositionID : 590),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 591),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B7; byte1 : $1E; byte2 : $00;); DecompositionID : 592),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 593),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B9; byte1 : $1E; byte2 : $00;); DecompositionID : 594),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 595),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BB; byte1 : $1E; byte2 : $00;); DecompositionID : 596),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 597),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BD; byte1 : $1E; byte2 : $00;); DecompositionID : 598),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 599),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BF; byte1 : $1E; byte2 : $00;); DecompositionID : 600),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 601),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C1; byte1 : $1E; byte2 : $00;); DecompositionID : 602),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 603),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C3; byte1 : $1E; byte2 : $00;); DecompositionID : 604),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C2; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 605),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C5; byte1 : $1E; byte2 : $00;); DecompositionID : 606),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 607),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C7; byte1 : $1E; byte2 : $00;); DecompositionID : 608),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C6; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 609),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $1E; byte2 : $00;); DecompositionID : 610),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C8; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 611),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CB; byte1 : $1E; byte2 : $00;); DecompositionID : 612),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 613),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CD; byte1 : $1E; byte2 : $00;); DecompositionID : 614),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CC; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 615),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CF; byte1 : $1E; byte2 : $00;); DecompositionID : 616),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CE; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 617),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D1; byte1 : $1E; byte2 : $00;); DecompositionID : 618),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D0; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 619),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D3; byte1 : $1E; byte2 : $00;); DecompositionID : 620),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D2; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 621),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D5; byte1 : $1E; byte2 : $00;); DecompositionID : 622),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D4; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 623),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D7; byte1 : $1E; byte2 : $00;); DecompositionID : 624),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D6; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 625),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $1E; byte2 : $00;); DecompositionID : 626),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D8; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 627),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $1E; byte2 : $00;); DecompositionID : 628),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DA; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 629),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $1E; byte2 : $00;); DecompositionID : 630),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DC; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 631),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $1E; byte2 : $00;); DecompositionID : 632),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DE; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 633),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $1E; byte2 : $00;); DecompositionID : 634),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E0; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 635),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $1E; byte2 : $00;); DecompositionID : 636),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E2; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 637),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $1E; byte2 : $00;); DecompositionID : 638),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E4; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 639),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $1E; byte2 : $00;); DecompositionID : 640),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E6; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 641),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $1E; byte2 : $00;); DecompositionID : 642),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E8; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 643),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EB; byte1 : $1E; byte2 : $00;); DecompositionID : 644),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EA; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 645),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $ED; byte1 : $1E; byte2 : $00;); DecompositionID : 646),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EC; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 647),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EF; byte1 : $1E; byte2 : $00;); DecompositionID : 648),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EE; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 649),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F1; byte1 : $1E; byte2 : $00;); DecompositionID : 650),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F0; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 651),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $1E; byte2 : $00;); DecompositionID : 652),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F2; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 653),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F5; byte1 : $1E; byte2 : $00;); DecompositionID : 654),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F4; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 655),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F7; byte1 : $1E; byte2 : $00;); DecompositionID : 656),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F6; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 657),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F9; byte1 : $1E; byte2 : $00;); DecompositionID : 658),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F8; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 659),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FB; byte1 : $1E; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FA; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FD; byte1 : $1E; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FC; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FF; byte1 : $1E; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FE; byte1 : $1E; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 660),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $09; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 661),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 662),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 663),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 664),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 665),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 666),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0F; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 667),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $1F; byte2 : $00;); DecompositionID : 668),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $01; byte1 : $1F; byte2 : $00;); DecompositionID : 669),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $02; byte1 : $1F; byte2 : $00;); DecompositionID : 670),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $03; byte1 : $1F; byte2 : $00;); DecompositionID : 671),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $04; byte1 : $1F; byte2 : $00;); DecompositionID : 672),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $05; byte1 : $1F; byte2 : $00;); DecompositionID : 673),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $06; byte1 : $1F; byte2 : $00;); DecompositionID : 674),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $07; byte1 : $1F; byte2 : $00;); DecompositionID : 675),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 676),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $19; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 677),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 678),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 679),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 680),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 681),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $10; byte1 : $1F; byte2 : $00;); DecompositionID : 682),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $11; byte1 : $1F; byte2 : $00;); DecompositionID : 683),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $12; byte1 : $1F; byte2 : $00;); DecompositionID : 684),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $13; byte1 : $1F; byte2 : $00;); DecompositionID : 685),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $14; byte1 : $1F; byte2 : $00;); DecompositionID : 686),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $15; byte1 : $1F; byte2 : $00;); DecompositionID : 687),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 688),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $29; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 689),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 690),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 691),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 692),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 693),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 694),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2F; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 695),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $20; byte1 : $1F; byte2 : $00;); DecompositionID : 696),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $21; byte1 : $1F; byte2 : $00;); DecompositionID : 697),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $22; byte1 : $1F; byte2 : $00;); DecompositionID : 698),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $1F; byte2 : $00;); DecompositionID : 699),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $24; byte1 : $1F; byte2 : $00;); DecompositionID : 700),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $1F; byte2 : $00;); DecompositionID : 701),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $26; byte1 : $1F; byte2 : $00;); DecompositionID : 702),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $1F; byte2 : $00;); DecompositionID : 703),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $38; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 704),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $39; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 705),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 706),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 707),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 708),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 709),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3E; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 710),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3F; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 711),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $30; byte1 : $1F; byte2 : $00;); DecompositionID : 712),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $31; byte1 : $1F; byte2 : $00;); DecompositionID : 713),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $32; byte1 : $1F; byte2 : $00;); DecompositionID : 714),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $1F; byte2 : $00;); DecompositionID : 715),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $34; byte1 : $1F; byte2 : $00;); DecompositionID : 716),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $1F; byte2 : $00;); DecompositionID : 717),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $36; byte1 : $1F; byte2 : $00;); DecompositionID : 718),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $1F; byte2 : $00;); DecompositionID : 719),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $48; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 720),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $49; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 721),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 722),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 723),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 724),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 725),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $40; byte1 : $1F; byte2 : $00;); DecompositionID : 726),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $1F; byte2 : $00;); DecompositionID : 727),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $1F; byte2 : $00;); DecompositionID : 728),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $1F; byte2 : $00;); DecompositionID : 729),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $44; byte1 : $1F; byte2 : $00;); DecompositionID : 730),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $1F; byte2 : $00;); DecompositionID : 731),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 732),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $59; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 733),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 734),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 735),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 736),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 737),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 738),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5F; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 739),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $1F; byte2 : $00;); DecompositionID : 740),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $1F; byte2 : $00;); DecompositionID : 741),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $1F; byte2 : $00;); DecompositionID : 742),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $1F; byte2 : $00;); DecompositionID : 743),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $68; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 744),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $69; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 745),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 746),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 747),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 748),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 749),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6E; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 750),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6F; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 751),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $60; byte1 : $1F; byte2 : $00;); DecompositionID : 752),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $1F; byte2 : $00;); DecompositionID : 753),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $62; byte1 : $1F; byte2 : $00;); DecompositionID : 754),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $1F; byte2 : $00;); DecompositionID : 755),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $64; byte1 : $1F; byte2 : $00;); DecompositionID : 756),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $1F; byte2 : $00;); DecompositionID : 757),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $66; byte1 : $1F; byte2 : $00;); DecompositionID : 758),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $1F; byte2 : $00;); DecompositionID : 759),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 760),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BB; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 761),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C8; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 762),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C9; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 763),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 764),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CB; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 765),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DA; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 766),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DB; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 767),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F8; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 768),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F9; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 769),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EA; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 770),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EB; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 771),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FA; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 772),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FB; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 773),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $88; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 774),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $89; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 775),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 776),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 777),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 778),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 779),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 780),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8F; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 781),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $80; byte1 : $1F; byte2 : $00;); DecompositionID : 782),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $1F; byte2 : $00;); DecompositionID : 783),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $82; byte1 : $1F; byte2 : $00;); DecompositionID : 784),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $1F; byte2 : $00;); DecompositionID : 785),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $84; byte1 : $1F; byte2 : $00;); DecompositionID : 786),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $1F; byte2 : $00;); DecompositionID : 787),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $86; byte1 : $1F; byte2 : $00;); DecompositionID : 788),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $87; byte1 : $1F; byte2 : $00;); DecompositionID : 789),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 790),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $99; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 791),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9A; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 792),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9B; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 793),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9C; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 794),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9D; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 795),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9E; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 796),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9F; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 797),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $90; byte1 : $1F; byte2 : $00;); DecompositionID : 798),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $91; byte1 : $1F; byte2 : $00;); DecompositionID : 799),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $92; byte1 : $1F; byte2 : $00;); DecompositionID : 800),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $93; byte1 : $1F; byte2 : $00;); DecompositionID : 801),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $94; byte1 : $1F; byte2 : $00;); DecompositionID : 802),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $95; byte1 : $1F; byte2 : $00;); DecompositionID : 803),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $96; byte1 : $1F; byte2 : $00;); DecompositionID : 804),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $97; byte1 : $1F; byte2 : $00;); DecompositionID : 805),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 806),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A9; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 807),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 808),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AB; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 809),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 810),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AD; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 811),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 812),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AF; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 813),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A0; byte1 : $1F; byte2 : $00;); DecompositionID : 814),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A1; byte1 : $1F; byte2 : $00;); DecompositionID : 815),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A2; byte1 : $1F; byte2 : $00;); DecompositionID : 816),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A3; byte1 : $1F; byte2 : $00;); DecompositionID : 817),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A4; byte1 : $1F; byte2 : $00;); DecompositionID : 818),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A5; byte1 : $1F; byte2 : $00;); DecompositionID : 819),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A6; byte1 : $1F; byte2 : $00;); DecompositionID : 820),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A7; byte1 : $1F; byte2 : $00;); DecompositionID : 821),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 822),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B9; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 823),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 824),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 825),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 826),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 827),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 828),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B0; byte1 : $1F; byte2 : $00;); DecompositionID : 829),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B1; byte1 : $1F; byte2 : $00;); DecompositionID : 830),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $70; byte1 : $1F; byte2 : $00;); DecompositionID : 831),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $1F; byte2 : $00;); DecompositionID : 832),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B3; byte1 : $1F; byte2 : $00;); DecompositionID : 833),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $99; byte1 : $03; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 834),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 835),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 836),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CC; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 837),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 838),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 839),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 840),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $72; byte1 : $1F; byte2 : $00;); DecompositionID : 841),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $1F; byte2 : $00;); DecompositionID : 842),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $74; byte1 : $1F; byte2 : $00;); DecompositionID : 843),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $1F; byte2 : $00;); DecompositionID : 844),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C3; byte1 : $1F; byte2 : $00;); DecompositionID : 845),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 846),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 847),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 848),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D8; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 849),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D9; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 850),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 851),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 852),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 853),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 854),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D0; byte1 : $1F; byte2 : $00;); DecompositionID : 855),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D1; byte1 : $1F; byte2 : $00;); DecompositionID : 856),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $76; byte1 : $1F; byte2 : $00;); DecompositionID : 857),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $1F; byte2 : $00;); DecompositionID : 858),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 859),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 860),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 861),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E8; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 862),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E9; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 863),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 864),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 865),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 866),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EC; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 867),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 868),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 869),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E0; byte1 : $1F; byte2 : $00;); DecompositionID : 870),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $1F; byte2 : $00;); DecompositionID : 871),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7A; byte1 : $1F; byte2 : $00;); DecompositionID : 872),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7B; byte1 : $1F; byte2 : $00;); DecompositionID : 873),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $1F; byte2 : $00;); DecompositionID : 874),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 875),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 876),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 877),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 878),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $FC; byte1 : $1F; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 879),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 880),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 881),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 882),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $78; byte1 : $1F; byte2 : $00;); DecompositionID : 883),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $1F; byte2 : $00;); DecompositionID : 884),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7C; byte1 : $1F; byte2 : $00;); DecompositionID : 885),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $1F; byte2 : $00;); DecompositionID : 886),
    (CategoryData : 16; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $1F; byte2 : $00;); DecompositionID : 887),
    (CategoryData : 160; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 888),
    (CategoryData : 177; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 889),
    (CategoryData : 177; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 890),
    (CategoryData : 185; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 193; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $03; byte2 : $00;); DecompositionID : 891),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $00; byte2 : $00;); DecompositionID : 892),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $00; byte2 : $00;); DecompositionID : 893),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4E; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $32; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 48; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 49; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 50; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 51; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 52; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 53; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 54; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 55; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 56; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 57; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 58; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 59; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 1; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $70; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 2; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 3; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $72; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 4; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 5; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $74; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 6; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $75; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 7; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $76; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 8; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $77; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 9; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $78; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 17; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 60; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7A; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 61; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7B; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 39; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7C; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 18; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 62; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7E; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 19; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7F; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 1; SimpleUpperCase : (byte0 : $60; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 2; SimpleUpperCase : (byte0 : $61; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 3; SimpleUpperCase : (byte0 : $62; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 4; SimpleUpperCase : (byte0 : $63; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 5; SimpleUpperCase : (byte0 : $64; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 6; SimpleUpperCase : (byte0 : $65; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 7; SimpleUpperCase : (byte0 : $66; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 8; SimpleUpperCase : (byte0 : $67; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 9; SimpleUpperCase : (byte0 : $68; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 17; SimpleUpperCase : (byte0 : $69; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 60; SimpleUpperCase : (byte0 : $6A; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 61; SimpleUpperCase : (byte0 : $6B; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 39; SimpleUpperCase : (byte0 : $6C; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 18; SimpleUpperCase : (byte0 : $6D; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 62; SimpleUpperCase : (byte0 : $6E; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 19; SimpleUpperCase : (byte0 : $6F; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 19; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 63; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 44; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $84; byte1 : $21; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $83; byte1 : $21; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 6; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 39; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 64; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 65; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 894),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 895),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 896),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 897),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 898),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 899),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 900),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 901),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 902),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 903),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 904),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 905),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 906),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 907),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 908),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 909),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 910),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 911),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 912),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 913),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 914),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 915),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 916),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 917),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 918),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 919),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 920),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 921),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 922),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 923),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 924),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 925),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 926),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 927),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 928),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 929),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 930),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 931),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 932),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 933),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 934),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 935),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 936),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 937),
    (CategoryData : 104; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 938),
    (CategoryData : 112; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 939),
    (CategoryData : 80; CCC : 0; NumericIndex : 60; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 61; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 66; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 67; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 68; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 45; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 46; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 47; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D0; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D1; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D2; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D3; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D4; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D5; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D6; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D7; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D8; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DA; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DC; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DE; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E0; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E2; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E4; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E6; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E8; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $24; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B7; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B9; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BB; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BD; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BF; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C1; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C2; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C3; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C5; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C6; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C8; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C9; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CB; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CC; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CD; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CE; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CF; byte1 : $24; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 144; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 940),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $30; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $31; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $32; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $34; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $36; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $38; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $39; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3A; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3B; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3C; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3D; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3E; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3F; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $40; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $44; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $46; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $48; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4A; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4C; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4E; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $50; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $52; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $54; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $56; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $58; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5A; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5B; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5C; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5D; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5E; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $01; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $03; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $05; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $07; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $09; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0B; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0D; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0F; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $11; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $13; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $15; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $17; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $19; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1B; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1D; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1F; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $21; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $23; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $25; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $27; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $29; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2B; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2D; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $60; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $1D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7D; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3A; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3E; byte1 : $02; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $68; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $67; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6A; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $69; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6C; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6B; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $71; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $50; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $52; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $73; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $72; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $76; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $75; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3F; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $40; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $80; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $82; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $84; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $87; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $86; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $89; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $88; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8B; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8D; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8C; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8F; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $91; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $90; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $93; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $92; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $95; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $94; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $97; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $96; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $99; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9B; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9A; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9D; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9C; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9F; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9E; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A1; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A3; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A5; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A7; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A9; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AB; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AD; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $AF; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B1; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B3; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B5; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B7; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B9; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BB; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BD; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $BF; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C1; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C3; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C2; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C5; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C7; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C6; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C8; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CB; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CD; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CC; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CF; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CE; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D1; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D0; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D3; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D2; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D5; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D4; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D7; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D6; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D8; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DA; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DC; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DE; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E0; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E2; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EC; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EB; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EE; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $ED; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $2C; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $F2; byte1 : $2C; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A1; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A3; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A5; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A7; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A9; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AB; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AD; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AF; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B1; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B3; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B5; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B7; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B9; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BB; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BD; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BF; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C1; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C2; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C3; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C5; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CD; byte1 : $10; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 1; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 2; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 3; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 4; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 5; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 7; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 8; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 9; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 40; CCC : 218; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 48; CCC : 224; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 17; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 36; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 37; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 941),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 942),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 943),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 944),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 945),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 946),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 947),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 948),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 949),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 950),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 951),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 952),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 953),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 954),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 955),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 956),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 957),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 958),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 959),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 960),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 961),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 962),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 963),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 964),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 965),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 966),
    (CategoryData : 40; CCC : 8; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 24; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 967),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 968),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 969),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 970),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 971),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 972),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 973),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 974),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 975),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 976),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 977),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 978),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 979),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 980),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 981),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 982),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 983),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 984),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 985),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 986),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 987),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 988),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 989),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 990),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 991),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 992),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 993),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 994),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 995),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 996),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 997),
    (CategoryData : 24; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 998),
    (CategoryData : 80; CCC : 0; NumericIndex : 69; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 70; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 71; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 72; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 73; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 74; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 75; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 76; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 77; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 78; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 79; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 80; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 81; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 82; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 83; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 84; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 85; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 86; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 87; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 88; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 89; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 90; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 91; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 92; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 93; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 94; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 95; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $40; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $42; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $44; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $46; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $48; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4E; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $50; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $52; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $54; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $56; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $58; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5B; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5A; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5D; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5C; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5F; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5E; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $60; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $62; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $64; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $66; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $68; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6A; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6D; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6C; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $80; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $82; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $84; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $87; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $86; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $89; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $88; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8B; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8D; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8C; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8F; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $91; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $90; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $93; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $92; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $95; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $94; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $97; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $96; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $99; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9B; byte1 : $A6; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9A; byte1 : $A6; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $29; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2B; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2F; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $32; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $34; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $36; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $39; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $38; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3B; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3A; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3D; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3C; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3F; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3E; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $40; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $42; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $44; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $46; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $48; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4A; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4C; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $4E; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $50; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $52; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $54; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $56; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $58; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5B; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5A; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5D; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5C; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5F; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $5E; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $60; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $63; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $62; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $64; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $67; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $66; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $69; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $68; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6B; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6A; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6D; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6C; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6F; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $6E; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7A; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $79; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7C; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7B; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $79; byte1 : $1D; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $7F; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $7E; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $81; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $80; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $83; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $82; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $85; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $84; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $87; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $86; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $8C; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8B; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $65; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $91; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $90; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $93; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $92; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $97; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $96; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $99; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9B; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9A; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9D; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9C; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9F; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9E; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A1; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A3; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A5; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A7; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $A9; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $66; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5C; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $61; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6C; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $6A; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9E; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $87; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $9D; byte1 : $02; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $AB; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B5; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $B7; byte1 : $A7; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B3; byte1 : $A7; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A1; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A3; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A5; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A7; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A9; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AB; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AD; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AF; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B1; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B3; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B5; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B7; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B9; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BB; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BD; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BF; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C1; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C2; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C3; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C5; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C6; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C8; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C9; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CB; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CC; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CD; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CE; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CF; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D0; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D1; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D2; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D3; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D4; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D5; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D6; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D7; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D8; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D9; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DA; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DB; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DC; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DD; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DE; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $DF; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E0; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E1; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E2; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E3; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E4; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E5; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E6; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E7; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E8; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $E9; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EA; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EB; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EC; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $ED; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EE; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $EF; byte1 : $13; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 216; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 224; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 999),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1000),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1001),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1002),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1003),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1004),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1005),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1006),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1007),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1008),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1009),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1010),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1011),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1012),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1013),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1014),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1015),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1016),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1017),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1018),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1019),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1020),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1021),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1022),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1023),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1024),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1025),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1026),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1027),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1028),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1029),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1030),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1031),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1032),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1033),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1034),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1035),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1036),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1037),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1038),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1039),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1040),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1041),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1042),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1043),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1044),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1045),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1046),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1047),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1048),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1049),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1050),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1051),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1052),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1053),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1054),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1055),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1056),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1057),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1058),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1059),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1060),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1061),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1062),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1063),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1064),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1065),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1066),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1067),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1068),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1069),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1070),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1071),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1072),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1073),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1074),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1075),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1076),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1077),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1078),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1079),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1080),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1081),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1082),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1083),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1084),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1085),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1086),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1087),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1088),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1089),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1090),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1091),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1092),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1093),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1094),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1095),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1096),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1097),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1098),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1099),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1100),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1101),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1102),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1103),
    (CategoryData : 32; CCC : 0; NumericIndex : 3; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1104),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1105),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1106),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1107),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1108),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1109),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1110),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1111),
    (CategoryData : 32; CCC : 0; NumericIndex : 17; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1112),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1113),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1114),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1115),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1116),
    (CategoryData : 32; CCC : 0; NumericIndex : 2; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1117),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1118),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1119),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1120),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1121),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1122),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1123),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1124),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1125),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1126),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1127),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1128),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1129),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1130),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1131),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1132),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1133),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1134),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1135),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1136),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1137),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1138),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1139),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1140),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1141),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1142),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1143),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1144),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1145),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1146),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1147),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1148),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1149),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1150),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1151),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1152),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1153),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1154),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1155),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1156),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1157),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1158),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1159),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1160),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1161),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1162),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1163),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1164),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1165),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1166),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1167),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1168),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1169),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1170),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1171),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1172),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1173),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1174),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1175),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1176),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1177),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1178),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1179),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1180),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1181),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1182),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1183),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1184),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1185),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1186),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1187),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1188),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1189),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1190),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1191),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1192),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1193),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1194),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1195),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1196),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1197),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1198),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1199),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1200),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1201),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1202),
    (CategoryData : 32; CCC : 0; NumericIndex : 6; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1203),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1204),
    (CategoryData : 32; CCC : 0; NumericIndex : 6; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1205),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1206),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1207),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1208),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1209),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1210),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1211),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1212),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1213),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1214),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1215),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1216),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1217),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1218),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1219),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1220),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1221),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1222),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1223),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1224),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1225),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1226),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1227),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1228),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1229),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1230),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1231),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1232),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1233),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1234),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1235),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1236),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1237),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1238),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1239),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1240),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1241),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1242),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1243),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1244),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1245),
    (CategoryData : 32; CCC : 0; NumericIndex : 17; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1246),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1247),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1248),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1249),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1250),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1251),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1252),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1253),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1254),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1255),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1256),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1257),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1258),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1259),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1260),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1261),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1262),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1263),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1264),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1265),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1266),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1267),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1268),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1269),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1270),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1271),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1272),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1273),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1274),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1275),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1276),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1277),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1278),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1279),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1280),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1281),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1282),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1283),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1284),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1285),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1286),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1287),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1288),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1289),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1290),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1291),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1292),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1293),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1294),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1295),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1296),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1297),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1298),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1299),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1300),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1301),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1302),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1303),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1304),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1305),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1306),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1307),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1308),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1309),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1310),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1311),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1312),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1313),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1314),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1315),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1316),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1317),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1318),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1319),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1320),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1321),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1322),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1323),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1324),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1325),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1326),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1327),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1328),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1329),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1330),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1331),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1332),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1333),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1334),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1335),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1336),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1337),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1338),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1339),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1340),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1341),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1342),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1343),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1344),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1345),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1346),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1347),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1348),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1349),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1350),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1351),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1352),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1353),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1354),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1355),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1356),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1357),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1358),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1359),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1360),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1361),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1362),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1363),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1364),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1365),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1366),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1367),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1368),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1369),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1370),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1371),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1372),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1373),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1374),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1375),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1376),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1377),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1378),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1379),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1380),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1381),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1382),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1383),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1384),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1385),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1386),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1387),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1388),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1389),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1390),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1391),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1392),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1393),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1394),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1395),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1396),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1397),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1398),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1399),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1400),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1401),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1402),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1403),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1404),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1405),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1406),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1407),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1408),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1409),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1410),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1411),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1412),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1413),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1414),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1415),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1416),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1417),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1418),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1419),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1420),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1421),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1422),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1423),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1424),
    (CategoryData : 40; CCC : 26; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1425),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1426),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1427),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1428),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1429),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1430),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1431),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1432),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1433),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1434),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1435),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1436),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1437),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1438),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1439),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1440),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1441),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1442),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1443),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1444),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1445),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1446),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1447),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1448),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1449),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1450),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1451),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1452),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1453),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1454),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1455),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1456),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1457),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $44; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $46; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $48; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4A; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4C; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4E; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $50; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $51; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $52; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $53; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $54; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $55; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $56; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $57; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $58; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $59; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $5A; byte1 : $FF; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $21; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $23; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $25; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $27; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $28; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $29; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2A; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2B; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2C; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2D; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2E; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $2F; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $30; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $31; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $32; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $33; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $34; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $35; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $36; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $37; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $38; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $39; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $3A; byte1 : $FF; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 96; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 97; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 98; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 62; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 99; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 100; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 101; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 102; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 103; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 104; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 105; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 63; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 106; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 107; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 108; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 109; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 110; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 111; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 112; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 64; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 113; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 114; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 115; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 116; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 10; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 11; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 62; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 18; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 97; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 43; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 102; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $28; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $29; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2A; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2B; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2C; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2E; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2F; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $30; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $31; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $32; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $34; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $36; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $38; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $39; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3A; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3B; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3C; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3D; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3E; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3F; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $40; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $44; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $45; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $46; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $47; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $48; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $49; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4A; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4B; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4C; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4D; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4E; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $4F; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $01; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $03; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $05; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $07; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $09; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0B; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0D; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0F; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $11; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $13; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $15; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $17; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $19; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1B; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1D; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1F; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $21; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $22; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $23; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $24; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $25; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $26; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $27; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D8; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DA; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DC; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DE; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E0; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E2; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E4; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E6; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E8; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EA; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EB; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EC; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $ED; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EE; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EF; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F0; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F1; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F2; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F3; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F4; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F5; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F6; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F7; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F8; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F9; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FA; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $FB; byte1 : $04; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B1; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B3; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B5; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B7; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B9; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BB; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BD; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BF; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C0; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C1; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C2; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C3; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C4; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C5; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C6; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C7; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C8; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $C9; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CA; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CB; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CC; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CD; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CE; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $CF; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D0; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D1; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D2; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $D3; byte1 : $04; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 117; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 65; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 118; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 119; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 120; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 121; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 122; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 123; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 124; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 125; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 126; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 127; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 128; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C0; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C1; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C2; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C3; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C4; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C5; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C6; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C7; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C8; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CA; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CB; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CC; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CD; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CE; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CF; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D0; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D1; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D2; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D3; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D4; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D5; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D6; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D7; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D8; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DA; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DC; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DE; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E0; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E1; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E2; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E3; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E4; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E5; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E6; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E7; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E8; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $E9; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EA; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EB; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EC; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $ED; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EE; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $EF; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F0; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F1; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $F2; byte1 : $0C; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $80; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $81; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $82; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $83; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $84; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $85; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $86; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $87; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $88; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $89; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8A; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8B; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8C; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8D; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8E; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $8F; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $90; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $91; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $92; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $93; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $94; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $95; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $96; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $97; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $98; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $99; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9A; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9B; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9C; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9D; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9E; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $9F; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A1; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A3; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A5; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A7; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A9; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AB; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AD; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AF; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B1; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $0C; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1458),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1459),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1460),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1461),
    (CategoryData : 40; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1462),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1463),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1464),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1465),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1466),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1467),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1468),
    (CategoryData : 48; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1469),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C0; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C1; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C2; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C3; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C4; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C5; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C6; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C7; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C8; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $C9; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CA; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CB; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CC; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CD; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CE; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $CF; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D0; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D1; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D2; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D3; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D4; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D5; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D6; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D7; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D8; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $D9; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DA; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DB; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DC; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DD; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DE; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $DF; byte1 : $18; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A0; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A1; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A2; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A3; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A4; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A5; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A6; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A7; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A8; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $A9; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AA; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AB; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AC; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AD; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AE; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $AF; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B0; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B1; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B2; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B3; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B4; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B5; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B6; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B7; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B8; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $B9; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BA; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BB; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BC; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BD; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BE; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $BF; byte1 : $18; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 129; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 130; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 50; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 51; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 56; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 14; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 55; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 72; CCC : 0; NumericIndex : 38; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 131; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 132; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 133; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 80; CCC : 0; NumericIndex : 134; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1470),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1471),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1472),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1473),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1474),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1475),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1476),
    (CategoryData : 48; CCC : 216; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 48; CCC : 226; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1477),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1478),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1479),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1480),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1481),
    (CategoryData : 168; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1482),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $22; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $23; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $24; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $25; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $26; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $27; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $28; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $29; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2A; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2B; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2C; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2D; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2E; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $2F; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $30; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $31; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $32; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $33; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $34; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $35; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $36; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $37; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $38; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $39; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3A; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3B; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3C; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3D; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3E; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $3F; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $40; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $41; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $42; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 0; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $43; byte1 : $E9; byte2 : $01;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $01; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $02; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $03; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $04; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $05; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $06; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $07; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $08; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $09; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0A; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0B; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0C; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0D; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0E; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $0F; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $10; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $11; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $12; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $13; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $14; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $15; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $16; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $17; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $18; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $19; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1A; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1B; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1C; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1D; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1E; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $1F; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $20; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 8; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $21; byte1 : $E9; byte2 : $01;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : -1),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1483),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1484),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1485),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1486),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1487),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1488),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1489),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1490),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1491),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1492),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1493),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1494),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1495),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1496),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1497),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1498),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1499),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1500),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1501),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1502),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1503),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1504),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1505),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1506),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1507),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1508),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1509),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1510),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1511),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1512),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1513),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1514),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1515),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1516),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1517),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1518),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1519),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1520),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1521),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1522),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1523),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1524),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1525),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1526),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1527),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1528),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1529),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1530),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1531),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1532),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1533),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1534),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1535),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1536),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1537),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1538),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1539),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1540),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1541),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1542),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1543),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1544),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1545),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1546),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1547),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1548),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1549),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1550),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1551),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1552),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1553),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1554),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1555),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1556),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1557),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1558),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1559),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1560),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1561),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1562),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1563),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1564),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1565),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1566),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1567),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1568),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1569),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1570),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1571),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1572),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1573),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1574),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1575),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1576),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1577),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1578),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1579),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1580),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1581),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1582),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1583),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1584),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1585),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1586),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1587),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1588),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1589),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1590),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1591),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1592),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1593),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1594),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1595),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1596),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1597),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1598),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1599),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1600),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1601),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1602),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1603),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1604),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1605),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1606),
    (CategoryData : 32; CCC : 0; NumericIndex : 9; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1607),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1608),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1609),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1610),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1611),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1612),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1613),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1614),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1615),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1616),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1617),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1618),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1619),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1620),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1621),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1622),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1623),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1624),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1625),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1626),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1627),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1628),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1629),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1630),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1631),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1632),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1633),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1634),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1635),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1636),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1637),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1638),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1639),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1640),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1641),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1642),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1643),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1644),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1645),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1646),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1647),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1648),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1649),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1650),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1651),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1652),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1653),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1654),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1655),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1656),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1657),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1658),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1659),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1660),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1661),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1662),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1663),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1664),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1665),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1666),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1667),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1668),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1669),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1670),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1671),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1672),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1673),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1674),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1675),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1676),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1677),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1678),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1679),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1680),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1681),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1682),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1683),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1684),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1685),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1686),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1687),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1688),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1689),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1690),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1691),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1692),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1693),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1694),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1695),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1696),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1697),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1698),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1699),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1700),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1701),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1702),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1703),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1704),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1705),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1706),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1707),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1708),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1709),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1710),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1711),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1712),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1713),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1714),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1715),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1716),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1717),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1718),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1719),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1720),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1721),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1722),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1723),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1724),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1725),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1726),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1727),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1728),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1729),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1730),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1731),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1732),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1733),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1734),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1735),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1736),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1737),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1738),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1739),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1740),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1741),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1742),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1743),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1744),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1745),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1746),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1747),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1748),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1749),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1750),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1751),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1752),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1753),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1754),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1755),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1756),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1757),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1758),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1759),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1760),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1761),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1762),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1763),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1764),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1765),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1766),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1767),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1768),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1769),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1770),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1771),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1772),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1773),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1774),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1775),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1776),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1777),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1778),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1779),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1780),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1781),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1782),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1783),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1784),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1785),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1786),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1787),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1788),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1789),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1790),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1791),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1792),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1793),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1794),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1795),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1796),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1797),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1798),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1799),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1800),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1801),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1802),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1803),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1804),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1805),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1806),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1807),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1808),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1809),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1810),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1811),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1812),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1813),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1814),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1815),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1816),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1817),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1818),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1819),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1820),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1821),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1822),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1823),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1824),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1825),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1826),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1827),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1828),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1829),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1830),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1831),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1832),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1833),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1834),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1835),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1836),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1837),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1838),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1839),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1840),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1841),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1842),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1843),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1844),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1845),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1846),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1847),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1848),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1849),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1850),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1851),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1852),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1853),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1854),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1855),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1856),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1857),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1858),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1859),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1860),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1861),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1862),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1863),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1864),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1865),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1866),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1867),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1868),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1869),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1870),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1871),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1872),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1873),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1874),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1875),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1876),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1877),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1878),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1879),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1880),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1881),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1882),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1883),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1884),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1885),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1886),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1887),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1888),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1889),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1890),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1891),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1892),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1893),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1894),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1895),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1896),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1897),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1898),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1899),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1900),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1901),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1902),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1903),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1904),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1905),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1906),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1907),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1908),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1909),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1910),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1911),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1912),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1913),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1914),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1915),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1916),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1917),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1918),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1919),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1920),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1921),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1922),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1923),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1924),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1925),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1926),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1927),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1928),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1929),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1930),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1931),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1932),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1933),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1934),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1935),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1936),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1937),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1938),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1939),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1940),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1941),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1942),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1943),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1944),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1945),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1946),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1947),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1948),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1949),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1950),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1951),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1952),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1953),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1954),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1955),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1956),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1957),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1958),
    (CategoryData : 32; CCC : 0; NumericIndex : 0; SimpleUpperCase : (byte0 : $00; byte1 : $00; byte2 : $00;); SimpleLowerCase : (byte0 : $00; byte1 : $00; byte2 : $00;); DecompositionID : 1959)
  );

const
  UC_DEC_BOOK_INDEX_LENGTH = 1960;
  UC_DEC_BOOK_DATA_LENGTH = 2985;
type
  TDecompositionIndexRec = packed record
    StartPosition : Word;
    Length        : Byte;
  end;
  TDecompositionBookRec = packed record
    Index      : array[0..(UC_DEC_BOOK_INDEX_LENGTH-1)] of TDecompositionIndexRec;
    CodePoints : array[0..(UC_DEC_BOOK_DATA_LENGTH-1)] of UInt24;
  end;
const
  UC_DEC_BOOK_DATA : TDecompositionBookRec = (
    Index : (// Index BEGIN
      (StartPosition : 0; Length : 2), (StartPosition : 2; Length : 2), 
      (StartPosition : 4; Length : 2), (StartPosition : 6; Length : 2), 
      (StartPosition : 8; Length : 2), (StartPosition : 10; Length : 2), 
      (StartPosition : 12; Length : 2), (StartPosition : 14; Length : 2), 
      (StartPosition : 16; Length : 2), (StartPosition : 18; Length : 2), 
      (StartPosition : 20; Length : 2), (StartPosition : 22; Length : 2), 
      (StartPosition : 24; Length : 2), (StartPosition : 26; Length : 2), 
      (StartPosition : 28; Length : 2), (StartPosition : 30; Length : 2), 
      (StartPosition : 32; Length : 2), (StartPosition : 34; Length : 2), 
      (StartPosition : 36; Length : 2), (StartPosition : 38; Length : 2), 
      (StartPosition : 40; Length : 2), (StartPosition : 42; Length : 2), 
      (StartPosition : 44; Length : 2), (StartPosition : 46; Length : 2), 
      (StartPosition : 48; Length : 2), (StartPosition : 50; Length : 2), 
      (StartPosition : 52; Length : 2), (StartPosition : 54; Length : 2), 
      (StartPosition : 56; Length : 2), (StartPosition : 58; Length : 2), 
      (StartPosition : 60; Length : 2), (StartPosition : 62; Length : 2), 
      (StartPosition : 64; Length : 2), (StartPosition : 66; Length : 2), 
      (StartPosition : 68; Length : 2), (StartPosition : 70; Length : 2), 
      (StartPosition : 72; Length : 2), (StartPosition : 74; Length : 2), 
      (StartPosition : 76; Length : 2), (StartPosition : 78; Length : 2), 
      (StartPosition : 80; Length : 2), (StartPosition : 82; Length : 2), 
      (StartPosition : 84; Length : 2), (StartPosition : 86; Length : 2), 
      (StartPosition : 88; Length : 2), (StartPosition : 90; Length : 2), 
      (StartPosition : 92; Length : 2), (StartPosition : 94; Length : 2), 
      (StartPosition : 96; Length : 2), (StartPosition : 98; Length : 2), 
      (StartPosition : 100; Length : 2), (StartPosition : 102; Length : 2), 
      (StartPosition : 104; Length : 2), (StartPosition : 106; Length : 2), 
      (StartPosition : 108; Length : 2), (StartPosition : 110; Length : 2), 
      (StartPosition : 112; Length : 2), (StartPosition : 114; Length : 2), 
      (StartPosition : 116; Length : 2), (StartPosition : 118; Length : 2), 
      (StartPosition : 120; Length : 2), (StartPosition : 122; Length : 2), 
      (StartPosition : 124; Length : 2), (StartPosition : 126; Length : 2), 
      (StartPosition : 128; Length : 2), (StartPosition : 130; Length : 2), 
      (StartPosition : 132; Length : 2), (StartPosition : 134; Length : 2), 
      (StartPosition : 136; Length : 2), (StartPosition : 138; Length : 2), 
      (StartPosition : 140; Length : 2), (StartPosition : 142; Length : 2), 
      (StartPosition : 144; Length : 2), (StartPosition : 146; Length : 2), 
      (StartPosition : 148; Length : 2), (StartPosition : 150; Length : 2), 
      (StartPosition : 152; Length : 2), (StartPosition : 154; Length : 2), 
      (StartPosition : 156; Length : 2), (StartPosition : 158; Length : 2), 
      (StartPosition : 160; Length : 2), (StartPosition : 162; Length : 2), 
      (StartPosition : 164; Length : 2), (StartPosition : 166; Length : 2), 
      (StartPosition : 168; Length : 2), (StartPosition : 170; Length : 2), 
      (StartPosition : 172; Length : 2), (StartPosition : 174; Length : 2), 
      (StartPosition : 176; Length : 2), (StartPosition : 178; Length : 2), 
      (StartPosition : 180; Length : 2), (StartPosition : 182; Length : 2), 
      (StartPosition : 184; Length : 2), (StartPosition : 186; Length : 2), 
      (StartPosition : 188; Length : 2), (StartPosition : 190; Length : 2), 
      (StartPosition : 192; Length : 2), (StartPosition : 194; Length : 2), 
      (StartPosition : 196; Length : 2), (StartPosition : 198; Length : 2), 
      (StartPosition : 200; Length : 2), (StartPosition : 202; Length : 2), 
      (StartPosition : 204; Length : 2), (StartPosition : 206; Length : 2), 
      (StartPosition : 208; Length : 2), (StartPosition : 210; Length : 2), 
      (StartPosition : 212; Length : 2), (StartPosition : 214; Length : 2), 
      (StartPosition : 216; Length : 2), (StartPosition : 218; Length : 2), 
      (StartPosition : 220; Length : 2), (StartPosition : 222; Length : 2), 
      (StartPosition : 224; Length : 2), (StartPosition : 226; Length : 2), 
      (StartPosition : 228; Length : 2), (StartPosition : 230; Length : 2), 
      (StartPosition : 232; Length : 2), (StartPosition : 234; Length : 2), 
      (StartPosition : 236; Length : 2), (StartPosition : 238; Length : 2), 
      (StartPosition : 240; Length : 2), (StartPosition : 242; Length : 2), 
      (StartPosition : 244; Length : 2), (StartPosition : 246; Length : 2), 
      (StartPosition : 248; Length : 2), (StartPosition : 250; Length : 2), 
      (StartPosition : 252; Length : 2), (StartPosition : 254; Length : 2), 
      (StartPosition : 256; Length : 2), (StartPosition : 258; Length : 2), 
      (StartPosition : 260; Length : 2), (StartPosition : 262; Length : 2), 
      (StartPosition : 264; Length : 2), (StartPosition : 266; Length : 2), 
      (StartPosition : 268; Length : 2), (StartPosition : 270; Length : 2), 
      (StartPosition : 272; Length : 2), (StartPosition : 274; Length : 2), 
      (StartPosition : 276; Length : 2), (StartPosition : 278; Length : 2), 
      (StartPosition : 280; Length : 2), (StartPosition : 282; Length : 2), 
      (StartPosition : 284; Length : 2), (StartPosition : 286; Length : 2), 
      (StartPosition : 288; Length : 2), (StartPosition : 290; Length : 2), 
      (StartPosition : 292; Length : 2), (StartPosition : 294; Length : 2), 
      (StartPosition : 296; Length : 2), (StartPosition : 298; Length : 2), 
      (StartPosition : 300; Length : 2), (StartPosition : 302; Length : 2), 
      (StartPosition : 304; Length : 2), (StartPosition : 306; Length : 2), 
      (StartPosition : 308; Length : 2), (StartPosition : 310; Length : 2), 
      (StartPosition : 312; Length : 2), (StartPosition : 314; Length : 2), 
      (StartPosition : 316; Length : 2), (StartPosition : 318; Length : 2), 
      (StartPosition : 320; Length : 2), (StartPosition : 322; Length : 2), 
      (StartPosition : 324; Length : 2), (StartPosition : 326; Length : 2), 
      (StartPosition : 328; Length : 2), (StartPosition : 330; Length : 2), 
      (StartPosition : 332; Length : 2), (StartPosition : 334; Length : 2), 
      (StartPosition : 336; Length : 2), (StartPosition : 338; Length : 2), 
      (StartPosition : 340; Length : 2), (StartPosition : 342; Length : 2), 
      (StartPosition : 344; Length : 2), (StartPosition : 346; Length : 2), 
      (StartPosition : 348; Length : 2), (StartPosition : 350; Length : 2), 
      (StartPosition : 352; Length : 2), (StartPosition : 354; Length : 2), 
      (StartPosition : 356; Length : 2), (StartPosition : 358; Length : 2), 
      (StartPosition : 360; Length : 2), (StartPosition : 362; Length : 2), 
      (StartPosition : 364; Length : 2), (StartPosition : 366; Length : 2), 
      (StartPosition : 368; Length : 2), (StartPosition : 370; Length : 2), 
      (StartPosition : 372; Length : 2), (StartPosition : 374; Length : 2), 
      (StartPosition : 376; Length : 2), (StartPosition : 378; Length : 2), 
      (StartPosition : 380; Length : 2), (StartPosition : 382; Length : 2), 
      (StartPosition : 384; Length : 2), (StartPosition : 386; Length : 2), 
      (StartPosition : 388; Length : 2), (StartPosition : 390; Length : 2), 
      (StartPosition : 392; Length : 2), (StartPosition : 394; Length : 2), 
      (StartPosition : 396; Length : 2), (StartPosition : 398; Length : 2), 
      (StartPosition : 400; Length : 2), (StartPosition : 402; Length : 2), 
      (StartPosition : 404; Length : 2), (StartPosition : 406; Length : 2), 
      (StartPosition : 408; Length : 2), (StartPosition : 410; Length : 2), 
      (StartPosition : 412; Length : 2), (StartPosition : 414; Length : 2), 
      (StartPosition : 416; Length : 2), (StartPosition : 418; Length : 2), 
      (StartPosition : 420; Length : 2), (StartPosition : 422; Length : 2), 
      (StartPosition : 424; Length : 2), (StartPosition : 426; Length : 2), 
      (StartPosition : 428; Length : 2), (StartPosition : 430; Length : 2), 
      (StartPosition : 432; Length : 2), (StartPosition : 434; Length : 2), 
      (StartPosition : 436; Length : 2), (StartPosition : 438; Length : 2), 
      (StartPosition : 440; Length : 2), (StartPosition : 442; Length : 2), 
      (StartPosition : 444; Length : 2), (StartPosition : 446; Length : 2), 
      (StartPosition : 448; Length : 2), (StartPosition : 450; Length : 2), 
      (StartPosition : 452; Length : 2), (StartPosition : 454; Length : 2), 
      (StartPosition : 456; Length : 2), (StartPosition : 458; Length : 2), 
      (StartPosition : 460; Length : 2), (StartPosition : 462; Length : 2), 
      (StartPosition : 464; Length : 2), (StartPosition : 466; Length : 2), 
      (StartPosition : 468; Length : 2), (StartPosition : 470; Length : 2), 
      (StartPosition : 472; Length : 2), (StartPosition : 474; Length : 2), 
      (StartPosition : 476; Length : 2), (StartPosition : 478; Length : 2), 
      (StartPosition : 480; Length : 2), (StartPosition : 482; Length : 2), 
      (StartPosition : 484; Length : 2), (StartPosition : 486; Length : 2), 
      (StartPosition : 488; Length : 2), (StartPosition : 490; Length : 2), 
      (StartPosition : 492; Length : 2), (StartPosition : 494; Length : 2), 
      (StartPosition : 496; Length : 2), (StartPosition : 498; Length : 2), 
      (StartPosition : 500; Length : 2), (StartPosition : 502; Length : 2), 
      (StartPosition : 504; Length : 1), (StartPosition : 505; Length : 1), 
      (StartPosition : 506; Length : 1), (StartPosition : 507; Length : 2), 
      (StartPosition : 509; Length : 1), (StartPosition : 510; Length : 1), 
      (StartPosition : 511; Length : 2), (StartPosition : 513; Length : 2), 
      (StartPosition : 515; Length : 1), (StartPosition : 516; Length : 2), 
      (StartPosition : 518; Length : 2), (StartPosition : 520; Length : 2), 
      (StartPosition : 522; Length : 2), (StartPosition : 524; Length : 2), 
      (StartPosition : 526; Length : 2), (StartPosition : 528; Length : 2), 
      (StartPosition : 530; Length : 2), (StartPosition : 532; Length : 2), 
      (StartPosition : 534; Length : 2), (StartPosition : 536; Length : 2), 
      (StartPosition : 538; Length : 2), (StartPosition : 540; Length : 2), 
      (StartPosition : 542; Length : 2), (StartPosition : 544; Length : 2), 
      (StartPosition : 546; Length : 2), (StartPosition : 548; Length : 2), 
      (StartPosition : 550; Length : 2), (StartPosition : 552; Length : 2), 
      (StartPosition : 554; Length : 2), (StartPosition : 556; Length : 2), 
      (StartPosition : 558; Length : 2), (StartPosition : 560; Length : 2), 
      (StartPosition : 562; Length : 2), (StartPosition : 564; Length : 2), 
      (StartPosition : 566; Length : 2), (StartPosition : 568; Length : 2), 
      (StartPosition : 570; Length : 2), (StartPosition : 572; Length : 2), 
      (StartPosition : 574; Length : 2), (StartPosition : 576; Length : 2), 
      (StartPosition : 578; Length : 2), (StartPosition : 580; Length : 2), 
      (StartPosition : 582; Length : 2), (StartPosition : 584; Length : 2), 
      (StartPosition : 586; Length : 2), (StartPosition : 588; Length : 2), 
      (StartPosition : 590; Length : 2), (StartPosition : 592; Length : 2), 
      (StartPosition : 594; Length : 2), (StartPosition : 596; Length : 2), 
      (StartPosition : 598; Length : 2), (StartPosition : 600; Length : 2), 
      (StartPosition : 602; Length : 2), (StartPosition : 604; Length : 2), 
      (StartPosition : 606; Length : 2), (StartPosition : 608; Length : 2), 
      (StartPosition : 610; Length : 2), (StartPosition : 612; Length : 2), 
      (StartPosition : 614; Length : 2), (StartPosition : 616; Length : 2), 
      (StartPosition : 618; Length : 2), (StartPosition : 620; Length : 2), 
      (StartPosition : 622; Length : 2), (StartPosition : 624; Length : 2), 
      (StartPosition : 626; Length : 2), (StartPosition : 628; Length : 2), 
      (StartPosition : 630; Length : 2), (StartPosition : 632; Length : 2), 
      (StartPosition : 634; Length : 2), (StartPosition : 636; Length : 2), 
      (StartPosition : 638; Length : 2), (StartPosition : 640; Length : 2), 
      (StartPosition : 642; Length : 2), (StartPosition : 644; Length : 2), 
      (StartPosition : 646; Length : 2), (StartPosition : 648; Length : 2), 
      (StartPosition : 650; Length : 2), (StartPosition : 652; Length : 2), 
      (StartPosition : 654; Length : 2), (StartPosition : 656; Length : 2), 
      (StartPosition : 658; Length : 2), (StartPosition : 660; Length : 2), 
      (StartPosition : 662; Length : 2), (StartPosition : 664; Length : 2), 
      (StartPosition : 666; Length : 2), (StartPosition : 668; Length : 2), 
      (StartPosition : 670; Length : 2), (StartPosition : 672; Length : 2), 
      (StartPosition : 674; Length : 2), (StartPosition : 676; Length : 2), 
      (StartPosition : 678; Length : 2), (StartPosition : 680; Length : 2), 
      (StartPosition : 682; Length : 2), (StartPosition : 684; Length : 2), 
      (StartPosition : 686; Length : 2), (StartPosition : 688; Length : 2), 
      (StartPosition : 690; Length : 2), (StartPosition : 692; Length : 2), 
      (StartPosition : 694; Length : 2), (StartPosition : 696; Length : 2), 
      (StartPosition : 698; Length : 2), (StartPosition : 700; Length : 2), 
      (StartPosition : 702; Length : 2), (StartPosition : 704; Length : 2), 
      (StartPosition : 706; Length : 2), (StartPosition : 708; Length : 2), 
      (StartPosition : 710; Length : 2), (StartPosition : 712; Length : 2), 
      (StartPosition : 714; Length : 2), (StartPosition : 716; Length : 2), 
      (StartPosition : 718; Length : 2), (StartPosition : 720; Length : 2), 
      (StartPosition : 722; Length : 2), (StartPosition : 724; Length : 2), 
      (StartPosition : 726; Length : 2), (StartPosition : 728; Length : 2), 
      (StartPosition : 730; Length : 2), (StartPosition : 732; Length : 2), 
      (StartPosition : 734; Length : 2), (StartPosition : 736; Length : 2), 
      (StartPosition : 738; Length : 2), (StartPosition : 740; Length : 2), 
      (StartPosition : 742; Length : 2), (StartPosition : 744; Length : 2), 
      (StartPosition : 746; Length : 2), (StartPosition : 748; Length : 2), 
      (StartPosition : 750; Length : 2), (StartPosition : 752; Length : 2), 
      (StartPosition : 754; Length : 2), (StartPosition : 756; Length : 2), 
      (StartPosition : 758; Length : 2), (StartPosition : 760; Length : 2), 
      (StartPosition : 762; Length : 2), (StartPosition : 764; Length : 2), 
      (StartPosition : 766; Length : 2), (StartPosition : 768; Length : 2), 
      (StartPosition : 770; Length : 2), (StartPosition : 772; Length : 2), 
      (StartPosition : 774; Length : 2), (StartPosition : 776; Length : 2), 
      (StartPosition : 778; Length : 2), (StartPosition : 780; Length : 2), 
      (StartPosition : 782; Length : 2), (StartPosition : 784; Length : 2), 
      (StartPosition : 786; Length : 2), (StartPosition : 788; Length : 2), 
      (StartPosition : 790; Length : 2), (StartPosition : 792; Length : 2), 
      (StartPosition : 794; Length : 2), (StartPosition : 796; Length : 2), 
      (StartPosition : 798; Length : 2), (StartPosition : 800; Length : 2), 
      (StartPosition : 802; Length : 2), (StartPosition : 804; Length : 2), 
      (StartPosition : 806; Length : 2), (StartPosition : 808; Length : 2), 
      (StartPosition : 810; Length : 2), (StartPosition : 812; Length : 2), 
      (StartPosition : 814; Length : 2), (StartPosition : 816; Length : 2), 
      (StartPosition : 818; Length : 2), (StartPosition : 820; Length : 2), 
      (StartPosition : 822; Length : 2), (StartPosition : 824; Length : 2), 
      (StartPosition : 826; Length : 2), (StartPosition : 828; Length : 2), 
      (StartPosition : 830; Length : 2), (StartPosition : 832; Length : 2), 
      (StartPosition : 834; Length : 2), (StartPosition : 836; Length : 2), 
      (StartPosition : 838; Length : 2), (StartPosition : 840; Length : 2), 
      (StartPosition : 842; Length : 2), (StartPosition : 844; Length : 2), 
      (StartPosition : 846; Length : 2), (StartPosition : 848; Length : 2), 
      (StartPosition : 850; Length : 2), (StartPosition : 852; Length : 2), 
      (StartPosition : 854; Length : 2), (StartPosition : 856; Length : 2), 
      (StartPosition : 858; Length : 2), (StartPosition : 860; Length : 2), 
      (StartPosition : 862; Length : 2), (StartPosition : 864; Length : 2), 
      (StartPosition : 866; Length : 2), (StartPosition : 868; Length : 2), 
      (StartPosition : 870; Length : 2), (StartPosition : 872; Length : 2), 
      (StartPosition : 874; Length : 2), (StartPosition : 876; Length : 2), 
      (StartPosition : 878; Length : 2), (StartPosition : 880; Length : 2), 
      (StartPosition : 882; Length : 2), (StartPosition : 884; Length : 2), 
      (StartPosition : 886; Length : 2), (StartPosition : 888; Length : 2), 
      (StartPosition : 890; Length : 2), (StartPosition : 892; Length : 2), 
      (StartPosition : 894; Length : 2), (StartPosition : 896; Length : 2), 
      (StartPosition : 898; Length : 2), (StartPosition : 900; Length : 2), 
      (StartPosition : 902; Length : 2), (StartPosition : 904; Length : 2), 
      (StartPosition : 906; Length : 2), (StartPosition : 908; Length : 2), 
      (StartPosition : 910; Length : 2), (StartPosition : 912; Length : 2), 
      (StartPosition : 914; Length : 2), (StartPosition : 916; Length : 2), 
      (StartPosition : 918; Length : 2), (StartPosition : 920; Length : 2), 
      (StartPosition : 922; Length : 2), (StartPosition : 924; Length : 2), 
      (StartPosition : 926; Length : 2), (StartPosition : 928; Length : 2), 
      (StartPosition : 930; Length : 2), (StartPosition : 932; Length : 2), 
      (StartPosition : 934; Length : 2), (StartPosition : 936; Length : 2), 
      (StartPosition : 938; Length : 2), (StartPosition : 940; Length : 2), 
      (StartPosition : 942; Length : 2), (StartPosition : 944; Length : 2), 
      (StartPosition : 946; Length : 2), (StartPosition : 948; Length : 2), 
      (StartPosition : 950; Length : 2), (StartPosition : 952; Length : 2), 
      (StartPosition : 954; Length : 2), (StartPosition : 956; Length : 2), 
      (StartPosition : 958; Length : 2), (StartPosition : 960; Length : 2), 
      (StartPosition : 962; Length : 2), (StartPosition : 964; Length : 2), 
      (StartPosition : 966; Length : 2), (StartPosition : 968; Length : 2), 
      (StartPosition : 970; Length : 2), (StartPosition : 972; Length : 2), 
      (StartPosition : 974; Length : 2), (StartPosition : 976; Length : 2), 
      (StartPosition : 978; Length : 2), (StartPosition : 980; Length : 2), 
      (StartPosition : 982; Length : 2), (StartPosition : 984; Length : 2), 
      (StartPosition : 986; Length : 2), (StartPosition : 988; Length : 2), 
      (StartPosition : 990; Length : 2), (StartPosition : 992; Length : 2), 
      (StartPosition : 994; Length : 2), (StartPosition : 996; Length : 2), 
      (StartPosition : 998; Length : 2), (StartPosition : 1000; Length : 2), 
      (StartPosition : 1002; Length : 2), (StartPosition : 1004; Length : 2), 
      (StartPosition : 1006; Length : 2), (StartPosition : 1008; Length : 2), 
      (StartPosition : 1010; Length : 2), (StartPosition : 1012; Length : 2), 
      (StartPosition : 1014; Length : 2), (StartPosition : 1016; Length : 2), 
      (StartPosition : 1018; Length : 2), (StartPosition : 1020; Length : 2), 
      (StartPosition : 1022; Length : 2), (StartPosition : 1024; Length : 2), 
      (StartPosition : 1026; Length : 2), (StartPosition : 1028; Length : 2), 
      (StartPosition : 1030; Length : 2), (StartPosition : 1032; Length : 2), 
      (StartPosition : 1034; Length : 2), (StartPosition : 1036; Length : 2), 
      (StartPosition : 1038; Length : 2), (StartPosition : 1040; Length : 2), 
      (StartPosition : 1042; Length : 2), (StartPosition : 1044; Length : 2), 
      (StartPosition : 1046; Length : 2), (StartPosition : 1048; Length : 2), 
      (StartPosition : 1050; Length : 2), (StartPosition : 1052; Length : 2), 
      (StartPosition : 1054; Length : 2), (StartPosition : 1056; Length : 2), 
      (StartPosition : 1058; Length : 2), (StartPosition : 1060; Length : 2), 
      (StartPosition : 1062; Length : 2), (StartPosition : 1064; Length : 2), 
      (StartPosition : 1066; Length : 2), (StartPosition : 1068; Length : 2), 
      (StartPosition : 1070; Length : 2), (StartPosition : 1072; Length : 2), 
      (StartPosition : 1074; Length : 2), (StartPosition : 1076; Length : 2), 
      (StartPosition : 1078; Length : 2), (StartPosition : 1080; Length : 2), 
      (StartPosition : 1082; Length : 2), (StartPosition : 1084; Length : 2), 
      (StartPosition : 1086; Length : 2), (StartPosition : 1088; Length : 2), 
      (StartPosition : 1090; Length : 2), (StartPosition : 1092; Length : 2), 
      (StartPosition : 1094; Length : 2), (StartPosition : 1096; Length : 2), 
      (StartPosition : 1098; Length : 2), (StartPosition : 1100; Length : 2), 
      (StartPosition : 1102; Length : 2), (StartPosition : 1104; Length : 2), 
      (StartPosition : 1106; Length : 2), (StartPosition : 1108; Length : 2), 
      (StartPosition : 1110; Length : 2), (StartPosition : 1112; Length : 2), 
      (StartPosition : 1114; Length : 2), (StartPosition : 1116; Length : 2), 
      (StartPosition : 1118; Length : 2), (StartPosition : 1120; Length : 2), 
      (StartPosition : 1122; Length : 2), (StartPosition : 1124; Length : 2), 
      (StartPosition : 1126; Length : 2), (StartPosition : 1128; Length : 2), 
      (StartPosition : 1130; Length : 2), (StartPosition : 1132; Length : 2), 
      (StartPosition : 1134; Length : 2), (StartPosition : 1136; Length : 2), 
      (StartPosition : 1138; Length : 2), (StartPosition : 1140; Length : 2), 
      (StartPosition : 1142; Length : 2), (StartPosition : 1144; Length : 2), 
      (StartPosition : 1146; Length : 2), (StartPosition : 1148; Length : 2), 
      (StartPosition : 1150; Length : 2), (StartPosition : 1152; Length : 2), 
      (StartPosition : 1154; Length : 2), (StartPosition : 1156; Length : 2), 
      (StartPosition : 1158; Length : 2), (StartPosition : 1160; Length : 2), 
      (StartPosition : 1162; Length : 2), (StartPosition : 1164; Length : 2), 
      (StartPosition : 1166; Length : 2), (StartPosition : 1168; Length : 2), 
      (StartPosition : 1170; Length : 2), (StartPosition : 1172; Length : 2), 
      (StartPosition : 1174; Length : 2), (StartPosition : 1176; Length : 2), 
      (StartPosition : 1178; Length : 2), (StartPosition : 1180; Length : 2), 
      (StartPosition : 1182; Length : 2), (StartPosition : 1184; Length : 2), 
      (StartPosition : 1186; Length : 2), (StartPosition : 1188; Length : 2), 
      (StartPosition : 1190; Length : 2), (StartPosition : 1192; Length : 2), 
      (StartPosition : 1194; Length : 2), (StartPosition : 1196; Length : 2), 
      (StartPosition : 1198; Length : 2), (StartPosition : 1200; Length : 2), 
      (StartPosition : 1202; Length : 2), (StartPosition : 1204; Length : 2), 
      (StartPosition : 1206; Length : 2), (StartPosition : 1208; Length : 2), 
      (StartPosition : 1210; Length : 2), (StartPosition : 1212; Length : 2), 
      (StartPosition : 1214; Length : 2), (StartPosition : 1216; Length : 2), 
      (StartPosition : 1218; Length : 2), (StartPosition : 1220; Length : 2), 
      (StartPosition : 1222; Length : 2), (StartPosition : 1224; Length : 2), 
      (StartPosition : 1226; Length : 2), (StartPosition : 1228; Length : 2), 
      (StartPosition : 1230; Length : 2), (StartPosition : 1232; Length : 2), 
      (StartPosition : 1234; Length : 2), (StartPosition : 1236; Length : 2), 
      (StartPosition : 1238; Length : 2), (StartPosition : 1240; Length : 2), 
      (StartPosition : 1242; Length : 2), (StartPosition : 1244; Length : 2), 
      (StartPosition : 1246; Length : 2), (StartPosition : 1248; Length : 2), 
      (StartPosition : 1250; Length : 2), (StartPosition : 1252; Length : 2), 
      (StartPosition : 1254; Length : 2), (StartPosition : 1256; Length : 2), 
      (StartPosition : 1258; Length : 2), (StartPosition : 1260; Length : 2), 
      (StartPosition : 1262; Length : 2), (StartPosition : 1264; Length : 2), 
      (StartPosition : 1266; Length : 2), (StartPosition : 1268; Length : 2), 
      (StartPosition : 1270; Length : 2), (StartPosition : 1272; Length : 2), 
      (StartPosition : 1274; Length : 2), (StartPosition : 1276; Length : 2), 
      (StartPosition : 1278; Length : 2), (StartPosition : 1280; Length : 2), 
      (StartPosition : 1282; Length : 2), (StartPosition : 1284; Length : 2), 
      (StartPosition : 1286; Length : 2), (StartPosition : 1288; Length : 2), 
      (StartPosition : 1290; Length : 2), (StartPosition : 1292; Length : 2), 
      (StartPosition : 1294; Length : 2), (StartPosition : 1296; Length : 2), 
      (StartPosition : 1298; Length : 2), (StartPosition : 1300; Length : 2), 
      (StartPosition : 1302; Length : 2), (StartPosition : 1304; Length : 2), 
      (StartPosition : 1306; Length : 2), (StartPosition : 1308; Length : 2), 
      (StartPosition : 1310; Length : 2), (StartPosition : 1312; Length : 2), 
      (StartPosition : 1314; Length : 2), (StartPosition : 1316; Length : 2), 
      (StartPosition : 1318; Length : 2), (StartPosition : 1320; Length : 2), 
      (StartPosition : 1322; Length : 2), (StartPosition : 1324; Length : 2), 
      (StartPosition : 1326; Length : 2), (StartPosition : 1328; Length : 2), 
      (StartPosition : 1330; Length : 2), (StartPosition : 1332; Length : 2), 
      (StartPosition : 1334; Length : 2), (StartPosition : 1336; Length : 2), 
      (StartPosition : 1338; Length : 2), (StartPosition : 1340; Length : 2), 
      (StartPosition : 1342; Length : 2), (StartPosition : 1344; Length : 2), 
      (StartPosition : 1346; Length : 2), (StartPosition : 1348; Length : 2), 
      (StartPosition : 1350; Length : 2), (StartPosition : 1352; Length : 2), 
      (StartPosition : 1354; Length : 2), (StartPosition : 1356; Length : 2), 
      (StartPosition : 1358; Length : 2), (StartPosition : 1360; Length : 2), 
      (StartPosition : 1362; Length : 2), (StartPosition : 1364; Length : 2), 
      (StartPosition : 1366; Length : 2), (StartPosition : 1368; Length : 2), 
      (StartPosition : 1370; Length : 2), (StartPosition : 1372; Length : 2), 
      (StartPosition : 1374; Length : 2), (StartPosition : 1376; Length : 2), 
      (StartPosition : 1378; Length : 2), (StartPosition : 1380; Length : 2), 
      (StartPosition : 1382; Length : 2), (StartPosition : 1384; Length : 2), 
      (StartPosition : 1386; Length : 2), (StartPosition : 1388; Length : 2), 
      (StartPosition : 1390; Length : 2), (StartPosition : 1392; Length : 2), 
      (StartPosition : 1394; Length : 2), (StartPosition : 1396; Length : 2), 
      (StartPosition : 1398; Length : 2), (StartPosition : 1400; Length : 2), 
      (StartPosition : 1402; Length : 2), (StartPosition : 1404; Length : 2), 
      (StartPosition : 1406; Length : 2), (StartPosition : 1408; Length : 2), 
      (StartPosition : 1410; Length : 2), (StartPosition : 1412; Length : 2), 
      (StartPosition : 1414; Length : 2), (StartPosition : 1416; Length : 2), 
      (StartPosition : 1418; Length : 2), (StartPosition : 1420; Length : 2), 
      (StartPosition : 1422; Length : 2), (StartPosition : 1424; Length : 2), 
      (StartPosition : 1426; Length : 2), (StartPosition : 1428; Length : 2), 
      (StartPosition : 1430; Length : 2), (StartPosition : 1432; Length : 2), 
      (StartPosition : 1434; Length : 2), (StartPosition : 1436; Length : 2), 
      (StartPosition : 1438; Length : 2), (StartPosition : 1440; Length : 2), 
      (StartPosition : 1442; Length : 2), (StartPosition : 1444; Length : 2), 
      (StartPosition : 1446; Length : 2), (StartPosition : 1448; Length : 2), 
      (StartPosition : 1450; Length : 2), (StartPosition : 1452; Length : 2), 
      (StartPosition : 1454; Length : 2), (StartPosition : 1456; Length : 2), 
      (StartPosition : 1458; Length : 2), (StartPosition : 1460; Length : 2), 
      (StartPosition : 1462; Length : 2), (StartPosition : 1464; Length : 2), 
      (StartPosition : 1466; Length : 2), (StartPosition : 1468; Length : 2), 
      (StartPosition : 1470; Length : 2), (StartPosition : 1472; Length : 2), 
      (StartPosition : 1474; Length : 2), (StartPosition : 1476; Length : 2), 
      (StartPosition : 1478; Length : 2), (StartPosition : 1480; Length : 2), 
      (StartPosition : 1482; Length : 2), (StartPosition : 1484; Length : 2), 
      (StartPosition : 1486; Length : 2), (StartPosition : 1488; Length : 2), 
      (StartPosition : 1490; Length : 2), (StartPosition : 1492; Length : 2), 
      (StartPosition : 1494; Length : 2), (StartPosition : 1496; Length : 2), 
      (StartPosition : 1498; Length : 2), (StartPosition : 1500; Length : 2), 
      (StartPosition : 1502; Length : 2), (StartPosition : 1504; Length : 2), 
      (StartPosition : 1506; Length : 2), (StartPosition : 1508; Length : 2), 
      (StartPosition : 1510; Length : 2), (StartPosition : 1512; Length : 2), 
      (StartPosition : 1514; Length : 2), (StartPosition : 1516; Length : 1), 
      (StartPosition : 1517; Length : 2), (StartPosition : 1519; Length : 1), 
      (StartPosition : 1520; Length : 2), (StartPosition : 1522; Length : 1), 
      (StartPosition : 1523; Length : 2), (StartPosition : 1525; Length : 1), 
      (StartPosition : 1526; Length : 2), (StartPosition : 1528; Length : 1), 
      (StartPosition : 1529; Length : 2), (StartPosition : 1531; Length : 1), 
      (StartPosition : 1532; Length : 2), (StartPosition : 1534; Length : 1), 
      (StartPosition : 1535; Length : 2), (StartPosition : 1537; Length : 2), 
      (StartPosition : 1539; Length : 2), (StartPosition : 1541; Length : 2), 
      (StartPosition : 1543; Length : 2), (StartPosition : 1545; Length : 2), 
      (StartPosition : 1547; Length : 2), (StartPosition : 1549; Length : 2), 
      (StartPosition : 1551; Length : 2), (StartPosition : 1553; Length : 2), 
      (StartPosition : 1555; Length : 2), (StartPosition : 1557; Length : 2), 
      (StartPosition : 1559; Length : 2), (StartPosition : 1561; Length : 2), 
      (StartPosition : 1563; Length : 2), (StartPosition : 1565; Length : 2), 
      (StartPosition : 1567; Length : 2), (StartPosition : 1569; Length : 2), 
      (StartPosition : 1571; Length : 2), (StartPosition : 1573; Length : 2), 
      (StartPosition : 1575; Length : 2), (StartPosition : 1577; Length : 2), 
      (StartPosition : 1579; Length : 2), (StartPosition : 1581; Length : 2), 
      (StartPosition : 1583; Length : 2), (StartPosition : 1585; Length : 2), 
      (StartPosition : 1587; Length : 2), (StartPosition : 1589; Length : 2), 
      (StartPosition : 1591; Length : 2), (StartPosition : 1593; Length : 2), 
      (StartPosition : 1595; Length : 2), (StartPosition : 1597; Length : 2), 
      (StartPosition : 1599; Length : 2), (StartPosition : 1601; Length : 2), 
      (StartPosition : 1603; Length : 2), (StartPosition : 1605; Length : 2), 
      (StartPosition : 1607; Length : 2), (StartPosition : 1609; Length : 2), 
      (StartPosition : 1611; Length : 2), (StartPosition : 1613; Length : 2), 
      (StartPosition : 1615; Length : 2), (StartPosition : 1617; Length : 2), 
      (StartPosition : 1619; Length : 2), (StartPosition : 1621; Length : 2), 
      (StartPosition : 1623; Length : 2), (StartPosition : 1625; Length : 2), 
      (StartPosition : 1627; Length : 2), (StartPosition : 1629; Length : 2), 
      (StartPosition : 1631; Length : 2), (StartPosition : 1633; Length : 2), 
      (StartPosition : 1635; Length : 2), (StartPosition : 1637; Length : 2), 
      (StartPosition : 1639; Length : 2), (StartPosition : 1641; Length : 2), 
      (StartPosition : 1643; Length : 2), (StartPosition : 1645; Length : 2), 
      (StartPosition : 1647; Length : 2), (StartPosition : 1649; Length : 2), 
      (StartPosition : 1651; Length : 1), (StartPosition : 1652; Length : 2), 
      (StartPosition : 1654; Length : 1), (StartPosition : 1655; Length : 2), 
      (StartPosition : 1657; Length : 2), (StartPosition : 1659; Length : 2), 
      (StartPosition : 1661; Length : 2), (StartPosition : 1663; Length : 2), 
      (StartPosition : 1665; Length : 2), (StartPosition : 1667; Length : 2), 
      (StartPosition : 1669; Length : 1), (StartPosition : 1670; Length : 2), 
      (StartPosition : 1672; Length : 1), (StartPosition : 1673; Length : 2), 
      (StartPosition : 1675; Length : 2), (StartPosition : 1677; Length : 2), 
      (StartPosition : 1679; Length : 2), (StartPosition : 1681; Length : 2), 
      (StartPosition : 1683; Length : 2), (StartPosition : 1685; Length : 2), 
      (StartPosition : 1687; Length : 1), (StartPosition : 1688; Length : 2), 
      (StartPosition : 1690; Length : 2), (StartPosition : 1692; Length : 2), 
      (StartPosition : 1694; Length : 2), (StartPosition : 1696; Length : 2), 
      (StartPosition : 1698; Length : 1), (StartPosition : 1699; Length : 2), 
      (StartPosition : 1701; Length : 2), (StartPosition : 1703; Length : 2), 
      (StartPosition : 1705; Length : 2), (StartPosition : 1707; Length : 2), 
      (StartPosition : 1709; Length : 2), (StartPosition : 1711; Length : 1), 
      (StartPosition : 1712; Length : 2), (StartPosition : 1714; Length : 2), 
      (StartPosition : 1716; Length : 2), (StartPosition : 1718; Length : 2), 
      (StartPosition : 1720; Length : 2), (StartPosition : 1722; Length : 2), 
      (StartPosition : 1724; Length : 2), (StartPosition : 1726; Length : 1), 
      (StartPosition : 1727; Length : 2), (StartPosition : 1729; Length : 2), 
      (StartPosition : 1731; Length : 1), (StartPosition : 1732; Length : 1), 
      (StartPosition : 1733; Length : 2), (StartPosition : 1735; Length : 2), 
      (StartPosition : 1737; Length : 2), (StartPosition : 1739; Length : 2), 
      (StartPosition : 1741; Length : 2), (StartPosition : 1743; Length : 2), 
      (StartPosition : 1745; Length : 1), (StartPosition : 1746; Length : 2), 
      (StartPosition : 1748; Length : 1), (StartPosition : 1749; Length : 2), 
      (StartPosition : 1751; Length : 1), (StartPosition : 1752; Length : 1), 
      (StartPosition : 1753; Length : 1), (StartPosition : 1754; Length : 1), 
      (StartPosition : 1755; Length : 1), (StartPosition : 1756; Length : 1), 
      (StartPosition : 1757; Length : 2), (StartPosition : 1759; Length : 2), 
      (StartPosition : 1761; Length : 2), (StartPosition : 1763; Length : 2), 
      (StartPosition : 1765; Length : 2), (StartPosition : 1767; Length : 2), 
      (StartPosition : 1769; Length : 2), (StartPosition : 1771; Length : 2), 
      (StartPosition : 1773; Length : 2), (StartPosition : 1775; Length : 2), 
      (StartPosition : 1777; Length : 2), (StartPosition : 1779; Length : 2), 
      (StartPosition : 1781; Length : 2), (StartPosition : 1783; Length : 2), 
      (StartPosition : 1785; Length : 2), (StartPosition : 1787; Length : 2), 
      (StartPosition : 1789; Length : 2), (StartPosition : 1791; Length : 2), 
      (StartPosition : 1793; Length : 2), (StartPosition : 1795; Length : 2), 
      (StartPosition : 1797; Length : 2), (StartPosition : 1799; Length : 2), 
      (StartPosition : 1801; Length : 2), (StartPosition : 1803; Length : 2), 
      (StartPosition : 1805; Length : 2), (StartPosition : 1807; Length : 2), 
      (StartPosition : 1809; Length : 2), (StartPosition : 1811; Length : 2), 
      (StartPosition : 1813; Length : 2), (StartPosition : 1815; Length : 2), 
      (StartPosition : 1817; Length : 2), (StartPosition : 1819; Length : 2), 
      (StartPosition : 1821; Length : 2), (StartPosition : 1823; Length : 2), 
      (StartPosition : 1825; Length : 2), (StartPosition : 1827; Length : 2), 
      (StartPosition : 1829; Length : 2), (StartPosition : 1831; Length : 2), 
      (StartPosition : 1833; Length : 2), (StartPosition : 1835; Length : 2), 
      (StartPosition : 1837; Length : 2), (StartPosition : 1839; Length : 2), 
      (StartPosition : 1841; Length : 2), (StartPosition : 1843; Length : 2), 
      (StartPosition : 1845; Length : 1), (StartPosition : 1846; Length : 1), 
      (StartPosition : 1847; Length : 2), (StartPosition : 1849; Length : 2), 
      (StartPosition : 1851; Length : 2), (StartPosition : 1853; Length : 2), 
      (StartPosition : 1855; Length : 2), (StartPosition : 1857; Length : 2), 
      (StartPosition : 1859; Length : 2), (StartPosition : 1861; Length : 2), 
      (StartPosition : 1863; Length : 2), (StartPosition : 1865; Length : 2), 
      (StartPosition : 1867; Length : 2), (StartPosition : 1869; Length : 2), 
      (StartPosition : 1871; Length : 2), (StartPosition : 1873; Length : 2), 
      (StartPosition : 1875; Length : 2), (StartPosition : 1877; Length : 2), 
      (StartPosition : 1879; Length : 2), (StartPosition : 1881; Length : 2), 
      (StartPosition : 1883; Length : 2), (StartPosition : 1885; Length : 2), 
      (StartPosition : 1887; Length : 2), (StartPosition : 1889; Length : 2), 
      (StartPosition : 1891; Length : 2), (StartPosition : 1893; Length : 2), 
      (StartPosition : 1895; Length : 2), (StartPosition : 1897; Length : 2), 
      (StartPosition : 1899; Length : 2), (StartPosition : 1901; Length : 2), 
      (StartPosition : 1903; Length : 2), (StartPosition : 1905; Length : 2), 
      (StartPosition : 1907; Length : 2), (StartPosition : 1909; Length : 2), 
      (StartPosition : 1911; Length : 2), (StartPosition : 1913; Length : 2), 
      (StartPosition : 1915; Length : 2), (StartPosition : 1917; Length : 2), 
      (StartPosition : 1919; Length : 2), (StartPosition : 1921; Length : 2), 
      (StartPosition : 1923; Length : 2), (StartPosition : 1925; Length : 2), 
      (StartPosition : 1927; Length : 2), (StartPosition : 1929; Length : 2), 
      (StartPosition : 1931; Length : 2), (StartPosition : 1933; Length : 2), 
      (StartPosition : 1935; Length : 2), (StartPosition : 1937; Length : 2), 
      (StartPosition : 1939; Length : 2), (StartPosition : 1941; Length : 2), 
      (StartPosition : 1943; Length : 2), (StartPosition : 1945; Length : 2), 
      (StartPosition : 1947; Length : 2), (StartPosition : 1949; Length : 2), 
      (StartPosition : 1951; Length : 2), (StartPosition : 1953; Length : 2), 
      (StartPosition : 1955; Length : 2), (StartPosition : 1957; Length : 2), 
      (StartPosition : 1959; Length : 2), (StartPosition : 1961; Length : 2), 
      (StartPosition : 1963; Length : 2), (StartPosition : 1965; Length : 1), 
      (StartPosition : 1966; Length : 1), (StartPosition : 1967; Length : 1), 
      (StartPosition : 1968; Length : 1), (StartPosition : 1969; Length : 1), 
      (StartPosition : 1970; Length : 1), (StartPosition : 1971; Length : 1), 
      (StartPosition : 1972; Length : 1), (StartPosition : 1973; Length : 1), 
      (StartPosition : 1974; Length : 1), (StartPosition : 1975; Length : 1), 
      (StartPosition : 1976; Length : 1), (StartPosition : 1977; Length : 1), 
      (StartPosition : 1978; Length : 1), (StartPosition : 1979; Length : 1), 
      (StartPosition : 1980; Length : 1), (StartPosition : 1981; Length : 1), 
      (StartPosition : 1982; Length : 1), (StartPosition : 1983; Length : 1), 
      (StartPosition : 1984; Length : 1), (StartPosition : 1985; Length : 1), 
      (StartPosition : 1986; Length : 1), (StartPosition : 1987; Length : 1), 
      (StartPosition : 1988; Length : 1), (StartPosition : 1989; Length : 1), 
      (StartPosition : 1990; Length : 1), (StartPosition : 1991; Length : 1), 
      (StartPosition : 1992; Length : 1), (StartPosition : 1993; Length : 1), 
      (StartPosition : 1994; Length : 1), (StartPosition : 1995; Length : 1), 
      (StartPosition : 1996; Length : 1), (StartPosition : 1997; Length : 1), 
      (StartPosition : 1998; Length : 1), (StartPosition : 1999; Length : 1), 
      (StartPosition : 2000; Length : 1), (StartPosition : 2001; Length : 1), 
      (StartPosition : 2002; Length : 1), (StartPosition : 2003; Length : 1), 
      (StartPosition : 2004; Length : 1), (StartPosition : 2005; Length : 1), 
      (StartPosition : 2006; Length : 1), (StartPosition : 2007; Length : 1), 
      (StartPosition : 2008; Length : 1), (StartPosition : 2009; Length : 1), 
      (StartPosition : 2010; Length : 1), (StartPosition : 2011; Length : 1), 
      (StartPosition : 2012; Length : 1), (StartPosition : 2013; Length : 1), 
      (StartPosition : 2014; Length : 1), (StartPosition : 2015; Length : 1), 
      (StartPosition : 2016; Length : 1), (StartPosition : 2017; Length : 1), 
      (StartPosition : 2018; Length : 1), (StartPosition : 2019; Length : 1), 
      (StartPosition : 2020; Length : 1), (StartPosition : 2021; Length : 1), 
      (StartPosition : 2022; Length : 1), (StartPosition : 2023; Length : 1), 
      (StartPosition : 2024; Length : 1), (StartPosition : 2025; Length : 1), 
      (StartPosition : 2026; Length : 1), (StartPosition : 2027; Length : 1), 
      (StartPosition : 2028; Length : 1), (StartPosition : 2029; Length : 1), 
      (StartPosition : 2030; Length : 1), (StartPosition : 2031; Length : 1), 
      (StartPosition : 2032; Length : 1), (StartPosition : 2033; Length : 1), 
      (StartPosition : 2034; Length : 1), (StartPosition : 2035; Length : 1), 
      (StartPosition : 2036; Length : 1), (StartPosition : 2037; Length : 1), 
      (StartPosition : 2038; Length : 1), (StartPosition : 2039; Length : 1), 
      (StartPosition : 2040; Length : 1), (StartPosition : 2041; Length : 1), 
      (StartPosition : 2042; Length : 1), (StartPosition : 2043; Length : 1), 
      (StartPosition : 2044; Length : 1), (StartPosition : 2045; Length : 1), 
      (StartPosition : 2046; Length : 1), (StartPosition : 2047; Length : 1), 
      (StartPosition : 2048; Length : 1), (StartPosition : 2049; Length : 1), 
      (StartPosition : 2050; Length : 1), (StartPosition : 2051; Length : 1), 
      (StartPosition : 2052; Length : 1), (StartPosition : 2053; Length : 1), 
      (StartPosition : 2054; Length : 1), (StartPosition : 2055; Length : 1), 
      (StartPosition : 2056; Length : 1), (StartPosition : 2057; Length : 1), 
      (StartPosition : 2058; Length : 1), (StartPosition : 2059; Length : 1), 
      (StartPosition : 2060; Length : 1), (StartPosition : 2061; Length : 1), 
      (StartPosition : 2062; Length : 1), (StartPosition : 2063; Length : 1), 
      (StartPosition : 2064; Length : 1), (StartPosition : 2065; Length : 1), 
      (StartPosition : 2066; Length : 1), (StartPosition : 2067; Length : 1), 
      (StartPosition : 2068; Length : 1), (StartPosition : 2069; Length : 1), 
      (StartPosition : 2070; Length : 1), (StartPosition : 2071; Length : 1), 
      (StartPosition : 2072; Length : 1), (StartPosition : 2073; Length : 1), 
      (StartPosition : 2074; Length : 1), (StartPosition : 2075; Length : 1), 
      (StartPosition : 2076; Length : 1), (StartPosition : 2077; Length : 1), 
      (StartPosition : 2078; Length : 1), (StartPosition : 2079; Length : 1), 
      (StartPosition : 2080; Length : 1), (StartPosition : 2081; Length : 1), 
      (StartPosition : 2082; Length : 1), (StartPosition : 2083; Length : 1), 
      (StartPosition : 2084; Length : 1), (StartPosition : 2085; Length : 1), 
      (StartPosition : 2086; Length : 1), (StartPosition : 2087; Length : 1), 
      (StartPosition : 2088; Length : 1), (StartPosition : 2089; Length : 1), 
      (StartPosition : 2090; Length : 1), (StartPosition : 2091; Length : 1), 
      (StartPosition : 2092; Length : 1), (StartPosition : 2093; Length : 1), 
      (StartPosition : 2094; Length : 1), (StartPosition : 2095; Length : 1), 
      (StartPosition : 2096; Length : 1), (StartPosition : 2097; Length : 1), 
      (StartPosition : 2098; Length : 1), (StartPosition : 2099; Length : 1), 
      (StartPosition : 2100; Length : 1), (StartPosition : 2101; Length : 1), 
      (StartPosition : 2102; Length : 1), (StartPosition : 2103; Length : 1), 
      (StartPosition : 2104; Length : 1), (StartPosition : 2105; Length : 1), 
      (StartPosition : 2106; Length : 1), (StartPosition : 2107; Length : 1), 
      (StartPosition : 2108; Length : 1), (StartPosition : 2109; Length : 1), 
      (StartPosition : 2110; Length : 1), (StartPosition : 2111; Length : 1), 
      (StartPosition : 2112; Length : 1), (StartPosition : 2113; Length : 1), 
      (StartPosition : 2114; Length : 1), (StartPosition : 2115; Length : 1), 
      (StartPosition : 2116; Length : 1), (StartPosition : 2117; Length : 1), 
      (StartPosition : 2118; Length : 1), (StartPosition : 2119; Length : 1), 
      (StartPosition : 2120; Length : 1), (StartPosition : 2121; Length : 1), 
      (StartPosition : 2122; Length : 1), (StartPosition : 2123; Length : 1), 
      (StartPosition : 2124; Length : 1), (StartPosition : 2125; Length : 1), 
      (StartPosition : 2126; Length : 1), (StartPosition : 2127; Length : 1), 
      (StartPosition : 2128; Length : 1), (StartPosition : 2129; Length : 1), 
      (StartPosition : 2130; Length : 1), (StartPosition : 2131; Length : 1), 
      (StartPosition : 2132; Length : 1), (StartPosition : 2133; Length : 1), 
      (StartPosition : 2134; Length : 1), (StartPosition : 2135; Length : 1), 
      (StartPosition : 2136; Length : 1), (StartPosition : 2137; Length : 1), 
      (StartPosition : 2138; Length : 1), (StartPosition : 2139; Length : 1), 
      (StartPosition : 2140; Length : 1), (StartPosition : 2141; Length : 1), 
      (StartPosition : 2142; Length : 1), (StartPosition : 2143; Length : 1), 
      (StartPosition : 2144; Length : 1), (StartPosition : 2145; Length : 1), 
      (StartPosition : 2146; Length : 1), (StartPosition : 2147; Length : 1), 
      (StartPosition : 2148; Length : 1), (StartPosition : 2149; Length : 1), 
      (StartPosition : 2150; Length : 1), (StartPosition : 2151; Length : 1), 
      (StartPosition : 2152; Length : 1), (StartPosition : 2153; Length : 1), 
      (StartPosition : 2154; Length : 1), (StartPosition : 2155; Length : 1), 
      (StartPosition : 2156; Length : 1), (StartPosition : 2157; Length : 1), 
      (StartPosition : 2158; Length : 1), (StartPosition : 2159; Length : 1), 
      (StartPosition : 2160; Length : 1), (StartPosition : 2161; Length : 1), 
      (StartPosition : 2162; Length : 1), (StartPosition : 2163; Length : 1), 
      (StartPosition : 2164; Length : 1), (StartPosition : 2165; Length : 1), 
      (StartPosition : 2166; Length : 1), (StartPosition : 2167; Length : 1), 
      (StartPosition : 2168; Length : 1), (StartPosition : 2169; Length : 1), 
      (StartPosition : 2170; Length : 1), (StartPosition : 2171; Length : 1), 
      (StartPosition : 2172; Length : 1), (StartPosition : 2173; Length : 1), 
      (StartPosition : 2174; Length : 1), (StartPosition : 2175; Length : 1), 
      (StartPosition : 2176; Length : 1), (StartPosition : 2177; Length : 1), 
      (StartPosition : 2178; Length : 1), (StartPosition : 2179; Length : 1), 
      (StartPosition : 2180; Length : 1), (StartPosition : 2181; Length : 1), 
      (StartPosition : 2182; Length : 1), (StartPosition : 2183; Length : 1), 
      (StartPosition : 2184; Length : 1), (StartPosition : 2185; Length : 1), 
      (StartPosition : 2186; Length : 1), (StartPosition : 2187; Length : 1), 
      (StartPosition : 2188; Length : 1), (StartPosition : 2189; Length : 1), 
      (StartPosition : 2190; Length : 1), (StartPosition : 2191; Length : 1), 
      (StartPosition : 2192; Length : 1), (StartPosition : 2193; Length : 1), 
      (StartPosition : 2194; Length : 1), (StartPosition : 2195; Length : 1), 
      (StartPosition : 2196; Length : 1), (StartPosition : 2197; Length : 1), 
      (StartPosition : 2198; Length : 1), (StartPosition : 2199; Length : 1), 
      (StartPosition : 2200; Length : 1), (StartPosition : 2201; Length : 1), 
      (StartPosition : 2202; Length : 1), (StartPosition : 2203; Length : 1), 
      (StartPosition : 2204; Length : 1), (StartPosition : 2205; Length : 1), 
      (StartPosition : 2206; Length : 1), (StartPosition : 2207; Length : 1), 
      (StartPosition : 2208; Length : 1), (StartPosition : 2209; Length : 1), 
      (StartPosition : 2210; Length : 1), (StartPosition : 2211; Length : 1), 
      (StartPosition : 2212; Length : 1), (StartPosition : 2213; Length : 1), 
      (StartPosition : 2214; Length : 1), (StartPosition : 2215; Length : 1), 
      (StartPosition : 2216; Length : 1), (StartPosition : 2217; Length : 1), 
      (StartPosition : 2218; Length : 1), (StartPosition : 2219; Length : 1), 
      (StartPosition : 2220; Length : 1), (StartPosition : 2221; Length : 1), 
      (StartPosition : 2222; Length : 1), (StartPosition : 2223; Length : 1), 
      (StartPosition : 2224; Length : 1), (StartPosition : 2225; Length : 1), 
      (StartPosition : 2226; Length : 1), (StartPosition : 2227; Length : 1), 
      (StartPosition : 2228; Length : 1), (StartPosition : 2229; Length : 1), 
      (StartPosition : 2230; Length : 1), (StartPosition : 2231; Length : 1), 
      (StartPosition : 2232; Length : 1), (StartPosition : 2233; Length : 1), 
      (StartPosition : 2234; Length : 1), (StartPosition : 2235; Length : 1), 
      (StartPosition : 2236; Length : 1), (StartPosition : 2237; Length : 1), 
      (StartPosition : 2238; Length : 1), (StartPosition : 2239; Length : 1), 
      (StartPosition : 2240; Length : 1), (StartPosition : 2241; Length : 1), 
      (StartPosition : 2242; Length : 1), (StartPosition : 2243; Length : 1), 
      (StartPosition : 2244; Length : 1), (StartPosition : 2245; Length : 1), 
      (StartPosition : 2246; Length : 1), (StartPosition : 2247; Length : 1), 
      (StartPosition : 2248; Length : 1), (StartPosition : 2249; Length : 1), 
      (StartPosition : 2250; Length : 1), (StartPosition : 2251; Length : 1), 
      (StartPosition : 2252; Length : 1), (StartPosition : 2253; Length : 1), 
      (StartPosition : 2254; Length : 1), (StartPosition : 2255; Length : 1), 
      (StartPosition : 2256; Length : 1), (StartPosition : 2257; Length : 1), 
      (StartPosition : 2258; Length : 1), (StartPosition : 2259; Length : 1), 
      (StartPosition : 2260; Length : 1), (StartPosition : 2261; Length : 1), 
      (StartPosition : 2262; Length : 1), (StartPosition : 2263; Length : 1), 
      (StartPosition : 2264; Length : 1), (StartPosition : 2265; Length : 1), 
      (StartPosition : 2266; Length : 1), (StartPosition : 2267; Length : 1), 
      (StartPosition : 2268; Length : 1), (StartPosition : 2269; Length : 1), 
      (StartPosition : 2270; Length : 1), (StartPosition : 2271; Length : 1), 
      (StartPosition : 2272; Length : 1), (StartPosition : 2273; Length : 1), 
      (StartPosition : 2274; Length : 1), (StartPosition : 2275; Length : 1), 
      (StartPosition : 2276; Length : 1), (StartPosition : 2277; Length : 1), 
      (StartPosition : 2278; Length : 1), (StartPosition : 2279; Length : 1), 
      (StartPosition : 2280; Length : 1), (StartPosition : 2281; Length : 1), 
      (StartPosition : 2282; Length : 1), (StartPosition : 2283; Length : 1), 
      (StartPosition : 2284; Length : 1), (StartPosition : 2285; Length : 1), 
      (StartPosition : 2286; Length : 1), (StartPosition : 2287; Length : 1), 
      (StartPosition : 2288; Length : 1), (StartPosition : 2289; Length : 1), 
      (StartPosition : 2290; Length : 1), (StartPosition : 2291; Length : 1), 
      (StartPosition : 2292; Length : 1), (StartPosition : 2293; Length : 1), 
      (StartPosition : 2294; Length : 1), (StartPosition : 2295; Length : 1), 
      (StartPosition : 2296; Length : 1), (StartPosition : 2297; Length : 1), 
      (StartPosition : 2298; Length : 1), (StartPosition : 2299; Length : 1), 
      (StartPosition : 2300; Length : 1), (StartPosition : 2301; Length : 1), 
      (StartPosition : 2302; Length : 1), (StartPosition : 2303; Length : 1), 
      (StartPosition : 2304; Length : 1), (StartPosition : 2305; Length : 1), 
      (StartPosition : 2306; Length : 1), (StartPosition : 2307; Length : 1), 
      (StartPosition : 2308; Length : 1), (StartPosition : 2309; Length : 1), 
      (StartPosition : 2310; Length : 1), (StartPosition : 2311; Length : 1), 
      (StartPosition : 2312; Length : 1), (StartPosition : 2313; Length : 1), 
      (StartPosition : 2314; Length : 1), (StartPosition : 2315; Length : 1), 
      (StartPosition : 2316; Length : 1), (StartPosition : 2317; Length : 1), 
      (StartPosition : 2318; Length : 1), (StartPosition : 2319; Length : 1), 
      (StartPosition : 2320; Length : 1), (StartPosition : 2321; Length : 1), 
      (StartPosition : 2322; Length : 1), (StartPosition : 2323; Length : 1), 
      (StartPosition : 2324; Length : 1), (StartPosition : 2325; Length : 1), 
      (StartPosition : 2326; Length : 1), (StartPosition : 2327; Length : 1), 
      (StartPosition : 2328; Length : 1), (StartPosition : 2329; Length : 1), 
      (StartPosition : 2330; Length : 1), (StartPosition : 2331; Length : 1), 
      (StartPosition : 2332; Length : 1), (StartPosition : 2333; Length : 1), 
      (StartPosition : 2334; Length : 1), (StartPosition : 2335; Length : 1), 
      (StartPosition : 2336; Length : 1), (StartPosition : 2337; Length : 1), 
      (StartPosition : 2338; Length : 1), (StartPosition : 2339; Length : 1), 
      (StartPosition : 2340; Length : 1), (StartPosition : 2341; Length : 1), 
      (StartPosition : 2342; Length : 1), (StartPosition : 2343; Length : 1), 
      (StartPosition : 2344; Length : 1), (StartPosition : 2345; Length : 1), 
      (StartPosition : 2346; Length : 1), (StartPosition : 2347; Length : 1), 
      (StartPosition : 2348; Length : 1), (StartPosition : 2349; Length : 1), 
      (StartPosition : 2350; Length : 1), (StartPosition : 2351; Length : 1), 
      (StartPosition : 2352; Length : 1), (StartPosition : 2353; Length : 1), 
      (StartPosition : 2354; Length : 1), (StartPosition : 2355; Length : 1), 
      (StartPosition : 2356; Length : 1), (StartPosition : 2357; Length : 1), 
      (StartPosition : 2358; Length : 1), (StartPosition : 2359; Length : 1), 
      (StartPosition : 2360; Length : 1), (StartPosition : 2361; Length : 1), 
      (StartPosition : 2362; Length : 1), (StartPosition : 2363; Length : 1), 
      (StartPosition : 2364; Length : 1), (StartPosition : 2365; Length : 1), 
      (StartPosition : 2366; Length : 1), (StartPosition : 2367; Length : 1), 
      (StartPosition : 2368; Length : 1), (StartPosition : 2369; Length : 1), 
      (StartPosition : 2370; Length : 1), (StartPosition : 2371; Length : 1), 
      (StartPosition : 2372; Length : 1), (StartPosition : 2373; Length : 1), 
      (StartPosition : 2374; Length : 1), (StartPosition : 2375; Length : 1), 
      (StartPosition : 2376; Length : 1), (StartPosition : 2377; Length : 1), 
      (StartPosition : 2378; Length : 1), (StartPosition : 2379; Length : 1), 
      (StartPosition : 2380; Length : 1), (StartPosition : 2381; Length : 1), 
      (StartPosition : 2382; Length : 1), (StartPosition : 2383; Length : 1), 
      (StartPosition : 2384; Length : 1), (StartPosition : 2385; Length : 1), 
      (StartPosition : 2386; Length : 1), (StartPosition : 2387; Length : 1), 
      (StartPosition : 2388; Length : 1), (StartPosition : 2389; Length : 1), 
      (StartPosition : 2390; Length : 2), (StartPosition : 2392; Length : 2), 
      (StartPosition : 2394; Length : 2), (StartPosition : 2396; Length : 2), 
      (StartPosition : 2398; Length : 2), (StartPosition : 2400; Length : 2), 
      (StartPosition : 2402; Length : 2), (StartPosition : 2404; Length : 2), 
      (StartPosition : 2406; Length : 2), (StartPosition : 2408; Length : 2), 
      (StartPosition : 2410; Length : 2), (StartPosition : 2412; Length : 2), 
      (StartPosition : 2414; Length : 2), (StartPosition : 2416; Length : 2), 
      (StartPosition : 2418; Length : 2), (StartPosition : 2420; Length : 2), 
      (StartPosition : 2422; Length : 2), (StartPosition : 2424; Length : 2), 
      (StartPosition : 2426; Length : 2), (StartPosition : 2428; Length : 2), 
      (StartPosition : 2430; Length : 2), (StartPosition : 2432; Length : 2), 
      (StartPosition : 2434; Length : 2), (StartPosition : 2436; Length : 2), 
      (StartPosition : 2438; Length : 2), (StartPosition : 2440; Length : 2), 
      (StartPosition : 2442; Length : 2), (StartPosition : 2444; Length : 2), 
      (StartPosition : 2446; Length : 2), (StartPosition : 2448; Length : 2), 
      (StartPosition : 2450; Length : 2), (StartPosition : 2452; Length : 2), 
      (StartPosition : 2454; Length : 2), (StartPosition : 2456; Length : 2), 
      (StartPosition : 2458; Length : 2), (StartPosition : 2460; Length : 2), 
      (StartPosition : 2462; Length : 2), (StartPosition : 2464; Length : 2), 
      (StartPosition : 2466; Length : 2), (StartPosition : 2468; Length : 2), 
      (StartPosition : 2470; Length : 2), (StartPosition : 2472; Length : 2), 
      (StartPosition : 2474; Length : 2), (StartPosition : 2476; Length : 2), 
      (StartPosition : 2478; Length : 2), (StartPosition : 2480; Length : 2), 
      (StartPosition : 2482; Length : 2), (StartPosition : 2484; Length : 2), 
      (StartPosition : 2486; Length : 2), (StartPosition : 2488; Length : 2), 
      (StartPosition : 2490; Length : 2), (StartPosition : 2492; Length : 2), 
      (StartPosition : 2494; Length : 2), (StartPosition : 2496; Length : 2), 
      (StartPosition : 2498; Length : 2), (StartPosition : 2500; Length : 2), 
      (StartPosition : 2502; Length : 2), (StartPosition : 2504; Length : 2), 
      (StartPosition : 2506; Length : 2), (StartPosition : 2508; Length : 1), 
      (StartPosition : 2509; Length : 1), (StartPosition : 2510; Length : 1), 
      (StartPosition : 2511; Length : 1), (StartPosition : 2512; Length : 1), 
      (StartPosition : 2513; Length : 1), (StartPosition : 2514; Length : 1), 
      (StartPosition : 2515; Length : 1), (StartPosition : 2516; Length : 1), 
      (StartPosition : 2517; Length : 1), (StartPosition : 2518; Length : 1), 
      (StartPosition : 2519; Length : 1), (StartPosition : 2520; Length : 1), 
      (StartPosition : 2521; Length : 1), (StartPosition : 2522; Length : 1), 
      (StartPosition : 2523; Length : 1), (StartPosition : 2524; Length : 1), 
      (StartPosition : 2525; Length : 1), (StartPosition : 2526; Length : 1), 
      (StartPosition : 2527; Length : 1), (StartPosition : 2528; Length : 1), 
      (StartPosition : 2529; Length : 1), (StartPosition : 2530; Length : 1), 
      (StartPosition : 2531; Length : 1), (StartPosition : 2532; Length : 1), 
      (StartPosition : 2533; Length : 1), (StartPosition : 2534; Length : 1), 
      (StartPosition : 2535; Length : 1), (StartPosition : 2536; Length : 1), 
      (StartPosition : 2537; Length : 1), (StartPosition : 2538; Length : 1), 
      (StartPosition : 2539; Length : 1), (StartPosition : 2540; Length : 1), 
      (StartPosition : 2541; Length : 1), (StartPosition : 2542; Length : 1), 
      (StartPosition : 2543; Length : 1), (StartPosition : 2544; Length : 1), 
      (StartPosition : 2545; Length : 1), (StartPosition : 2546; Length : 1), 
      (StartPosition : 2547; Length : 1), (StartPosition : 2548; Length : 1), 
      (StartPosition : 2549; Length : 1), (StartPosition : 2550; Length : 1), 
      (StartPosition : 2551; Length : 1), (StartPosition : 2552; Length : 1), 
      (StartPosition : 2553; Length : 1), (StartPosition : 2554; Length : 1), 
      (StartPosition : 2555; Length : 1), (StartPosition : 2556; Length : 1), 
      (StartPosition : 2557; Length : 1), (StartPosition : 2558; Length : 1), 
      (StartPosition : 2559; Length : 1), (StartPosition : 2560; Length : 1), 
      (StartPosition : 2561; Length : 1), (StartPosition : 2562; Length : 1), 
      (StartPosition : 2563; Length : 1), (StartPosition : 2564; Length : 1), 
      (StartPosition : 2565; Length : 1), (StartPosition : 2566; Length : 1), 
      (StartPosition : 2567; Length : 1), (StartPosition : 2568; Length : 1), 
      (StartPosition : 2569; Length : 1), (StartPosition : 2570; Length : 1), 
      (StartPosition : 2571; Length : 1), (StartPosition : 2572; Length : 1), 
      (StartPosition : 2573; Length : 1), (StartPosition : 2574; Length : 1), 
      (StartPosition : 2575; Length : 1), (StartPosition : 2576; Length : 1), 
      (StartPosition : 2577; Length : 1), (StartPosition : 2578; Length : 1), 
      (StartPosition : 2579; Length : 1), (StartPosition : 2580; Length : 1), 
      (StartPosition : 2581; Length : 1), (StartPosition : 2582; Length : 1), 
      (StartPosition : 2583; Length : 1), (StartPosition : 2584; Length : 1), 
      (StartPosition : 2585; Length : 1), (StartPosition : 2586; Length : 1), 
      (StartPosition : 2587; Length : 1), (StartPosition : 2588; Length : 1), 
      (StartPosition : 2589; Length : 1), (StartPosition : 2590; Length : 1), 
      (StartPosition : 2591; Length : 1), (StartPosition : 2592; Length : 1), 
      (StartPosition : 2593; Length : 1), (StartPosition : 2594; Length : 1), 
      (StartPosition : 2595; Length : 1), (StartPosition : 2596; Length : 1), 
      (StartPosition : 2597; Length : 1), (StartPosition : 2598; Length : 1), 
      (StartPosition : 2599; Length : 1), (StartPosition : 2600; Length : 1), 
      (StartPosition : 2601; Length : 1), (StartPosition : 2602; Length : 1), 
      (StartPosition : 2603; Length : 1), (StartPosition : 2604; Length : 1), 
      (StartPosition : 2605; Length : 1), (StartPosition : 2606; Length : 1), 
      (StartPosition : 2607; Length : 1), (StartPosition : 2608; Length : 1), 
      (StartPosition : 2609; Length : 1), (StartPosition : 2610; Length : 1), 
      (StartPosition : 2611; Length : 1), (StartPosition : 2612; Length : 1), 
      (StartPosition : 2613; Length : 1), (StartPosition : 2614; Length : 1), 
      (StartPosition : 2615; Length : 1), (StartPosition : 2616; Length : 1), 
      (StartPosition : 2617; Length : 1), (StartPosition : 2618; Length : 1), 
      (StartPosition : 2619; Length : 1), (StartPosition : 2620; Length : 1), 
      (StartPosition : 2621; Length : 1), (StartPosition : 2622; Length : 1), 
      (StartPosition : 2623; Length : 1), (StartPosition : 2624; Length : 1), 
      (StartPosition : 2625; Length : 1), (StartPosition : 2626; Length : 1), 
      (StartPosition : 2627; Length : 1), (StartPosition : 2628; Length : 1), 
      (StartPosition : 2629; Length : 1), (StartPosition : 2630; Length : 1), 
      (StartPosition : 2631; Length : 1), (StartPosition : 2632; Length : 1), 
      (StartPosition : 2633; Length : 1), (StartPosition : 2634; Length : 1), 
      (StartPosition : 2635; Length : 1), (StartPosition : 2636; Length : 1), 
      (StartPosition : 2637; Length : 1), (StartPosition : 2638; Length : 1), 
      (StartPosition : 2639; Length : 1), (StartPosition : 2640; Length : 1), 
      (StartPosition : 2641; Length : 1), (StartPosition : 2642; Length : 1), 
      (StartPosition : 2643; Length : 1), (StartPosition : 2644; Length : 1), 
      (StartPosition : 2645; Length : 1), (StartPosition : 2646; Length : 1), 
      (StartPosition : 2647; Length : 1), (StartPosition : 2648; Length : 1), 
      (StartPosition : 2649; Length : 1), (StartPosition : 2650; Length : 1), 
      (StartPosition : 2651; Length : 1), (StartPosition : 2652; Length : 1), 
      (StartPosition : 2653; Length : 1), (StartPosition : 2654; Length : 1), 
      (StartPosition : 2655; Length : 1), (StartPosition : 2656; Length : 1), 
      (StartPosition : 2657; Length : 1), (StartPosition : 2658; Length : 1), 
      (StartPosition : 2659; Length : 1), (StartPosition : 2660; Length : 1), 
      (StartPosition : 2661; Length : 1), (StartPosition : 2662; Length : 1), 
      (StartPosition : 2663; Length : 1), (StartPosition : 2664; Length : 1), 
      (StartPosition : 2665; Length : 1), (StartPosition : 2666; Length : 1), 
      (StartPosition : 2667; Length : 1), (StartPosition : 2668; Length : 1), 
      (StartPosition : 2669; Length : 1), (StartPosition : 2670; Length : 1), 
      (StartPosition : 2671; Length : 1), (StartPosition : 2672; Length : 1), 
      (StartPosition : 2673; Length : 1), (StartPosition : 2674; Length : 1), 
      (StartPosition : 2675; Length : 1), (StartPosition : 2676; Length : 1), 
      (StartPosition : 2677; Length : 1), (StartPosition : 2678; Length : 1), 
      (StartPosition : 2679; Length : 1), (StartPosition : 2680; Length : 1), 
      (StartPosition : 2681; Length : 1), (StartPosition : 2682; Length : 1), 
      (StartPosition : 2683; Length : 1), (StartPosition : 2684; Length : 1), 
      (StartPosition : 2685; Length : 1), (StartPosition : 2686; Length : 1), 
      (StartPosition : 2687; Length : 1), (StartPosition : 2688; Length : 1), 
      (StartPosition : 2689; Length : 1), (StartPosition : 2690; Length : 1), 
      (StartPosition : 2691; Length : 1), (StartPosition : 2692; Length : 1), 
      (StartPosition : 2693; Length : 1), (StartPosition : 2694; Length : 1), 
      (StartPosition : 2695; Length : 1), (StartPosition : 2696; Length : 1), 
      (StartPosition : 2697; Length : 1), (StartPosition : 2698; Length : 1), 
      (StartPosition : 2699; Length : 1), (StartPosition : 2700; Length : 1), 
      (StartPosition : 2701; Length : 1), (StartPosition : 2702; Length : 1), 
      (StartPosition : 2703; Length : 1), (StartPosition : 2704; Length : 1), 
      (StartPosition : 2705; Length : 1), (StartPosition : 2706; Length : 1), 
      (StartPosition : 2707; Length : 1), (StartPosition : 2708; Length : 1), 
      (StartPosition : 2709; Length : 1), (StartPosition : 2710; Length : 1), 
      (StartPosition : 2711; Length : 1), (StartPosition : 2712; Length : 1), 
      (StartPosition : 2713; Length : 1), (StartPosition : 2714; Length : 1), 
      (StartPosition : 2715; Length : 1), (StartPosition : 2716; Length : 1), 
      (StartPosition : 2717; Length : 1), (StartPosition : 2718; Length : 1), 
      (StartPosition : 2719; Length : 1), (StartPosition : 2720; Length : 1), 
      (StartPosition : 2721; Length : 1), (StartPosition : 2722; Length : 1), 
      (StartPosition : 2723; Length : 1), (StartPosition : 2724; Length : 1), 
      (StartPosition : 2725; Length : 1), (StartPosition : 2726; Length : 1), 
      (StartPosition : 2727; Length : 1), (StartPosition : 2728; Length : 1), 
      (StartPosition : 2729; Length : 1), (StartPosition : 2730; Length : 1), 
      (StartPosition : 2731; Length : 1), (StartPosition : 2732; Length : 1), 
      (StartPosition : 2733; Length : 1), (StartPosition : 2734; Length : 1), 
      (StartPosition : 2735; Length : 1), (StartPosition : 2736; Length : 1), 
      (StartPosition : 2737; Length : 1), (StartPosition : 2738; Length : 1), 
      (StartPosition : 2739; Length : 1), (StartPosition : 2740; Length : 1), 
      (StartPosition : 2741; Length : 1), (StartPosition : 2742; Length : 1), 
      (StartPosition : 2743; Length : 1), (StartPosition : 2744; Length : 1), 
      (StartPosition : 2745; Length : 1), (StartPosition : 2746; Length : 1), 
      (StartPosition : 2747; Length : 1), (StartPosition : 2748; Length : 1), 
      (StartPosition : 2749; Length : 1), (StartPosition : 2750; Length : 1), 
      (StartPosition : 2751; Length : 1), (StartPosition : 2752; Length : 1), 
      (StartPosition : 2753; Length : 1), (StartPosition : 2754; Length : 1), 
      (StartPosition : 2755; Length : 1), (StartPosition : 2756; Length : 1), 
      (StartPosition : 2757; Length : 1), (StartPosition : 2758; Length : 1), 
      (StartPosition : 2759; Length : 1), (StartPosition : 2760; Length : 1), 
      (StartPosition : 2761; Length : 1), (StartPosition : 2762; Length : 1), 
      (StartPosition : 2763; Length : 1), (StartPosition : 2764; Length : 1), 
      (StartPosition : 2765; Length : 1), (StartPosition : 2766; Length : 1), 
      (StartPosition : 2767; Length : 1), (StartPosition : 2768; Length : 1), 
      (StartPosition : 2769; Length : 1), (StartPosition : 2770; Length : 1), 
      (StartPosition : 2771; Length : 1), (StartPosition : 2772; Length : 1), 
      (StartPosition : 2773; Length : 1), (StartPosition : 2774; Length : 1), 
      (StartPosition : 2775; Length : 1), (StartPosition : 2776; Length : 1), 
      (StartPosition : 2777; Length : 1), (StartPosition : 2778; Length : 1), 
      (StartPosition : 2779; Length : 1), (StartPosition : 2780; Length : 1), 
      (StartPosition : 2781; Length : 1), (StartPosition : 2782; Length : 1), 
      (StartPosition : 2783; Length : 1), (StartPosition : 2784; Length : 1), 
      (StartPosition : 2785; Length : 1), (StartPosition : 2786; Length : 1), 
      (StartPosition : 2787; Length : 1), (StartPosition : 2788; Length : 1), 
      (StartPosition : 2789; Length : 1), (StartPosition : 2790; Length : 1), 
      (StartPosition : 2791; Length : 1), (StartPosition : 2792; Length : 1), 
      (StartPosition : 2793; Length : 1), (StartPosition : 2794; Length : 1), 
      (StartPosition : 2795; Length : 1), (StartPosition : 2796; Length : 1), 
      (StartPosition : 2797; Length : 1), (StartPosition : 2798; Length : 1), 
      (StartPosition : 2799; Length : 1), (StartPosition : 2800; Length : 1), 
      (StartPosition : 2801; Length : 1), (StartPosition : 2802; Length : 1), 
      (StartPosition : 2803; Length : 1), (StartPosition : 2804; Length : 1), 
      (StartPosition : 2805; Length : 1), (StartPosition : 2806; Length : 1), 
      (StartPosition : 2807; Length : 1), (StartPosition : 2808; Length : 1), 
      (StartPosition : 2809; Length : 1), (StartPosition : 2810; Length : 1), 
      (StartPosition : 2811; Length : 1), (StartPosition : 2812; Length : 1), 
      (StartPosition : 2813; Length : 1), (StartPosition : 2814; Length : 1), 
      (StartPosition : 2815; Length : 1), (StartPosition : 2816; Length : 1), 
      (StartPosition : 2817; Length : 1), (StartPosition : 2818; Length : 1), 
      (StartPosition : 2819; Length : 1), (StartPosition : 2820; Length : 1), 
      (StartPosition : 2821; Length : 1), (StartPosition : 2822; Length : 1), 
      (StartPosition : 2823; Length : 1), (StartPosition : 2824; Length : 1), 
      (StartPosition : 2825; Length : 1), (StartPosition : 2826; Length : 1), 
      (StartPosition : 2827; Length : 1), (StartPosition : 2828; Length : 1), 
      (StartPosition : 2829; Length : 1), (StartPosition : 2830; Length : 1), 
      (StartPosition : 2831; Length : 1), (StartPosition : 2832; Length : 1), 
      (StartPosition : 2833; Length : 1), (StartPosition : 2834; Length : 1), 
      (StartPosition : 2835; Length : 1), (StartPosition : 2836; Length : 1), 
      (StartPosition : 2837; Length : 1), (StartPosition : 2838; Length : 1), 
      (StartPosition : 2839; Length : 1), (StartPosition : 2840; Length : 1), 
      (StartPosition : 2841; Length : 1), (StartPosition : 2842; Length : 1), 
      (StartPosition : 2843; Length : 1), (StartPosition : 2844; Length : 1), 
      (StartPosition : 2845; Length : 1), (StartPosition : 2846; Length : 1), 
      (StartPosition : 2847; Length : 1), (StartPosition : 2848; Length : 1), 
      (StartPosition : 2849; Length : 1), (StartPosition : 2850; Length : 1), 
      (StartPosition : 2851; Length : 1), (StartPosition : 2852; Length : 1), 
      (StartPosition : 2853; Length : 1), (StartPosition : 2854; Length : 1), 
      (StartPosition : 2855; Length : 1), (StartPosition : 2856; Length : 1), 
      (StartPosition : 2857; Length : 1), (StartPosition : 2858; Length : 1), 
      (StartPosition : 2859; Length : 1), (StartPosition : 2860; Length : 1), 
      (StartPosition : 2861; Length : 1), (StartPosition : 2862; Length : 1), 
      (StartPosition : 2863; Length : 1), (StartPosition : 2864; Length : 1), 
      (StartPosition : 2865; Length : 1), (StartPosition : 2866; Length : 1), 
      (StartPosition : 2867; Length : 1), (StartPosition : 2868; Length : 1), 
      (StartPosition : 2869; Length : 1), (StartPosition : 2870; Length : 1), 
      (StartPosition : 2871; Length : 1), (StartPosition : 2872; Length : 1), 
      (StartPosition : 2873; Length : 1), (StartPosition : 2874; Length : 1), 
      (StartPosition : 2875; Length : 1), (StartPosition : 2876; Length : 1), 
      (StartPosition : 2877; Length : 1), (StartPosition : 2878; Length : 1), 
      (StartPosition : 2879; Length : 1), (StartPosition : 2880; Length : 1), 
      (StartPosition : 2881; Length : 1), (StartPosition : 2882; Length : 1), 
      (StartPosition : 2883; Length : 1), (StartPosition : 2884; Length : 1), 
      (StartPosition : 2885; Length : 1), (StartPosition : 2886; Length : 1), 
      (StartPosition : 2887; Length : 1), (StartPosition : 2888; Length : 1), 
      (StartPosition : 2889; Length : 1), (StartPosition : 2890; Length : 1), 
      (StartPosition : 2891; Length : 1), (StartPosition : 2892; Length : 1), 
      (StartPosition : 2893; Length : 1), (StartPosition : 2894; Length : 1), 
      (StartPosition : 2895; Length : 1), (StartPosition : 2896; Length : 1), 
      (StartPosition : 2897; Length : 1), (StartPosition : 2898; Length : 1), 
      (StartPosition : 2899; Length : 1), (StartPosition : 2900; Length : 1), 
      (StartPosition : 2901; Length : 1), (StartPosition : 2902; Length : 1), 
      (StartPosition : 2903; Length : 1), (StartPosition : 2904; Length : 1), 
      (StartPosition : 2905; Length : 1), (StartPosition : 2906; Length : 1), 
      (StartPosition : 2907; Length : 1), (StartPosition : 2908; Length : 1), 
      (StartPosition : 2909; Length : 1), (StartPosition : 2910; Length : 1), 
      (StartPosition : 2911; Length : 1), (StartPosition : 2912; Length : 1), 
      (StartPosition : 2913; Length : 1), (StartPosition : 2914; Length : 1), 
      (StartPosition : 2915; Length : 1), (StartPosition : 2916; Length : 1), 
      (StartPosition : 2917; Length : 1), (StartPosition : 2918; Length : 1), 
      (StartPosition : 2919; Length : 1), (StartPosition : 2920; Length : 1), 
      (StartPosition : 2921; Length : 1), (StartPosition : 2922; Length : 1), 
      (StartPosition : 2923; Length : 1), (StartPosition : 2924; Length : 1), 
      (StartPosition : 2925; Length : 1), (StartPosition : 2926; Length : 1), 
      (StartPosition : 2927; Length : 1), (StartPosition : 2928; Length : 1), 
      (StartPosition : 2929; Length : 1), (StartPosition : 2930; Length : 1), 
      (StartPosition : 2931; Length : 1), (StartPosition : 2932; Length : 1), 
      (StartPosition : 2933; Length : 1), (StartPosition : 2934; Length : 1), 
      (StartPosition : 2935; Length : 1), (StartPosition : 2936; Length : 1), 
      (StartPosition : 2937; Length : 1), (StartPosition : 2938; Length : 1), 
      (StartPosition : 2939; Length : 1), (StartPosition : 2940; Length : 1), 
      (StartPosition : 2941; Length : 1), (StartPosition : 2942; Length : 1), 
      (StartPosition : 2943; Length : 1), (StartPosition : 2944; Length : 1), 
      (StartPosition : 2945; Length : 1), (StartPosition : 2946; Length : 1), 
      (StartPosition : 2947; Length : 1), (StartPosition : 2948; Length : 1), 
      (StartPosition : 2949; Length : 1), (StartPosition : 2950; Length : 1), 
      (StartPosition : 2951; Length : 1), (StartPosition : 2952; Length : 1), 
      (StartPosition : 2953; Length : 1), (StartPosition : 2954; Length : 1), 
      (StartPosition : 2955; Length : 1), (StartPosition : 2956; Length : 1), 
      (StartPosition : 2957; Length : 1), (StartPosition : 2958; Length : 1), 
      (StartPosition : 2959; Length : 1), (StartPosition : 2960; Length : 1), 
      (StartPosition : 2961; Length : 1), (StartPosition : 2962; Length : 1), 
      (StartPosition : 2963; Length : 1), (StartPosition : 2964; Length : 1), 
      (StartPosition : 2965; Length : 1), (StartPosition : 2966; Length : 1), 
      (StartPosition : 2967; Length : 1), (StartPosition : 2968; Length : 1), 
      (StartPosition : 2969; Length : 1), (StartPosition : 2970; Length : 1), 
      (StartPosition : 2971; Length : 1), (StartPosition : 2972; Length : 1), 
      (StartPosition : 2973; Length : 1), (StartPosition : 2974; Length : 1), 
      (StartPosition : 2975; Length : 1), (StartPosition : 2976; Length : 1), 
      (StartPosition : 2977; Length : 1), (StartPosition : 2978; Length : 1), 
      (StartPosition : 2979; Length : 1), (StartPosition : 2980; Length : 1), 
      (StartPosition : 2981; Length : 1), (StartPosition : 2982; Length : 1), 
      (StartPosition : 2983; Length : 1), (StartPosition : 2984; Length : 1)
    ); // Index END
    CodePoints : (// CodePoints BEGIN
      (byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $0A; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),
      (byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),
      (byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $0A; byte1 : $03; byte2 : $00;),
      (byte0 : $63; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),
      (byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),
      (byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $63; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $63; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $63; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $63; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $44; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),
      (byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $47; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),
      (byte0 : $67; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $47; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $67; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $47; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $67; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $47; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $67; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),
      (byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),
      (byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $4A; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $6A; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $4B; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $6B; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $4C; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $6C; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $4C; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $6C; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $4C; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $6C; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),
      (byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $0B; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $0B; byte1 : $03; byte2 : $00;),
      (byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $53; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $53; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $53; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $53; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $54; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),
      (byte0 : $54; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),
      (byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $0A; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $0A; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $0B; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $0B; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $57; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),
      (byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $5A; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $7A; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $5A; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $7A; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $5A; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),
      (byte0 : $7A; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $1B; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $1B; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $1B; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $1B; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),
      (byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $DC; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $FC; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $DC; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $FC; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $DC; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $FC; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $DC; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $FC; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $C4; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $E4; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $26; byte1 : $02; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),
      (byte0 : $27; byte1 : $02; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $C6; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $E6; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $47; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $67; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $4B; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $6B; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),
      (byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $28; byte1 : $03; byte2 : $00;),(byte0 : $EA; byte1 : $01; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $EB; byte1 : $01; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $B7; byte1 : $01; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $92; byte1 : $02; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $6A; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $47; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $67; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $E5; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $C6; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $E6; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $D8; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $F8; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),
      (byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),
      (byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $11; byte1 : $03; byte2 : $00;),
      (byte0 : $53; byte1 : $00; byte2 : $00;),(byte0 : $26; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $00; byte2 : $00;),(byte0 : $26; byte1 : $03; byte2 : $00;),(byte0 : $54; byte1 : $00; byte2 : $00;),(byte0 : $26; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $26; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $0C; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $D6; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $F6; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $D5; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $F5; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $2E; byte1 : $02; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $2F; byte1 : $02; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $02; byte2 : $00;),(byte0 : $3B; byte1 : $00; byte2 : $00;),(byte0 : $A8; byte1 : $00; byte2 : $00;),
      (byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $B7; byte1 : $00; byte2 : $00;),(byte0 : $95; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $97; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $99; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $9F; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $A5; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $A9; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $CA; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $99; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $A5; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $B5; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $B7; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $CB; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $BF; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $C9; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $D2; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $D2; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $15; byte1 : $04; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $15; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $04; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $06; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $1A; byte1 : $04; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $18; byte1 : $04; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $23; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $18; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $38; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),
      (byte0 : $35; byte1 : $04; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $35; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $33; byte1 : $04; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $56; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $3A; byte1 : $04; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $38; byte1 : $04; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $04; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),
      (byte0 : $75; byte1 : $04; byte2 : $00;),(byte0 : $0F; byte1 : $03; byte2 : $00;),(byte0 : $16; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $36; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $10; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $30; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $10; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $30; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $15; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),
      (byte0 : $35; byte1 : $04; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $D8; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $D9; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $16; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $36; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $17; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $37; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $18; byte1 : $04; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),
      (byte0 : $38; byte1 : $04; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $18; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $38; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $1E; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $3E; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $E8; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $E9; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $2D; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),
      (byte0 : $4D; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $23; byte1 : $04; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $04; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $23; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $23; byte1 : $04; byte2 : $00;),(byte0 : $0B; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $04; byte2 : $00;),(byte0 : $0B; byte1 : $03; byte2 : $00;),(byte0 : $27; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),
      (byte0 : $47; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $2B; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $4B; byte1 : $04; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $27; byte1 : $06; byte2 : $00;),(byte0 : $53; byte1 : $06; byte2 : $00;),(byte0 : $27; byte1 : $06; byte2 : $00;),(byte0 : $54; byte1 : $06; byte2 : $00;),(byte0 : $48; byte1 : $06; byte2 : $00;),(byte0 : $54; byte1 : $06; byte2 : $00;),(byte0 : $27; byte1 : $06; byte2 : $00;),(byte0 : $55; byte1 : $06; byte2 : $00;),(byte0 : $4A; byte1 : $06; byte2 : $00;),(byte0 : $54; byte1 : $06; byte2 : $00;),
      (byte0 : $D5; byte1 : $06; byte2 : $00;),(byte0 : $54; byte1 : $06; byte2 : $00;),(byte0 : $C1; byte1 : $06; byte2 : $00;),(byte0 : $54; byte1 : $06; byte2 : $00;),(byte0 : $D2; byte1 : $06; byte2 : $00;),(byte0 : $54; byte1 : $06; byte2 : $00;),(byte0 : $28; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $30; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $33; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $15; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $16; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),
      (byte0 : $17; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $1C; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $21; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $22; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $2B; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $2F; byte1 : $09; byte2 : $00;),(byte0 : $3C; byte1 : $09; byte2 : $00;),(byte0 : $C7; byte1 : $09; byte2 : $00;),(byte0 : $BE; byte1 : $09; byte2 : $00;),(byte0 : $C7; byte1 : $09; byte2 : $00;),(byte0 : $D7; byte1 : $09; byte2 : $00;),
      (byte0 : $A1; byte1 : $09; byte2 : $00;),(byte0 : $BC; byte1 : $09; byte2 : $00;),(byte0 : $A2; byte1 : $09; byte2 : $00;),(byte0 : $BC; byte1 : $09; byte2 : $00;),(byte0 : $AF; byte1 : $09; byte2 : $00;),(byte0 : $BC; byte1 : $09; byte2 : $00;),(byte0 : $32; byte1 : $0A; byte2 : $00;),(byte0 : $3C; byte1 : $0A; byte2 : $00;),(byte0 : $38; byte1 : $0A; byte2 : $00;),(byte0 : $3C; byte1 : $0A; byte2 : $00;),(byte0 : $16; byte1 : $0A; byte2 : $00;),(byte0 : $3C; byte1 : $0A; byte2 : $00;),(byte0 : $17; byte1 : $0A; byte2 : $00;),(byte0 : $3C; byte1 : $0A; byte2 : $00;),(byte0 : $1C; byte1 : $0A; byte2 : $00;),(byte0 : $3C; byte1 : $0A; byte2 : $00;),
      (byte0 : $2B; byte1 : $0A; byte2 : $00;),(byte0 : $3C; byte1 : $0A; byte2 : $00;),(byte0 : $47; byte1 : $0B; byte2 : $00;),(byte0 : $56; byte1 : $0B; byte2 : $00;),(byte0 : $47; byte1 : $0B; byte2 : $00;),(byte0 : $3E; byte1 : $0B; byte2 : $00;),(byte0 : $47; byte1 : $0B; byte2 : $00;),(byte0 : $57; byte1 : $0B; byte2 : $00;),(byte0 : $21; byte1 : $0B; byte2 : $00;),(byte0 : $3C; byte1 : $0B; byte2 : $00;),(byte0 : $22; byte1 : $0B; byte2 : $00;),(byte0 : $3C; byte1 : $0B; byte2 : $00;),(byte0 : $92; byte1 : $0B; byte2 : $00;),(byte0 : $D7; byte1 : $0B; byte2 : $00;),(byte0 : $C6; byte1 : $0B; byte2 : $00;),(byte0 : $BE; byte1 : $0B; byte2 : $00;),
      (byte0 : $C7; byte1 : $0B; byte2 : $00;),(byte0 : $BE; byte1 : $0B; byte2 : $00;),(byte0 : $C6; byte1 : $0B; byte2 : $00;),(byte0 : $D7; byte1 : $0B; byte2 : $00;),(byte0 : $46; byte1 : $0C; byte2 : $00;),(byte0 : $56; byte1 : $0C; byte2 : $00;),(byte0 : $BF; byte1 : $0C; byte2 : $00;),(byte0 : $D5; byte1 : $0C; byte2 : $00;),(byte0 : $C6; byte1 : $0C; byte2 : $00;),(byte0 : $D5; byte1 : $0C; byte2 : $00;),(byte0 : $C6; byte1 : $0C; byte2 : $00;),(byte0 : $D6; byte1 : $0C; byte2 : $00;),(byte0 : $C6; byte1 : $0C; byte2 : $00;),(byte0 : $C2; byte1 : $0C; byte2 : $00;),(byte0 : $CA; byte1 : $0C; byte2 : $00;),(byte0 : $D5; byte1 : $0C; byte2 : $00;),
      (byte0 : $46; byte1 : $0D; byte2 : $00;),(byte0 : $3E; byte1 : $0D; byte2 : $00;),(byte0 : $47; byte1 : $0D; byte2 : $00;),(byte0 : $3E; byte1 : $0D; byte2 : $00;),(byte0 : $46; byte1 : $0D; byte2 : $00;),(byte0 : $57; byte1 : $0D; byte2 : $00;),(byte0 : $D9; byte1 : $0D; byte2 : $00;),(byte0 : $CA; byte1 : $0D; byte2 : $00;),(byte0 : $D9; byte1 : $0D; byte2 : $00;),(byte0 : $CF; byte1 : $0D; byte2 : $00;),(byte0 : $DC; byte1 : $0D; byte2 : $00;),(byte0 : $CA; byte1 : $0D; byte2 : $00;),(byte0 : $D9; byte1 : $0D; byte2 : $00;),(byte0 : $DF; byte1 : $0D; byte2 : $00;),(byte0 : $42; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),
      (byte0 : $4C; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $51; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $56; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $5B; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $40; byte1 : $0F; byte2 : $00;),(byte0 : $B5; byte1 : $0F; byte2 : $00;),(byte0 : $71; byte1 : $0F; byte2 : $00;),(byte0 : $72; byte1 : $0F; byte2 : $00;),(byte0 : $71; byte1 : $0F; byte2 : $00;),(byte0 : $74; byte1 : $0F; byte2 : $00;),(byte0 : $B2; byte1 : $0F; byte2 : $00;),(byte0 : $80; byte1 : $0F; byte2 : $00;),
      (byte0 : $B3; byte1 : $0F; byte2 : $00;),(byte0 : $80; byte1 : $0F; byte2 : $00;),(byte0 : $71; byte1 : $0F; byte2 : $00;),(byte0 : $80; byte1 : $0F; byte2 : $00;),(byte0 : $92; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $9C; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $A1; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $A6; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $AB; byte1 : $0F; byte2 : $00;),(byte0 : $B7; byte1 : $0F; byte2 : $00;),(byte0 : $90; byte1 : $0F; byte2 : $00;),(byte0 : $B5; byte1 : $0F; byte2 : $00;),
      (byte0 : $25; byte1 : $10; byte2 : $00;),(byte0 : $2E; byte1 : $10; byte2 : $00;),(byte0 : $05; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $07; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $09; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $0B; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $0D; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $11; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $3A; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),
      (byte0 : $3C; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $3E; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $3F; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $42; byte1 : $1B; byte2 : $00;),(byte0 : $35; byte1 : $1B; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $25; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $25; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $62; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $42; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $62; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $62; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $C7; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $E7; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $44; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $44; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $44; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $44; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $44; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),
      (byte0 : $12; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $12; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $30; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $30; byte1 : $03; byte2 : $00;),
      (byte0 : $28; byte1 : $02; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $29; byte1 : $02; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $46; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $66; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $47; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $67; byte1 : $00; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $48; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $27; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $00; byte2 : $00;),(byte0 : $2E; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $2E; byte1 : $03; byte2 : $00;),
      (byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $30; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $30; byte1 : $03; byte2 : $00;),(byte0 : $CF; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $EF; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $4B; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $6B; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $4B; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $6B; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $4B; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $6B; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $4C; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $6C; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $36; byte1 : $1E; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $37; byte1 : $1E; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $4C; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $6C; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),
      (byte0 : $4C; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $6C; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $4D; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $6D; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $4D; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $6D; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $4D; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $6D; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $4E; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),
      (byte0 : $D5; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $F5; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $D5; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $F5; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $4C; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $4D; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $4C; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $4D; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $50; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $70; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $50; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $70; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $5A; byte1 : $1E; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $5B; byte1 : $1E; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $52; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $53; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $53; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $5A; byte1 : $01; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $5B; byte1 : $01; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $60; byte1 : $01; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $01; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $62; byte1 : $1E; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $63; byte1 : $1E; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $54; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $54; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $54; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $54; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $24; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $24; byte1 : $03; byte2 : $00;),
      (byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $30; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $30; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $2D; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $6A; byte1 : $01; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $6B; byte1 : $01; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),
      (byte0 : $56; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $76; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $56; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $76; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $57; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $57; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $57; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $57; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $57; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $58; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $78; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),
      (byte0 : $58; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $78; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $5A; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $7A; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $5A; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $7A; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $5A; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $7A; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $00; byte2 : $00;),(byte0 : $31; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $00; byte2 : $00;),(byte0 : $08; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $00; byte2 : $00;),(byte0 : $0A; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $0A; byte1 : $03; byte2 : $00;),(byte0 : $7F; byte1 : $01; byte2 : $00;),(byte0 : $07; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $C2; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $E2; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $C2; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $E2; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $C2; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),
      (byte0 : $E2; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $C2; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $E2; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $A0; byte1 : $1E; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $A1; byte1 : $1E; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $02; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $03; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $02; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $03; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $02; byte1 : $01; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $03; byte1 : $01; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $02; byte1 : $01; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $03; byte1 : $01; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $A0; byte1 : $1E; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $A1; byte1 : $1E; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $CA; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $EA; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $CA; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $EA; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $CA; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $EA; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $CA; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $EA; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $B8; byte1 : $1E; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $1E; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),
      (byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $4F; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $D4; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $F4; byte1 : $00; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $D4; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $F4; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $D4; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $F4; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $D4; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $F4; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $CC; byte1 : $1E; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),
      (byte0 : $CD; byte1 : $1E; byte2 : $00;),(byte0 : $02; byte1 : $03; byte2 : $00;),(byte0 : $A0; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $A1; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $A0; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $A1; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $A0; byte1 : $01; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $A1; byte1 : $01; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $A0; byte1 : $01; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),
      (byte0 : $A1; byte1 : $01; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $A0; byte1 : $01; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $A1; byte1 : $01; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $55; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $75; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $AF; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $B0; byte1 : $01; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $AF; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $B0; byte1 : $01; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $AF; byte1 : $01; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $B0; byte1 : $01; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $AF; byte1 : $01; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $B0; byte1 : $01; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $AF; byte1 : $01; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),
      (byte0 : $B0; byte1 : $01; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $23; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $09; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),
      (byte0 : $79; byte1 : $00; byte2 : $00;),(byte0 : $03; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),
      (byte0 : $01; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $09; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $09; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),
      (byte0 : $09; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $B5; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $B5; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $10; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $11; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $10; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $11; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $95; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),
      (byte0 : $95; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $18; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $19; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $18; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $19; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $B7; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $B7; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $20; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $21; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $20; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $21; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $20; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $21; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $97; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $97; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $28; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $29; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $28; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $29; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $28; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $29; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $30; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $31; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $30; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $31; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $30; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $31; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $99; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $99; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $38; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $39; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $38; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $39; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $38; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $39; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $BF; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $BF; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $40; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $41; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $40; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $41; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $9F; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $9F; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $49; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),
      (byte0 : $49; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $50; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $51; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $50; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $51; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $50; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),
      (byte0 : $51; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $A5; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $59; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $C9; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $C9; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $60; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $61; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $60; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $60; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $A9; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $A9; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),
      (byte0 : $69; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $AC; byte1 : $03; byte2 : $00;),(byte0 : $B5; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $AD; byte1 : $03; byte2 : $00;),
      (byte0 : $B7; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $AE; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $AF; byte1 : $03; byte2 : $00;),(byte0 : $BF; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $CC; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $CD; byte1 : $03; byte2 : $00;),(byte0 : $C9; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $CE; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $1F; byte2 : $00;),
      (byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $01; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $02; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $03; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $04; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $05; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $06; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $07; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $1F; byte2 : $00;),
      (byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $09; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $0A; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $0B; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $0C; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $0D; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $0E; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $0F; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $20; byte1 : $1F; byte2 : $00;),
      (byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $21; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $22; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $23; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $24; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $25; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $26; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $27; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $28; byte1 : $1F; byte2 : $00;),
      (byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $29; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $2A; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $2B; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $2C; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $2D; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $2E; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $2F; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $60; byte1 : $1F; byte2 : $00;),
      (byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $62; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $63; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $66; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $67; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $68; byte1 : $1F; byte2 : $00;),
      (byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $69; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $6A; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $6B; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $6C; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $6D; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $6E; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $6F; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),
      (byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $70; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $AC; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $B1; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $B6; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $03; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $03; byte2 : $00;),
      (byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $86; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $A8; byte1 : $00; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $74; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $B7; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $AE; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $B7; byte1 : $03; byte2 : $00;),
      (byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $C6; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $95; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $88; byte1 : $03; byte2 : $00;),(byte0 : $97; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $89; byte1 : $03; byte2 : $00;),(byte0 : $97; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $BF; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $BF; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $BF; byte1 : $1F; byte2 : $00;),
      (byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $CA; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $90; byte1 : $03; byte2 : $00;),(byte0 : $B9; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $CA; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $99; byte1 : $03; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $99; byte1 : $03; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),
      (byte0 : $99; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $8A; byte1 : $03; byte2 : $00;),(byte0 : $FE; byte1 : $1F; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $FE; byte1 : $1F; byte2 : $00;),(byte0 : $01; byte1 : $03; byte2 : $00;),(byte0 : $FE; byte1 : $1F; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $CB; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $B0; byte1 : $03; byte2 : $00;),
      (byte0 : $C1; byte1 : $03; byte2 : $00;),(byte0 : $13; byte1 : $03; byte2 : $00;),(byte0 : $C1; byte1 : $03; byte2 : $00;),(byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $C5; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $CB; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $A5; byte1 : $03; byte2 : $00;),(byte0 : $06; byte1 : $03; byte2 : $00;),(byte0 : $A5; byte1 : $03; byte2 : $00;),(byte0 : $04; byte1 : $03; byte2 : $00;),(byte0 : $A5; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $8E; byte1 : $03; byte2 : $00;),(byte0 : $A1; byte1 : $03; byte2 : $00;),
      (byte0 : $14; byte1 : $03; byte2 : $00;),(byte0 : $A8; byte1 : $00; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $85; byte1 : $03; byte2 : $00;),(byte0 : $60; byte1 : $00; byte2 : $00;),(byte0 : $7C; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $C9; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $CE; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $C9; byte1 : $03; byte2 : $00;),(byte0 : $42; byte1 : $03; byte2 : $00;),(byte0 : $F6; byte1 : $1F; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $9F; byte1 : $03; byte2 : $00;),
      (byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $8C; byte1 : $03; byte2 : $00;),(byte0 : $A9; byte1 : $03; byte2 : $00;),(byte0 : $00; byte1 : $03; byte2 : $00;),(byte0 : $8F; byte1 : $03; byte2 : $00;),(byte0 : $A9; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $03; byte2 : $00;),(byte0 : $B4; byte1 : $00; byte2 : $00;),(byte0 : $02; byte1 : $20; byte2 : $00;),(byte0 : $03; byte1 : $20; byte2 : $00;),(byte0 : $A9; byte1 : $03; byte2 : $00;),(byte0 : $4B; byte1 : $00; byte2 : $00;),(byte0 : $C5; byte1 : $00; byte2 : $00;),(byte0 : $90; byte1 : $21; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $92; byte1 : $21; byte2 : $00;),
      (byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $94; byte1 : $21; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $D0; byte1 : $21; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $D4; byte1 : $21; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $D2; byte1 : $21; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $03; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $0B; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $23; byte1 : $22; byte2 : $00;),
      (byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $25; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $3C; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $43; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $45; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $48; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $3D; byte1 : $00; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $61; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $4D; byte1 : $22; byte2 : $00;),
      (byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $3C; byte1 : $00; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $3E; byte1 : $00; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $64; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $65; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $72; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $73; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $76; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $77; byte1 : $22; byte2 : $00;),
      (byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $7A; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $7B; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $82; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $83; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $86; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $87; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $A2; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $A8; byte1 : $22; byte2 : $00;),
      (byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $A9; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $AB; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $7C; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $7D; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $91; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $92; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $B2; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $B3; byte1 : $22; byte2 : $00;),
      (byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $B4; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $B5; byte1 : $22; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $08; byte1 : $30; byte2 : $00;),(byte0 : $09; byte1 : $30; byte2 : $00;),(byte0 : $DD; byte1 : $2A; byte2 : $00;),(byte0 : $38; byte1 : $03; byte2 : $00;),(byte0 : $4B; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $4D; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $4F; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $51; byte1 : $30; byte2 : $00;),
      (byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $53; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $55; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $57; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $59; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $5B; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $5D; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $5F; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $61; byte1 : $30; byte2 : $00;),
      (byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $64; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $66; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $68; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $6F; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $6F; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $72; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $72; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $75; byte1 : $30; byte2 : $00;),
      (byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $75; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $78; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $78; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $7B; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $7B; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $46; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $9D; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $AB; byte1 : $30; byte2 : $00;),
      (byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $AD; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $AF; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $B1; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $B3; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $B5; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $B7; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $B9; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $BB; byte1 : $30; byte2 : $00;),
      (byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $BD; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $BF; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $C1; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $C4; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $C6; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $C8; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $CF; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $CF; byte1 : $30; byte2 : $00;),
      (byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $D2; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $D2; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $D5; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $D5; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $D8; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $D8; byte1 : $30; byte2 : $00;),(byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $DB; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $DB; byte1 : $30; byte2 : $00;),
      (byte0 : $9A; byte1 : $30; byte2 : $00;),(byte0 : $A6; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $EF; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $F0; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $F1; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $F2; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $FD; byte1 : $30; byte2 : $00;),(byte0 : $99; byte1 : $30; byte2 : $00;),(byte0 : $48; byte1 : $8C; byte2 : $00;),(byte0 : $F4; byte1 : $66; byte2 : $00;),(byte0 : $CA; byte1 : $8E; byte2 : $00;),
      (byte0 : $C8; byte1 : $8C; byte2 : $00;),(byte0 : $D1; byte1 : $6E; byte2 : $00;),(byte0 : $32; byte1 : $4E; byte2 : $00;),(byte0 : $E5; byte1 : $53; byte2 : $00;),(byte0 : $9C; byte1 : $9F; byte2 : $00;),(byte0 : $51; byte1 : $59; byte2 : $00;),(byte0 : $D1; byte1 : $91; byte2 : $00;),(byte0 : $87; byte1 : $55; byte2 : $00;),(byte0 : $48; byte1 : $59; byte2 : $00;),(byte0 : $F6; byte1 : $61; byte2 : $00;),(byte0 : $69; byte1 : $76; byte2 : $00;),(byte0 : $85; byte1 : $7F; byte2 : $00;),(byte0 : $3F; byte1 : $86; byte2 : $00;),(byte0 : $BA; byte1 : $87; byte2 : $00;),(byte0 : $F8; byte1 : $88; byte2 : $00;),(byte0 : $8F; byte1 : $90; byte2 : $00;),
      (byte0 : $02; byte1 : $6A; byte2 : $00;),(byte0 : $1B; byte1 : $6D; byte2 : $00;),(byte0 : $D9; byte1 : $70; byte2 : $00;),(byte0 : $DE; byte1 : $73; byte2 : $00;),(byte0 : $3D; byte1 : $84; byte2 : $00;),(byte0 : $6A; byte1 : $91; byte2 : $00;),(byte0 : $F1; byte1 : $99; byte2 : $00;),(byte0 : $82; byte1 : $4E; byte2 : $00;),(byte0 : $75; byte1 : $53; byte2 : $00;),(byte0 : $04; byte1 : $6B; byte2 : $00;),(byte0 : $1B; byte1 : $72; byte2 : $00;),(byte0 : $2D; byte1 : $86; byte2 : $00;),(byte0 : $1E; byte1 : $9E; byte2 : $00;),(byte0 : $50; byte1 : $5D; byte2 : $00;),(byte0 : $EB; byte1 : $6F; byte2 : $00;),(byte0 : $CD; byte1 : $85; byte2 : $00;),
      (byte0 : $64; byte1 : $89; byte2 : $00;),(byte0 : $C9; byte1 : $62; byte2 : $00;),(byte0 : $D8; byte1 : $81; byte2 : $00;),(byte0 : $1F; byte1 : $88; byte2 : $00;),(byte0 : $CA; byte1 : $5E; byte2 : $00;),(byte0 : $17; byte1 : $67; byte2 : $00;),(byte0 : $6A; byte1 : $6D; byte2 : $00;),(byte0 : $FC; byte1 : $72; byte2 : $00;),(byte0 : $CE; byte1 : $90; byte2 : $00;),(byte0 : $86; byte1 : $4F; byte2 : $00;),(byte0 : $B7; byte1 : $51; byte2 : $00;),(byte0 : $DE; byte1 : $52; byte2 : $00;),(byte0 : $C4; byte1 : $64; byte2 : $00;),(byte0 : $D3; byte1 : $6A; byte2 : $00;),(byte0 : $10; byte1 : $72; byte2 : $00;),(byte0 : $E7; byte1 : $76; byte2 : $00;),
      (byte0 : $01; byte1 : $80; byte2 : $00;),(byte0 : $06; byte1 : $86; byte2 : $00;),(byte0 : $5C; byte1 : $86; byte2 : $00;),(byte0 : $EF; byte1 : $8D; byte2 : $00;),(byte0 : $32; byte1 : $97; byte2 : $00;),(byte0 : $6F; byte1 : $9B; byte2 : $00;),(byte0 : $FA; byte1 : $9D; byte2 : $00;),(byte0 : $8C; byte1 : $78; byte2 : $00;),(byte0 : $7F; byte1 : $79; byte2 : $00;),(byte0 : $A0; byte1 : $7D; byte2 : $00;),(byte0 : $C9; byte1 : $83; byte2 : $00;),(byte0 : $04; byte1 : $93; byte2 : $00;),(byte0 : $7F; byte1 : $9E; byte2 : $00;),(byte0 : $D6; byte1 : $8A; byte2 : $00;),(byte0 : $DF; byte1 : $58; byte2 : $00;),(byte0 : $04; byte1 : $5F; byte2 : $00;),
      (byte0 : $60; byte1 : $7C; byte2 : $00;),(byte0 : $7E; byte1 : $80; byte2 : $00;),(byte0 : $62; byte1 : $72; byte2 : $00;),(byte0 : $CA; byte1 : $78; byte2 : $00;),(byte0 : $C2; byte1 : $8C; byte2 : $00;),(byte0 : $F7; byte1 : $96; byte2 : $00;),(byte0 : $D8; byte1 : $58; byte2 : $00;),(byte0 : $62; byte1 : $5C; byte2 : $00;),(byte0 : $13; byte1 : $6A; byte2 : $00;),(byte0 : $DA; byte1 : $6D; byte2 : $00;),(byte0 : $0F; byte1 : $6F; byte2 : $00;),(byte0 : $2F; byte1 : $7D; byte2 : $00;),(byte0 : $37; byte1 : $7E; byte2 : $00;),(byte0 : $4B; byte1 : $96; byte2 : $00;),(byte0 : $D2; byte1 : $52; byte2 : $00;),(byte0 : $8B; byte1 : $80; byte2 : $00;),
      (byte0 : $DC; byte1 : $51; byte2 : $00;),(byte0 : $CC; byte1 : $51; byte2 : $00;),(byte0 : $1C; byte1 : $7A; byte2 : $00;),(byte0 : $BE; byte1 : $7D; byte2 : $00;),(byte0 : $F1; byte1 : $83; byte2 : $00;),(byte0 : $75; byte1 : $96; byte2 : $00;),(byte0 : $80; byte1 : $8B; byte2 : $00;),(byte0 : $CF; byte1 : $62; byte2 : $00;),(byte0 : $FE; byte1 : $8A; byte2 : $00;),(byte0 : $39; byte1 : $4E; byte2 : $00;),(byte0 : $E7; byte1 : $5B; byte2 : $00;),(byte0 : $12; byte1 : $60; byte2 : $00;),(byte0 : $87; byte1 : $73; byte2 : $00;),(byte0 : $70; byte1 : $75; byte2 : $00;),(byte0 : $17; byte1 : $53; byte2 : $00;),(byte0 : $FB; byte1 : $78; byte2 : $00;),
      (byte0 : $BF; byte1 : $4F; byte2 : $00;),(byte0 : $A9; byte1 : $5F; byte2 : $00;),(byte0 : $0D; byte1 : $4E; byte2 : $00;),(byte0 : $CC; byte1 : $6C; byte2 : $00;),(byte0 : $78; byte1 : $65; byte2 : $00;),(byte0 : $22; byte1 : $7D; byte2 : $00;),(byte0 : $C3; byte1 : $53; byte2 : $00;),(byte0 : $5E; byte1 : $58; byte2 : $00;),(byte0 : $01; byte1 : $77; byte2 : $00;),(byte0 : $49; byte1 : $84; byte2 : $00;),(byte0 : $AA; byte1 : $8A; byte2 : $00;),(byte0 : $BA; byte1 : $6B; byte2 : $00;),(byte0 : $B0; byte1 : $8F; byte2 : $00;),(byte0 : $88; byte1 : $6C; byte2 : $00;),(byte0 : $FE; byte1 : $62; byte2 : $00;),(byte0 : $E5; byte1 : $82; byte2 : $00;),
      (byte0 : $A0; byte1 : $63; byte2 : $00;),(byte0 : $65; byte1 : $75; byte2 : $00;),(byte0 : $AE; byte1 : $4E; byte2 : $00;),(byte0 : $69; byte1 : $51; byte2 : $00;),(byte0 : $C9; byte1 : $51; byte2 : $00;),(byte0 : $81; byte1 : $68; byte2 : $00;),(byte0 : $E7; byte1 : $7C; byte2 : $00;),(byte0 : $6F; byte1 : $82; byte2 : $00;),(byte0 : $D2; byte1 : $8A; byte2 : $00;),(byte0 : $CF; byte1 : $91; byte2 : $00;),(byte0 : $F5; byte1 : $52; byte2 : $00;),(byte0 : $42; byte1 : $54; byte2 : $00;),(byte0 : $73; byte1 : $59; byte2 : $00;),(byte0 : $EC; byte1 : $5E; byte2 : $00;),(byte0 : $C5; byte1 : $65; byte2 : $00;),(byte0 : $FE; byte1 : $6F; byte2 : $00;),
      (byte0 : $2A; byte1 : $79; byte2 : $00;),(byte0 : $AD; byte1 : $95; byte2 : $00;),(byte0 : $6A; byte1 : $9A; byte2 : $00;),(byte0 : $97; byte1 : $9E; byte2 : $00;),(byte0 : $CE; byte1 : $9E; byte2 : $00;),(byte0 : $9B; byte1 : $52; byte2 : $00;),(byte0 : $C6; byte1 : $66; byte2 : $00;),(byte0 : $77; byte1 : $6B; byte2 : $00;),(byte0 : $62; byte1 : $8F; byte2 : $00;),(byte0 : $74; byte1 : $5E; byte2 : $00;),(byte0 : $90; byte1 : $61; byte2 : $00;),(byte0 : $00; byte1 : $62; byte2 : $00;),(byte0 : $9A; byte1 : $64; byte2 : $00;),(byte0 : $23; byte1 : $6F; byte2 : $00;),(byte0 : $49; byte1 : $71; byte2 : $00;),(byte0 : $89; byte1 : $74; byte2 : $00;),
      (byte0 : $CA; byte1 : $79; byte2 : $00;),(byte0 : $F4; byte1 : $7D; byte2 : $00;),(byte0 : $6F; byte1 : $80; byte2 : $00;),(byte0 : $26; byte1 : $8F; byte2 : $00;),(byte0 : $EE; byte1 : $84; byte2 : $00;),(byte0 : $23; byte1 : $90; byte2 : $00;),(byte0 : $4A; byte1 : $93; byte2 : $00;),(byte0 : $17; byte1 : $52; byte2 : $00;),(byte0 : $A3; byte1 : $52; byte2 : $00;),(byte0 : $BD; byte1 : $54; byte2 : $00;),(byte0 : $C8; byte1 : $70; byte2 : $00;),(byte0 : $C2; byte1 : $88; byte2 : $00;),(byte0 : $C9; byte1 : $5E; byte2 : $00;),(byte0 : $F5; byte1 : $5F; byte2 : $00;),(byte0 : $7B; byte1 : $63; byte2 : $00;),(byte0 : $AE; byte1 : $6B; byte2 : $00;),
      (byte0 : $3E; byte1 : $7C; byte2 : $00;),(byte0 : $75; byte1 : $73; byte2 : $00;),(byte0 : $E4; byte1 : $4E; byte2 : $00;),(byte0 : $F9; byte1 : $56; byte2 : $00;),(byte0 : $BA; byte1 : $5D; byte2 : $00;),(byte0 : $1C; byte1 : $60; byte2 : $00;),(byte0 : $B2; byte1 : $73; byte2 : $00;),(byte0 : $69; byte1 : $74; byte2 : $00;),(byte0 : $9A; byte1 : $7F; byte2 : $00;),(byte0 : $46; byte1 : $80; byte2 : $00;),(byte0 : $34; byte1 : $92; byte2 : $00;),(byte0 : $F6; byte1 : $96; byte2 : $00;),(byte0 : $48; byte1 : $97; byte2 : $00;),(byte0 : $18; byte1 : $98; byte2 : $00;),(byte0 : $8B; byte1 : $4F; byte2 : $00;),(byte0 : $AE; byte1 : $79; byte2 : $00;),
      (byte0 : $B4; byte1 : $91; byte2 : $00;),(byte0 : $B8; byte1 : $96; byte2 : $00;),(byte0 : $E1; byte1 : $60; byte2 : $00;),(byte0 : $86; byte1 : $4E; byte2 : $00;),(byte0 : $DA; byte1 : $50; byte2 : $00;),(byte0 : $EE; byte1 : $5B; byte2 : $00;),(byte0 : $3F; byte1 : $5C; byte2 : $00;),(byte0 : $99; byte1 : $65; byte2 : $00;),(byte0 : $CE; byte1 : $71; byte2 : $00;),(byte0 : $42; byte1 : $76; byte2 : $00;),(byte0 : $FC; byte1 : $84; byte2 : $00;),(byte0 : $7C; byte1 : $90; byte2 : $00;),(byte0 : $8D; byte1 : $9F; byte2 : $00;),(byte0 : $88; byte1 : $66; byte2 : $00;),(byte0 : $2E; byte1 : $96; byte2 : $00;),(byte0 : $89; byte1 : $52; byte2 : $00;),
      (byte0 : $7B; byte1 : $67; byte2 : $00;),(byte0 : $F3; byte1 : $67; byte2 : $00;),(byte0 : $41; byte1 : $6D; byte2 : $00;),(byte0 : $9C; byte1 : $6E; byte2 : $00;),(byte0 : $09; byte1 : $74; byte2 : $00;),(byte0 : $59; byte1 : $75; byte2 : $00;),(byte0 : $6B; byte1 : $78; byte2 : $00;),(byte0 : $10; byte1 : $7D; byte2 : $00;),(byte0 : $5E; byte1 : $98; byte2 : $00;),(byte0 : $6D; byte1 : $51; byte2 : $00;),(byte0 : $2E; byte1 : $62; byte2 : $00;),(byte0 : $78; byte1 : $96; byte2 : $00;),(byte0 : $2B; byte1 : $50; byte2 : $00;),(byte0 : $19; byte1 : $5D; byte2 : $00;),(byte0 : $EA; byte1 : $6D; byte2 : $00;),(byte0 : $2A; byte1 : $8F; byte2 : $00;),
      (byte0 : $8B; byte1 : $5F; byte2 : $00;),(byte0 : $44; byte1 : $61; byte2 : $00;),(byte0 : $17; byte1 : $68; byte2 : $00;),(byte0 : $86; byte1 : $96; byte2 : $00;),(byte0 : $29; byte1 : $52; byte2 : $00;),(byte0 : $0F; byte1 : $54; byte2 : $00;),(byte0 : $65; byte1 : $5C; byte2 : $00;),(byte0 : $13; byte1 : $66; byte2 : $00;),(byte0 : $4E; byte1 : $67; byte2 : $00;),(byte0 : $A8; byte1 : $68; byte2 : $00;),(byte0 : $E5; byte1 : $6C; byte2 : $00;),(byte0 : $06; byte1 : $74; byte2 : $00;),(byte0 : $E2; byte1 : $75; byte2 : $00;),(byte0 : $79; byte1 : $7F; byte2 : $00;),(byte0 : $CF; byte1 : $88; byte2 : $00;),(byte0 : $E1; byte1 : $88; byte2 : $00;),
      (byte0 : $CC; byte1 : $91; byte2 : $00;),(byte0 : $E2; byte1 : $96; byte2 : $00;),(byte0 : $3F; byte1 : $53; byte2 : $00;),(byte0 : $BA; byte1 : $6E; byte2 : $00;),(byte0 : $1D; byte1 : $54; byte2 : $00;),(byte0 : $D0; byte1 : $71; byte2 : $00;),(byte0 : $98; byte1 : $74; byte2 : $00;),(byte0 : $FA; byte1 : $85; byte2 : $00;),(byte0 : $A3; byte1 : $96; byte2 : $00;),(byte0 : $57; byte1 : $9C; byte2 : $00;),(byte0 : $9F; byte1 : $9E; byte2 : $00;),(byte0 : $97; byte1 : $67; byte2 : $00;),(byte0 : $CB; byte1 : $6D; byte2 : $00;),(byte0 : $E8; byte1 : $81; byte2 : $00;),(byte0 : $CB; byte1 : $7A; byte2 : $00;),(byte0 : $20; byte1 : $7B; byte2 : $00;),
      (byte0 : $92; byte1 : $7C; byte2 : $00;),(byte0 : $C0; byte1 : $72; byte2 : $00;),(byte0 : $99; byte1 : $70; byte2 : $00;),(byte0 : $58; byte1 : $8B; byte2 : $00;),(byte0 : $C0; byte1 : $4E; byte2 : $00;),(byte0 : $36; byte1 : $83; byte2 : $00;),(byte0 : $3A; byte1 : $52; byte2 : $00;),(byte0 : $07; byte1 : $52; byte2 : $00;),(byte0 : $A6; byte1 : $5E; byte2 : $00;),(byte0 : $D3; byte1 : $62; byte2 : $00;),(byte0 : $D6; byte1 : $7C; byte2 : $00;),(byte0 : $85; byte1 : $5B; byte2 : $00;),(byte0 : $1E; byte1 : $6D; byte2 : $00;),(byte0 : $B4; byte1 : $66; byte2 : $00;),(byte0 : $3B; byte1 : $8F; byte2 : $00;),(byte0 : $4C; byte1 : $88; byte2 : $00;),
      (byte0 : $4D; byte1 : $96; byte2 : $00;),(byte0 : $8B; byte1 : $89; byte2 : $00;),(byte0 : $D3; byte1 : $5E; byte2 : $00;),(byte0 : $40; byte1 : $51; byte2 : $00;),(byte0 : $C0; byte1 : $55; byte2 : $00;),(byte0 : $5A; byte1 : $58; byte2 : $00;),(byte0 : $74; byte1 : $66; byte2 : $00;),(byte0 : $DE; byte1 : $51; byte2 : $00;),(byte0 : $2A; byte1 : $73; byte2 : $00;),(byte0 : $CA; byte1 : $76; byte2 : $00;),(byte0 : $3C; byte1 : $79; byte2 : $00;),(byte0 : $5E; byte1 : $79; byte2 : $00;),(byte0 : $65; byte1 : $79; byte2 : $00;),(byte0 : $8F; byte1 : $79; byte2 : $00;),(byte0 : $56; byte1 : $97; byte2 : $00;),(byte0 : $BE; byte1 : $7C; byte2 : $00;),
      (byte0 : $BD; byte1 : $7F; byte2 : $00;),(byte0 : $12; byte1 : $86; byte2 : $00;),(byte0 : $F8; byte1 : $8A; byte2 : $00;),(byte0 : $38; byte1 : $90; byte2 : $00;),(byte0 : $FD; byte1 : $90; byte2 : $00;),(byte0 : $EF; byte1 : $98; byte2 : $00;),(byte0 : $FC; byte1 : $98; byte2 : $00;),(byte0 : $28; byte1 : $99; byte2 : $00;),(byte0 : $B4; byte1 : $9D; byte2 : $00;),(byte0 : $DE; byte1 : $90; byte2 : $00;),(byte0 : $B7; byte1 : $96; byte2 : $00;),(byte0 : $AE; byte1 : $4F; byte2 : $00;),(byte0 : $E7; byte1 : $50; byte2 : $00;),(byte0 : $4D; byte1 : $51; byte2 : $00;),(byte0 : $C9; byte1 : $52; byte2 : $00;),(byte0 : $E4; byte1 : $52; byte2 : $00;),
      (byte0 : $51; byte1 : $53; byte2 : $00;),(byte0 : $9D; byte1 : $55; byte2 : $00;),(byte0 : $06; byte1 : $56; byte2 : $00;),(byte0 : $68; byte1 : $56; byte2 : $00;),(byte0 : $40; byte1 : $58; byte2 : $00;),(byte0 : $A8; byte1 : $58; byte2 : $00;),(byte0 : $64; byte1 : $5C; byte2 : $00;),(byte0 : $6E; byte1 : $5C; byte2 : $00;),(byte0 : $94; byte1 : $60; byte2 : $00;),(byte0 : $68; byte1 : $61; byte2 : $00;),(byte0 : $8E; byte1 : $61; byte2 : $00;),(byte0 : $F2; byte1 : $61; byte2 : $00;),(byte0 : $4F; byte1 : $65; byte2 : $00;),(byte0 : $E2; byte1 : $65; byte2 : $00;),(byte0 : $91; byte1 : $66; byte2 : $00;),(byte0 : $85; byte1 : $68; byte2 : $00;),
      (byte0 : $77; byte1 : $6D; byte2 : $00;),(byte0 : $1A; byte1 : $6E; byte2 : $00;),(byte0 : $22; byte1 : $6F; byte2 : $00;),(byte0 : $6E; byte1 : $71; byte2 : $00;),(byte0 : $2B; byte1 : $72; byte2 : $00;),(byte0 : $22; byte1 : $74; byte2 : $00;),(byte0 : $91; byte1 : $78; byte2 : $00;),(byte0 : $3E; byte1 : $79; byte2 : $00;),(byte0 : $49; byte1 : $79; byte2 : $00;),(byte0 : $48; byte1 : $79; byte2 : $00;),(byte0 : $50; byte1 : $79; byte2 : $00;),(byte0 : $56; byte1 : $79; byte2 : $00;),(byte0 : $5D; byte1 : $79; byte2 : $00;),(byte0 : $8D; byte1 : $79; byte2 : $00;),(byte0 : $8E; byte1 : $79; byte2 : $00;),(byte0 : $40; byte1 : $7A; byte2 : $00;),
      (byte0 : $81; byte1 : $7A; byte2 : $00;),(byte0 : $C0; byte1 : $7B; byte2 : $00;),(byte0 : $09; byte1 : $7E; byte2 : $00;),(byte0 : $41; byte1 : $7E; byte2 : $00;),(byte0 : $72; byte1 : $7F; byte2 : $00;),(byte0 : $05; byte1 : $80; byte2 : $00;),(byte0 : $ED; byte1 : $81; byte2 : $00;),(byte0 : $79; byte1 : $82; byte2 : $00;),(byte0 : $57; byte1 : $84; byte2 : $00;),(byte0 : $10; byte1 : $89; byte2 : $00;),(byte0 : $96; byte1 : $89; byte2 : $00;),(byte0 : $01; byte1 : $8B; byte2 : $00;),(byte0 : $39; byte1 : $8B; byte2 : $00;),(byte0 : $D3; byte1 : $8C; byte2 : $00;),(byte0 : $08; byte1 : $8D; byte2 : $00;),(byte0 : $B6; byte1 : $8F; byte2 : $00;),
      (byte0 : $E3; byte1 : $96; byte2 : $00;),(byte0 : $FF; byte1 : $97; byte2 : $00;),(byte0 : $3B; byte1 : $98; byte2 : $00;),(byte0 : $75; byte1 : $60; byte2 : $00;),(byte0 : $EE; byte1 : $42; byte2 : $02;),(byte0 : $18; byte1 : $82; byte2 : $00;),(byte0 : $26; byte1 : $4E; byte2 : $00;),(byte0 : $B5; byte1 : $51; byte2 : $00;),(byte0 : $68; byte1 : $51; byte2 : $00;),(byte0 : $80; byte1 : $4F; byte2 : $00;),(byte0 : $45; byte1 : $51; byte2 : $00;),(byte0 : $80; byte1 : $51; byte2 : $00;),(byte0 : $C7; byte1 : $52; byte2 : $00;),(byte0 : $FA; byte1 : $52; byte2 : $00;),(byte0 : $55; byte1 : $55; byte2 : $00;),(byte0 : $99; byte1 : $55; byte2 : $00;),
      (byte0 : $E2; byte1 : $55; byte2 : $00;),(byte0 : $B3; byte1 : $58; byte2 : $00;),(byte0 : $44; byte1 : $59; byte2 : $00;),(byte0 : $54; byte1 : $59; byte2 : $00;),(byte0 : $62; byte1 : $5A; byte2 : $00;),(byte0 : $28; byte1 : $5B; byte2 : $00;),(byte0 : $D2; byte1 : $5E; byte2 : $00;),(byte0 : $D9; byte1 : $5E; byte2 : $00;),(byte0 : $69; byte1 : $5F; byte2 : $00;),(byte0 : $AD; byte1 : $5F; byte2 : $00;),(byte0 : $D8; byte1 : $60; byte2 : $00;),(byte0 : $4E; byte1 : $61; byte2 : $00;),(byte0 : $08; byte1 : $61; byte2 : $00;),(byte0 : $60; byte1 : $61; byte2 : $00;),(byte0 : $34; byte1 : $62; byte2 : $00;),(byte0 : $C4; byte1 : $63; byte2 : $00;),
      (byte0 : $1C; byte1 : $64; byte2 : $00;),(byte0 : $52; byte1 : $64; byte2 : $00;),(byte0 : $56; byte1 : $65; byte2 : $00;),(byte0 : $1B; byte1 : $67; byte2 : $00;),(byte0 : $56; byte1 : $67; byte2 : $00;),(byte0 : $79; byte1 : $6B; byte2 : $00;),(byte0 : $DB; byte1 : $6E; byte2 : $00;),(byte0 : $CB; byte1 : $6E; byte2 : $00;),(byte0 : $1E; byte1 : $70; byte2 : $00;),(byte0 : $A7; byte1 : $77; byte2 : $00;),(byte0 : $35; byte1 : $72; byte2 : $00;),(byte0 : $AF; byte1 : $72; byte2 : $00;),(byte0 : $71; byte1 : $74; byte2 : $00;),(byte0 : $06; byte1 : $75; byte2 : $00;),(byte0 : $3B; byte1 : $75; byte2 : $00;),(byte0 : $1D; byte1 : $76; byte2 : $00;),
      (byte0 : $1F; byte1 : $76; byte2 : $00;),(byte0 : $DB; byte1 : $76; byte2 : $00;),(byte0 : $F4; byte1 : $76; byte2 : $00;),(byte0 : $4A; byte1 : $77; byte2 : $00;),(byte0 : $40; byte1 : $77; byte2 : $00;),(byte0 : $CC; byte1 : $78; byte2 : $00;),(byte0 : $B1; byte1 : $7A; byte2 : $00;),(byte0 : $7B; byte1 : $7C; byte2 : $00;),(byte0 : $5B; byte1 : $7D; byte2 : $00;),(byte0 : $3E; byte1 : $7F; byte2 : $00;),(byte0 : $52; byte1 : $83; byte2 : $00;),(byte0 : $EF; byte1 : $83; byte2 : $00;),(byte0 : $79; byte1 : $87; byte2 : $00;),(byte0 : $41; byte1 : $89; byte2 : $00;),(byte0 : $86; byte1 : $89; byte2 : $00;),(byte0 : $BF; byte1 : $8A; byte2 : $00;),
      (byte0 : $CB; byte1 : $8A; byte2 : $00;),(byte0 : $ED; byte1 : $8A; byte2 : $00;),(byte0 : $8A; byte1 : $8B; byte2 : $00;),(byte0 : $38; byte1 : $8F; byte2 : $00;),(byte0 : $72; byte1 : $90; byte2 : $00;),(byte0 : $99; byte1 : $91; byte2 : $00;),(byte0 : $76; byte1 : $92; byte2 : $00;),(byte0 : $7C; byte1 : $96; byte2 : $00;),(byte0 : $DB; byte1 : $97; byte2 : $00;),(byte0 : $0B; byte1 : $98; byte2 : $00;),(byte0 : $12; byte1 : $9B; byte2 : $00;),(byte0 : $4A; byte1 : $28; byte2 : $02;),(byte0 : $44; byte1 : $28; byte2 : $02;),(byte0 : $D5; byte1 : $33; byte2 : $02;),(byte0 : $9D; byte1 : $3B; byte2 : $00;),(byte0 : $18; byte1 : $40; byte2 : $00;),
      (byte0 : $39; byte1 : $40; byte2 : $00;),(byte0 : $49; byte1 : $52; byte2 : $02;),(byte0 : $D0; byte1 : $5C; byte2 : $02;),(byte0 : $D3; byte1 : $7E; byte2 : $02;),(byte0 : $43; byte1 : $9F; byte2 : $00;),(byte0 : $8E; byte1 : $9F; byte2 : $00;),(byte0 : $D9; byte1 : $05; byte2 : $00;),(byte0 : $B4; byte1 : $05; byte2 : $00;),(byte0 : $F2; byte1 : $05; byte2 : $00;),(byte0 : $B7; byte1 : $05; byte2 : $00;),(byte0 : $E9; byte1 : $05; byte2 : $00;),(byte0 : $C1; byte1 : $05; byte2 : $00;),(byte0 : $E9; byte1 : $05; byte2 : $00;),(byte0 : $C2; byte1 : $05; byte2 : $00;),(byte0 : $49; byte1 : $FB; byte2 : $00;),(byte0 : $C1; byte1 : $05; byte2 : $00;),
      (byte0 : $49; byte1 : $FB; byte2 : $00;),(byte0 : $C2; byte1 : $05; byte2 : $00;),(byte0 : $D0; byte1 : $05; byte2 : $00;),(byte0 : $B7; byte1 : $05; byte2 : $00;),(byte0 : $D0; byte1 : $05; byte2 : $00;),(byte0 : $B8; byte1 : $05; byte2 : $00;),(byte0 : $D0; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D1; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D2; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D3; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D4; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),
      (byte0 : $D5; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D6; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D8; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D9; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $DA; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $DB; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $DC; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $DE; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),
      (byte0 : $E0; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $E1; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $E3; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $E4; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $E6; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $E7; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $E8; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $E9; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),
      (byte0 : $EA; byte1 : $05; byte2 : $00;),(byte0 : $BC; byte1 : $05; byte2 : $00;),(byte0 : $D5; byte1 : $05; byte2 : $00;),(byte0 : $B9; byte1 : $05; byte2 : $00;),(byte0 : $D1; byte1 : $05; byte2 : $00;),(byte0 : $BF; byte1 : $05; byte2 : $00;),(byte0 : $DB; byte1 : $05; byte2 : $00;),(byte0 : $BF; byte1 : $05; byte2 : $00;),(byte0 : $E4; byte1 : $05; byte2 : $00;),(byte0 : $BF; byte1 : $05; byte2 : $00;),(byte0 : $99; byte1 : $10; byte2 : $01;),(byte0 : $BA; byte1 : $10; byte2 : $01;),(byte0 : $9B; byte1 : $10; byte2 : $01;),(byte0 : $BA; byte1 : $10; byte2 : $01;),(byte0 : $A5; byte1 : $10; byte2 : $01;),(byte0 : $BA; byte1 : $10; byte2 : $01;),
      (byte0 : $31; byte1 : $11; byte2 : $01;),(byte0 : $27; byte1 : $11; byte2 : $01;),(byte0 : $32; byte1 : $11; byte2 : $01;),(byte0 : $27; byte1 : $11; byte2 : $01;),(byte0 : $47; byte1 : $13; byte2 : $01;),(byte0 : $3E; byte1 : $13; byte2 : $01;),(byte0 : $47; byte1 : $13; byte2 : $01;),(byte0 : $57; byte1 : $13; byte2 : $01;),(byte0 : $B9; byte1 : $14; byte2 : $01;),(byte0 : $BA; byte1 : $14; byte2 : $01;),(byte0 : $B9; byte1 : $14; byte2 : $01;),(byte0 : $B0; byte1 : $14; byte2 : $01;),(byte0 : $B9; byte1 : $14; byte2 : $01;),(byte0 : $BD; byte1 : $14; byte2 : $01;),(byte0 : $B8; byte1 : $15; byte2 : $01;),(byte0 : $AF; byte1 : $15; byte2 : $01;),
      (byte0 : $B9; byte1 : $15; byte2 : $01;),(byte0 : $AF; byte1 : $15; byte2 : $01;),(byte0 : $57; byte1 : $D1; byte2 : $01;),(byte0 : $65; byte1 : $D1; byte2 : $01;),(byte0 : $58; byte1 : $D1; byte2 : $01;),(byte0 : $65; byte1 : $D1; byte2 : $01;),(byte0 : $5F; byte1 : $D1; byte2 : $01;),(byte0 : $6E; byte1 : $D1; byte2 : $01;),(byte0 : $5F; byte1 : $D1; byte2 : $01;),(byte0 : $6F; byte1 : $D1; byte2 : $01;),(byte0 : $5F; byte1 : $D1; byte2 : $01;),(byte0 : $70; byte1 : $D1; byte2 : $01;),(byte0 : $5F; byte1 : $D1; byte2 : $01;),(byte0 : $71; byte1 : $D1; byte2 : $01;),(byte0 : $5F; byte1 : $D1; byte2 : $01;),(byte0 : $72; byte1 : $D1; byte2 : $01;),
      (byte0 : $B9; byte1 : $D1; byte2 : $01;),(byte0 : $65; byte1 : $D1; byte2 : $01;),(byte0 : $BA; byte1 : $D1; byte2 : $01;),(byte0 : $65; byte1 : $D1; byte2 : $01;),(byte0 : $BB; byte1 : $D1; byte2 : $01;),(byte0 : $6E; byte1 : $D1; byte2 : $01;),(byte0 : $BC; byte1 : $D1; byte2 : $01;),(byte0 : $6E; byte1 : $D1; byte2 : $01;),(byte0 : $BB; byte1 : $D1; byte2 : $01;),(byte0 : $6F; byte1 : $D1; byte2 : $01;),(byte0 : $BC; byte1 : $D1; byte2 : $01;),(byte0 : $6F; byte1 : $D1; byte2 : $01;),(byte0 : $3D; byte1 : $4E; byte2 : $00;),(byte0 : $38; byte1 : $4E; byte2 : $00;),(byte0 : $41; byte1 : $4E; byte2 : $00;),(byte0 : $22; byte1 : $01; byte2 : $02;),
      (byte0 : $60; byte1 : $4F; byte2 : $00;),(byte0 : $BB; byte1 : $4F; byte2 : $00;),(byte0 : $02; byte1 : $50; byte2 : $00;),(byte0 : $7A; byte1 : $50; byte2 : $00;),(byte0 : $99; byte1 : $50; byte2 : $00;),(byte0 : $CF; byte1 : $50; byte2 : $00;),(byte0 : $9E; byte1 : $34; byte2 : $00;),(byte0 : $3A; byte1 : $06; byte2 : $02;),(byte0 : $54; byte1 : $51; byte2 : $00;),(byte0 : $64; byte1 : $51; byte2 : $00;),(byte0 : $77; byte1 : $51; byte2 : $00;),(byte0 : $1C; byte1 : $05; byte2 : $02;),(byte0 : $B9; byte1 : $34; byte2 : $00;),(byte0 : $67; byte1 : $51; byte2 : $00;),(byte0 : $8D; byte1 : $51; byte2 : $00;),(byte0 : $4B; byte1 : $05; byte2 : $02;),
      (byte0 : $97; byte1 : $51; byte2 : $00;),(byte0 : $A4; byte1 : $51; byte2 : $00;),(byte0 : $CC; byte1 : $4E; byte2 : $00;),(byte0 : $AC; byte1 : $51; byte2 : $00;),(byte0 : $DF; byte1 : $91; byte2 : $02;),(byte0 : $F5; byte1 : $51; byte2 : $00;),(byte0 : $03; byte1 : $52; byte2 : $00;),(byte0 : $DF; byte1 : $34; byte2 : $00;),(byte0 : $3B; byte1 : $52; byte2 : $00;),(byte0 : $46; byte1 : $52; byte2 : $00;),(byte0 : $72; byte1 : $52; byte2 : $00;),(byte0 : $77; byte1 : $52; byte2 : $00;),(byte0 : $15; byte1 : $35; byte2 : $00;),(byte0 : $05; byte1 : $53; byte2 : $00;),(byte0 : $06; byte1 : $53; byte2 : $00;),(byte0 : $49; byte1 : $53; byte2 : $00;),
      (byte0 : $5A; byte1 : $53; byte2 : $00;),(byte0 : $73; byte1 : $53; byte2 : $00;),(byte0 : $7D; byte1 : $53; byte2 : $00;),(byte0 : $7F; byte1 : $53; byte2 : $00;),(byte0 : $2C; byte1 : $0A; byte2 : $02;),(byte0 : $70; byte1 : $70; byte2 : $00;),(byte0 : $CA; byte1 : $53; byte2 : $00;),(byte0 : $DF; byte1 : $53; byte2 : $00;),(byte0 : $63; byte1 : $0B; byte2 : $02;),(byte0 : $EB; byte1 : $53; byte2 : $00;),(byte0 : $F1; byte1 : $53; byte2 : $00;),(byte0 : $06; byte1 : $54; byte2 : $00;),(byte0 : $9E; byte1 : $54; byte2 : $00;),(byte0 : $38; byte1 : $54; byte2 : $00;),(byte0 : $48; byte1 : $54; byte2 : $00;),(byte0 : $68; byte1 : $54; byte2 : $00;),
      (byte0 : $A2; byte1 : $54; byte2 : $00;),(byte0 : $F6; byte1 : $54; byte2 : $00;),(byte0 : $10; byte1 : $55; byte2 : $00;),(byte0 : $53; byte1 : $55; byte2 : $00;),(byte0 : $63; byte1 : $55; byte2 : $00;),(byte0 : $84; byte1 : $55; byte2 : $00;),(byte0 : $AB; byte1 : $55; byte2 : $00;),(byte0 : $B3; byte1 : $55; byte2 : $00;),(byte0 : $C2; byte1 : $55; byte2 : $00;),(byte0 : $16; byte1 : $57; byte2 : $00;),(byte0 : $17; byte1 : $57; byte2 : $00;),(byte0 : $51; byte1 : $56; byte2 : $00;),(byte0 : $74; byte1 : $56; byte2 : $00;),(byte0 : $EE; byte1 : $58; byte2 : $00;),(byte0 : $CE; byte1 : $57; byte2 : $00;),(byte0 : $F4; byte1 : $57; byte2 : $00;),
      (byte0 : $0D; byte1 : $58; byte2 : $00;),(byte0 : $8B; byte1 : $57; byte2 : $00;),(byte0 : $32; byte1 : $58; byte2 : $00;),(byte0 : $31; byte1 : $58; byte2 : $00;),(byte0 : $AC; byte1 : $58; byte2 : $00;),(byte0 : $E4; byte1 : $14; byte2 : $02;),(byte0 : $F2; byte1 : $58; byte2 : $00;),(byte0 : $F7; byte1 : $58; byte2 : $00;),(byte0 : $06; byte1 : $59; byte2 : $00;),(byte0 : $1A; byte1 : $59; byte2 : $00;),(byte0 : $22; byte1 : $59; byte2 : $00;),(byte0 : $62; byte1 : $59; byte2 : $00;),(byte0 : $A8; byte1 : $16; byte2 : $02;),(byte0 : $EA; byte1 : $16; byte2 : $02;),(byte0 : $EC; byte1 : $59; byte2 : $00;),(byte0 : $1B; byte1 : $5A; byte2 : $00;),
      (byte0 : $27; byte1 : $5A; byte2 : $00;),(byte0 : $D8; byte1 : $59; byte2 : $00;),(byte0 : $66; byte1 : $5A; byte2 : $00;),(byte0 : $EE; byte1 : $36; byte2 : $00;),(byte0 : $FC; byte1 : $36; byte2 : $00;),(byte0 : $08; byte1 : $5B; byte2 : $00;),(byte0 : $3E; byte1 : $5B; byte2 : $00;),(byte0 : $C8; byte1 : $19; byte2 : $02;),(byte0 : $C3; byte1 : $5B; byte2 : $00;),(byte0 : $D8; byte1 : $5B; byte2 : $00;),(byte0 : $F3; byte1 : $5B; byte2 : $00;),(byte0 : $18; byte1 : $1B; byte2 : $02;),(byte0 : $FF; byte1 : $5B; byte2 : $00;),(byte0 : $06; byte1 : $5C; byte2 : $00;),(byte0 : $53; byte1 : $5F; byte2 : $00;),(byte0 : $22; byte1 : $5C; byte2 : $00;),
      (byte0 : $81; byte1 : $37; byte2 : $00;),(byte0 : $60; byte1 : $5C; byte2 : $00;),(byte0 : $C0; byte1 : $5C; byte2 : $00;),(byte0 : $8D; byte1 : $5C; byte2 : $00;),(byte0 : $E4; byte1 : $1D; byte2 : $02;),(byte0 : $43; byte1 : $5D; byte2 : $00;),(byte0 : $E6; byte1 : $1D; byte2 : $02;),(byte0 : $6E; byte1 : $5D; byte2 : $00;),(byte0 : $6B; byte1 : $5D; byte2 : $00;),(byte0 : $7C; byte1 : $5D; byte2 : $00;),(byte0 : $E1; byte1 : $5D; byte2 : $00;),(byte0 : $E2; byte1 : $5D; byte2 : $00;),(byte0 : $2F; byte1 : $38; byte2 : $00;),(byte0 : $FD; byte1 : $5D; byte2 : $00;),(byte0 : $28; byte1 : $5E; byte2 : $00;),(byte0 : $3D; byte1 : $5E; byte2 : $00;),
      (byte0 : $69; byte1 : $5E; byte2 : $00;),(byte0 : $62; byte1 : $38; byte2 : $00;),(byte0 : $83; byte1 : $21; byte2 : $02;),(byte0 : $7C; byte1 : $38; byte2 : $00;),(byte0 : $B0; byte1 : $5E; byte2 : $00;),(byte0 : $B3; byte1 : $5E; byte2 : $00;),(byte0 : $B6; byte1 : $5E; byte2 : $00;),(byte0 : $92; byte1 : $A3; byte2 : $02;),(byte0 : $FE; byte1 : $5E; byte2 : $00;),(byte0 : $31; byte1 : $23; byte2 : $02;),(byte0 : $01; byte1 : $82; byte2 : $00;),(byte0 : $22; byte1 : $5F; byte2 : $00;),(byte0 : $C7; byte1 : $38; byte2 : $00;),(byte0 : $B8; byte1 : $32; byte2 : $02;),(byte0 : $DA; byte1 : $61; byte2 : $02;),(byte0 : $62; byte1 : $5F; byte2 : $00;),
      (byte0 : $6B; byte1 : $5F; byte2 : $00;),(byte0 : $E3; byte1 : $38; byte2 : $00;),(byte0 : $9A; byte1 : $5F; byte2 : $00;),(byte0 : $CD; byte1 : $5F; byte2 : $00;),(byte0 : $D7; byte1 : $5F; byte2 : $00;),(byte0 : $F9; byte1 : $5F; byte2 : $00;),(byte0 : $81; byte1 : $60; byte2 : $00;),(byte0 : $3A; byte1 : $39; byte2 : $00;),(byte0 : $1C; byte1 : $39; byte2 : $00;),(byte0 : $D4; byte1 : $26; byte2 : $02;),(byte0 : $C7; byte1 : $60; byte2 : $00;),(byte0 : $48; byte1 : $61; byte2 : $00;),(byte0 : $4C; byte1 : $61; byte2 : $00;),(byte0 : $7A; byte1 : $61; byte2 : $00;),(byte0 : $B2; byte1 : $61; byte2 : $00;),(byte0 : $A4; byte1 : $61; byte2 : $00;),
      (byte0 : $AF; byte1 : $61; byte2 : $00;),(byte0 : $DE; byte1 : $61; byte2 : $00;),(byte0 : $10; byte1 : $62; byte2 : $00;),(byte0 : $1B; byte1 : $62; byte2 : $00;),(byte0 : $5D; byte1 : $62; byte2 : $00;),(byte0 : $B1; byte1 : $62; byte2 : $00;),(byte0 : $D4; byte1 : $62; byte2 : $00;),(byte0 : $50; byte1 : $63; byte2 : $00;),(byte0 : $0C; byte1 : $2B; byte2 : $02;),(byte0 : $3D; byte1 : $63; byte2 : $00;),(byte0 : $FC; byte1 : $62; byte2 : $00;),(byte0 : $68; byte1 : $63; byte2 : $00;),(byte0 : $83; byte1 : $63; byte2 : $00;),(byte0 : $E4; byte1 : $63; byte2 : $00;),(byte0 : $F1; byte1 : $2B; byte2 : $02;),(byte0 : $22; byte1 : $64; byte2 : $00;),
      (byte0 : $C5; byte1 : $63; byte2 : $00;),(byte0 : $A9; byte1 : $63; byte2 : $00;),(byte0 : $2E; byte1 : $3A; byte2 : $00;),(byte0 : $69; byte1 : $64; byte2 : $00;),(byte0 : $7E; byte1 : $64; byte2 : $00;),(byte0 : $9D; byte1 : $64; byte2 : $00;),(byte0 : $77; byte1 : $64; byte2 : $00;),(byte0 : $6C; byte1 : $3A; byte2 : $00;),(byte0 : $6C; byte1 : $65; byte2 : $00;),(byte0 : $0A; byte1 : $30; byte2 : $02;),(byte0 : $E3; byte1 : $65; byte2 : $00;),(byte0 : $F8; byte1 : $66; byte2 : $00;),(byte0 : $49; byte1 : $66; byte2 : $00;),(byte0 : $19; byte1 : $3B; byte2 : $00;),(byte0 : $08; byte1 : $3B; byte2 : $00;),(byte0 : $E4; byte1 : $3A; byte2 : $00;),
      (byte0 : $92; byte1 : $51; byte2 : $00;),(byte0 : $95; byte1 : $51; byte2 : $00;),(byte0 : $00; byte1 : $67; byte2 : $00;),(byte0 : $9C; byte1 : $66; byte2 : $00;),(byte0 : $AD; byte1 : $80; byte2 : $00;),(byte0 : $D9; byte1 : $43; byte2 : $00;),(byte0 : $21; byte1 : $67; byte2 : $00;),(byte0 : $5E; byte1 : $67; byte2 : $00;),(byte0 : $53; byte1 : $67; byte2 : $00;),(byte0 : $C3; byte1 : $33; byte2 : $02;),(byte0 : $49; byte1 : $3B; byte2 : $00;),(byte0 : $FA; byte1 : $67; byte2 : $00;),(byte0 : $85; byte1 : $67; byte2 : $00;),(byte0 : $52; byte1 : $68; byte2 : $00;),(byte0 : $6D; byte1 : $34; byte2 : $02;),(byte0 : $8E; byte1 : $68; byte2 : $00;),
      (byte0 : $1F; byte1 : $68; byte2 : $00;),(byte0 : $14; byte1 : $69; byte2 : $00;),(byte0 : $42; byte1 : $69; byte2 : $00;),(byte0 : $A3; byte1 : $69; byte2 : $00;),(byte0 : $EA; byte1 : $69; byte2 : $00;),(byte0 : $A8; byte1 : $6A; byte2 : $00;),(byte0 : $A3; byte1 : $36; byte2 : $02;),(byte0 : $DB; byte1 : $6A; byte2 : $00;),(byte0 : $18; byte1 : $3C; byte2 : $00;),(byte0 : $21; byte1 : $6B; byte2 : $00;),(byte0 : $A7; byte1 : $38; byte2 : $02;),(byte0 : $54; byte1 : $6B; byte2 : $00;),(byte0 : $4E; byte1 : $3C; byte2 : $00;),(byte0 : $72; byte1 : $6B; byte2 : $00;),(byte0 : $9F; byte1 : $6B; byte2 : $00;),(byte0 : $BB; byte1 : $6B; byte2 : $00;),
      (byte0 : $8D; byte1 : $3A; byte2 : $02;),(byte0 : $0B; byte1 : $1D; byte2 : $02;),(byte0 : $FA; byte1 : $3A; byte2 : $02;),(byte0 : $4E; byte1 : $6C; byte2 : $00;),(byte0 : $BC; byte1 : $3C; byte2 : $02;),(byte0 : $BF; byte1 : $6C; byte2 : $00;),(byte0 : $CD; byte1 : $6C; byte2 : $00;),(byte0 : $67; byte1 : $6C; byte2 : $00;),(byte0 : $16; byte1 : $6D; byte2 : $00;),(byte0 : $3E; byte1 : $6D; byte2 : $00;),(byte0 : $69; byte1 : $6D; byte2 : $00;),(byte0 : $78; byte1 : $6D; byte2 : $00;),(byte0 : $85; byte1 : $6D; byte2 : $00;),(byte0 : $1E; byte1 : $3D; byte2 : $02;),(byte0 : $34; byte1 : $6D; byte2 : $00;),(byte0 : $2F; byte1 : $6E; byte2 : $00;),
      (byte0 : $6E; byte1 : $6E; byte2 : $00;),(byte0 : $33; byte1 : $3D; byte2 : $00;),(byte0 : $C7; byte1 : $6E; byte2 : $00;),(byte0 : $D1; byte1 : $3E; byte2 : $02;),(byte0 : $F9; byte1 : $6D; byte2 : $00;),(byte0 : $6E; byte1 : $6F; byte2 : $00;),(byte0 : $5E; byte1 : $3F; byte2 : $02;),(byte0 : $8E; byte1 : $3F; byte2 : $02;),(byte0 : $C6; byte1 : $6F; byte2 : $00;),(byte0 : $39; byte1 : $70; byte2 : $00;),(byte0 : $1B; byte1 : $70; byte2 : $00;),(byte0 : $96; byte1 : $3D; byte2 : $00;),(byte0 : $4A; byte1 : $70; byte2 : $00;),(byte0 : $7D; byte1 : $70; byte2 : $00;),(byte0 : $77; byte1 : $70; byte2 : $00;),(byte0 : $AD; byte1 : $70; byte2 : $00;),
      (byte0 : $25; byte1 : $05; byte2 : $02;),(byte0 : $45; byte1 : $71; byte2 : $00;),(byte0 : $63; byte1 : $42; byte2 : $02;),(byte0 : $9C; byte1 : $71; byte2 : $00;),(byte0 : $AB; byte1 : $43; byte2 : $02;),(byte0 : $28; byte1 : $72; byte2 : $00;),(byte0 : $50; byte1 : $72; byte2 : $00;),(byte0 : $08; byte1 : $46; byte2 : $02;),(byte0 : $80; byte1 : $72; byte2 : $00;),(byte0 : $95; byte1 : $72; byte2 : $00;),(byte0 : $35; byte1 : $47; byte2 : $02;),(byte0 : $14; byte1 : $48; byte2 : $02;),(byte0 : $7A; byte1 : $73; byte2 : $00;),(byte0 : $8B; byte1 : $73; byte2 : $00;),(byte0 : $AC; byte1 : $3E; byte2 : $00;),(byte0 : $A5; byte1 : $73; byte2 : $00;),
      (byte0 : $B8; byte1 : $3E; byte2 : $00;),(byte0 : $47; byte1 : $74; byte2 : $00;),(byte0 : $5C; byte1 : $74; byte2 : $00;),(byte0 : $85; byte1 : $74; byte2 : $00;),(byte0 : $CA; byte1 : $74; byte2 : $00;),(byte0 : $1B; byte1 : $3F; byte2 : $00;),(byte0 : $24; byte1 : $75; byte2 : $00;),(byte0 : $36; byte1 : $4C; byte2 : $02;),(byte0 : $3E; byte1 : $75; byte2 : $00;),(byte0 : $92; byte1 : $4C; byte2 : $02;),(byte0 : $9F; byte1 : $21; byte2 : $02;),(byte0 : $10; byte1 : $76; byte2 : $00;),(byte0 : $A1; byte1 : $4F; byte2 : $02;),(byte0 : $B8; byte1 : $4F; byte2 : $02;),(byte0 : $44; byte1 : $50; byte2 : $02;),(byte0 : $FC; byte1 : $3F; byte2 : $00;),
      (byte0 : $08; byte1 : $40; byte2 : $00;),(byte0 : $F3; byte1 : $50; byte2 : $02;),(byte0 : $F2; byte1 : $50; byte2 : $02;),(byte0 : $19; byte1 : $51; byte2 : $02;),(byte0 : $33; byte1 : $51; byte2 : $02;),(byte0 : $1E; byte1 : $77; byte2 : $00;),(byte0 : $1F; byte1 : $77; byte2 : $00;),(byte0 : $8B; byte1 : $77; byte2 : $00;),(byte0 : $46; byte1 : $40; byte2 : $00;),(byte0 : $96; byte1 : $40; byte2 : $00;),(byte0 : $1D; byte1 : $54; byte2 : $02;),(byte0 : $4E; byte1 : $78; byte2 : $00;),(byte0 : $E3; byte1 : $40; byte2 : $00;),(byte0 : $26; byte1 : $56; byte2 : $02;),(byte0 : $9A; byte1 : $56; byte2 : $02;),(byte0 : $C5; byte1 : $56; byte2 : $02;),
      (byte0 : $EB; byte1 : $79; byte2 : $00;),(byte0 : $2F; byte1 : $41; byte2 : $00;),(byte0 : $4A; byte1 : $7A; byte2 : $00;),(byte0 : $4F; byte1 : $7A; byte2 : $00;),(byte0 : $7C; byte1 : $59; byte2 : $02;),(byte0 : $A7; byte1 : $5A; byte2 : $02;),(byte0 : $EE; byte1 : $7A; byte2 : $00;),(byte0 : $02; byte1 : $42; byte2 : $00;),(byte0 : $AB; byte1 : $5B; byte2 : $02;),(byte0 : $C6; byte1 : $7B; byte2 : $00;),(byte0 : $C9; byte1 : $7B; byte2 : $00;),(byte0 : $27; byte1 : $42; byte2 : $00;),(byte0 : $80; byte1 : $5C; byte2 : $02;),(byte0 : $D2; byte1 : $7C; byte2 : $00;),(byte0 : $A0; byte1 : $42; byte2 : $00;),(byte0 : $E8; byte1 : $7C; byte2 : $00;),
      (byte0 : $E3; byte1 : $7C; byte2 : $00;),(byte0 : $00; byte1 : $7D; byte2 : $00;),(byte0 : $86; byte1 : $5F; byte2 : $02;),(byte0 : $63; byte1 : $7D; byte2 : $00;),(byte0 : $01; byte1 : $43; byte2 : $00;),(byte0 : $C7; byte1 : $7D; byte2 : $00;),(byte0 : $02; byte1 : $7E; byte2 : $00;),(byte0 : $45; byte1 : $7E; byte2 : $00;),(byte0 : $34; byte1 : $43; byte2 : $00;),(byte0 : $28; byte1 : $62; byte2 : $02;),(byte0 : $47; byte1 : $62; byte2 : $02;),(byte0 : $59; byte1 : $43; byte2 : $00;),(byte0 : $D9; byte1 : $62; byte2 : $02;),(byte0 : $7A; byte1 : $7F; byte2 : $00;),(byte0 : $3E; byte1 : $63; byte2 : $02;),(byte0 : $95; byte1 : $7F; byte2 : $00;),
      (byte0 : $FA; byte1 : $7F; byte2 : $00;),(byte0 : $DA; byte1 : $64; byte2 : $02;),(byte0 : $23; byte1 : $65; byte2 : $02;),(byte0 : $60; byte1 : $80; byte2 : $00;),(byte0 : $A8; byte1 : $65; byte2 : $02;),(byte0 : $70; byte1 : $80; byte2 : $00;),(byte0 : $5F; byte1 : $33; byte2 : $02;),(byte0 : $D5; byte1 : $43; byte2 : $00;),(byte0 : $B2; byte1 : $80; byte2 : $00;),(byte0 : $03; byte1 : $81; byte2 : $00;),(byte0 : $0B; byte1 : $44; byte2 : $00;),(byte0 : $3E; byte1 : $81; byte2 : $00;),(byte0 : $B5; byte1 : $5A; byte2 : $00;),(byte0 : $A7; byte1 : $67; byte2 : $02;),(byte0 : $B5; byte1 : $67; byte2 : $02;),(byte0 : $93; byte1 : $33; byte2 : $02;),
      (byte0 : $9C; byte1 : $33; byte2 : $02;),(byte0 : $04; byte1 : $82; byte2 : $00;),(byte0 : $9E; byte1 : $8F; byte2 : $00;),(byte0 : $6B; byte1 : $44; byte2 : $00;),(byte0 : $91; byte1 : $82; byte2 : $00;),(byte0 : $8B; byte1 : $82; byte2 : $00;),(byte0 : $9D; byte1 : $82; byte2 : $00;),(byte0 : $B3; byte1 : $52; byte2 : $00;),(byte0 : $B1; byte1 : $82; byte2 : $00;),(byte0 : $B3; byte1 : $82; byte2 : $00;),(byte0 : $BD; byte1 : $82; byte2 : $00;),(byte0 : $E6; byte1 : $82; byte2 : $00;),(byte0 : $3C; byte1 : $6B; byte2 : $02;),(byte0 : $1D; byte1 : $83; byte2 : $00;),(byte0 : $63; byte1 : $83; byte2 : $00;),(byte0 : $AD; byte1 : $83; byte2 : $00;),
      (byte0 : $23; byte1 : $83; byte2 : $00;),(byte0 : $BD; byte1 : $83; byte2 : $00;),(byte0 : $E7; byte1 : $83; byte2 : $00;),(byte0 : $53; byte1 : $83; byte2 : $00;),(byte0 : $CA; byte1 : $83; byte2 : $00;),(byte0 : $CC; byte1 : $83; byte2 : $00;),(byte0 : $DC; byte1 : $83; byte2 : $00;),(byte0 : $36; byte1 : $6C; byte2 : $02;),(byte0 : $6B; byte1 : $6D; byte2 : $02;),(byte0 : $D5; byte1 : $6C; byte2 : $02;),(byte0 : $2B; byte1 : $45; byte2 : $00;),(byte0 : $F1; byte1 : $84; byte2 : $00;),(byte0 : $F3; byte1 : $84; byte2 : $00;),(byte0 : $16; byte1 : $85; byte2 : $00;),(byte0 : $CA; byte1 : $73; byte2 : $02;),(byte0 : $64; byte1 : $85; byte2 : $00;),
      (byte0 : $2C; byte1 : $6F; byte2 : $02;),(byte0 : $5D; byte1 : $45; byte2 : $00;),(byte0 : $61; byte1 : $45; byte2 : $00;),(byte0 : $B1; byte1 : $6F; byte2 : $02;),(byte0 : $D2; byte1 : $70; byte2 : $02;),(byte0 : $6B; byte1 : $45; byte2 : $00;),(byte0 : $50; byte1 : $86; byte2 : $00;),(byte0 : $67; byte1 : $86; byte2 : $00;),(byte0 : $69; byte1 : $86; byte2 : $00;),(byte0 : $A9; byte1 : $86; byte2 : $00;),(byte0 : $88; byte1 : $86; byte2 : $00;),(byte0 : $0E; byte1 : $87; byte2 : $00;),(byte0 : $E2; byte1 : $86; byte2 : $00;),(byte0 : $28; byte1 : $87; byte2 : $00;),(byte0 : $6B; byte1 : $87; byte2 : $00;),(byte0 : $86; byte1 : $87; byte2 : $00;),
      (byte0 : $D7; byte1 : $45; byte2 : $00;),(byte0 : $E1; byte1 : $87; byte2 : $00;),(byte0 : $01; byte1 : $88; byte2 : $00;),(byte0 : $F9; byte1 : $45; byte2 : $00;),(byte0 : $60; byte1 : $88; byte2 : $00;),(byte0 : $63; byte1 : $88; byte2 : $00;),(byte0 : $67; byte1 : $76; byte2 : $02;),(byte0 : $D7; byte1 : $88; byte2 : $00;),(byte0 : $DE; byte1 : $88; byte2 : $00;),(byte0 : $35; byte1 : $46; byte2 : $00;),(byte0 : $FA; byte1 : $88; byte2 : $00;),(byte0 : $BB; byte1 : $34; byte2 : $00;),(byte0 : $AE; byte1 : $78; byte2 : $02;),(byte0 : $66; byte1 : $79; byte2 : $02;),(byte0 : $BE; byte1 : $46; byte2 : $00;),(byte0 : $C7; byte1 : $46; byte2 : $00;),
      (byte0 : $A0; byte1 : $8A; byte2 : $00;),(byte0 : $55; byte1 : $8C; byte2 : $00;),(byte0 : $A8; byte1 : $7C; byte2 : $02;),(byte0 : $AB; byte1 : $8C; byte2 : $00;),(byte0 : $C1; byte1 : $8C; byte2 : $00;),(byte0 : $1B; byte1 : $8D; byte2 : $00;),(byte0 : $77; byte1 : $8D; byte2 : $00;),(byte0 : $2F; byte1 : $7F; byte2 : $02;),(byte0 : $04; byte1 : $08; byte2 : $02;),(byte0 : $CB; byte1 : $8D; byte2 : $00;),(byte0 : $BC; byte1 : $8D; byte2 : $00;),(byte0 : $F0; byte1 : $8D; byte2 : $00;),(byte0 : $DE; byte1 : $08; byte2 : $02;),(byte0 : $D4; byte1 : $8E; byte2 : $00;),(byte0 : $D2; byte1 : $85; byte2 : $02;),(byte0 : $ED; byte1 : $85; byte2 : $02;),
      (byte0 : $94; byte1 : $90; byte2 : $00;),(byte0 : $F1; byte1 : $90; byte2 : $00;),(byte0 : $11; byte1 : $91; byte2 : $00;),(byte0 : $2E; byte1 : $87; byte2 : $02;),(byte0 : $1B; byte1 : $91; byte2 : $00;),(byte0 : $38; byte1 : $92; byte2 : $00;),(byte0 : $D7; byte1 : $92; byte2 : $00;),(byte0 : $D8; byte1 : $92; byte2 : $00;),(byte0 : $7C; byte1 : $92; byte2 : $00;),(byte0 : $F9; byte1 : $93; byte2 : $00;),(byte0 : $15; byte1 : $94; byte2 : $00;),(byte0 : $FA; byte1 : $8B; byte2 : $02;),(byte0 : $8B; byte1 : $95; byte2 : $00;),(byte0 : $95; byte1 : $49; byte2 : $00;),(byte0 : $B7; byte1 : $95; byte2 : $00;),(byte0 : $77; byte1 : $8D; byte2 : $02;),
      (byte0 : $E6; byte1 : $49; byte2 : $00;),(byte0 : $C3; byte1 : $96; byte2 : $00;),(byte0 : $B2; byte1 : $5D; byte2 : $00;),(byte0 : $23; byte1 : $97; byte2 : $00;),(byte0 : $45; byte1 : $91; byte2 : $02;),(byte0 : $1A; byte1 : $92; byte2 : $02;),(byte0 : $6E; byte1 : $4A; byte2 : $00;),(byte0 : $76; byte1 : $4A; byte2 : $00;),(byte0 : $E0; byte1 : $97; byte2 : $00;),(byte0 : $0A; byte1 : $94; byte2 : $02;),(byte0 : $B2; byte1 : $4A; byte2 : $00;),(byte0 : $96; byte1 : $94; byte2 : $02;),(byte0 : $29; byte1 : $98; byte2 : $00;),(byte0 : $B6; byte1 : $95; byte2 : $02;),(byte0 : $E2; byte1 : $98; byte2 : $00;),(byte0 : $33; byte1 : $4B; byte2 : $00;),
      (byte0 : $29; byte1 : $99; byte2 : $00;),(byte0 : $A7; byte1 : $99; byte2 : $00;),(byte0 : $C2; byte1 : $99; byte2 : $00;),(byte0 : $FE; byte1 : $99; byte2 : $00;),(byte0 : $CE; byte1 : $4B; byte2 : $00;),(byte0 : $30; byte1 : $9B; byte2 : $02;),(byte0 : $40; byte1 : $9C; byte2 : $00;),(byte0 : $FD; byte1 : $9C; byte2 : $00;),(byte0 : $CE; byte1 : $4C; byte2 : $00;),(byte0 : $ED; byte1 : $4C; byte2 : $00;),(byte0 : $67; byte1 : $9D; byte2 : $00;),(byte0 : $CE; byte1 : $A0; byte2 : $02;),(byte0 : $F8; byte1 : $4C; byte2 : $00;),(byte0 : $05; byte1 : $A1; byte2 : $02;),(byte0 : $0E; byte1 : $A2; byte2 : $02;),(byte0 : $91; byte1 : $A2; byte2 : $02;),
      (byte0 : $BB; byte1 : $9E; byte2 : $00;),(byte0 : $56; byte1 : $4D; byte2 : $00;),(byte0 : $F9; byte1 : $9E; byte2 : $00;),(byte0 : $FE; byte1 : $9E; byte2 : $00;),(byte0 : $05; byte1 : $9F; byte2 : $00;),(byte0 : $0F; byte1 : $9F; byte2 : $00;),(byte0 : $16; byte1 : $9F; byte2 : $00;),(byte0 : $3B; byte1 : $9F; byte2 : $00;),(byte0 : $00; byte1 : $A6; byte2 : $02;)
    ); // CodePoints END
  );

